/* directionTable.h
 */
#ifndef BOARD_TABLE_H
#define BOARD_TABLE_H

#include "osl/direction.h"
#include "osl/misc/carray.h"
#include "osl/offset32.h"

namespace osl
{
  class BoardTable
  {
    CArray<Direction,Offset32::SIZE> directions;
    CArray<Offset, Offset32::SIZE> short_offsets;
    CArray<Offset, Offset32::SIZE> short_offsets_not_knight;
#ifndef MINIMAL
    CArray<int,Offset32Wide::SIZE> space_counts;
#endif
  public:
    static const CArray<Offset, DIRECTION_SIZE> offsets;
    static const CArray<int, DIRECTION_SIZE> dxs;
    static const CArray<int, DIRECTION_SIZE> dys;
  private:
    template<Direction Dir>
    void setDirections();
    template<Direction Dir>
    void setKnightDirections();
    void init();
  public:
    /**
     * ˤȤäƤoffset֤
     */
    const Offset getOffsetForBlack(Direction dir) const{
      return offsets[static_cast<int>(dir)];
    }
    int getDxForBlack(Direction dir) const{
      return dxs[static_cast<int>(dir)];
    }
    int getDyForBlack(Direction dir) const{
      return dys[static_cast<int>(dir)];
    }
    template<Player P>
    const Offset getOffset(Direction dir) const{
      return getOffsetForBlack(dir)*playerToMul(P);
    }
    const Offset getOffset(Player pl,Direction dir) const{
      if (pl==BLACK)
	return getOffset<BLACK>(dir);
      else
	return getOffset<WHITE>(dir);
    }

    /**
     * next position from pos for player P.
     *  isOnBoard Ȥϸ¤ʤ
     */
    const Position nextPosition(Player P, Position pos, Direction dr) const
    {
      assert(pos.isOnBoard());
      const Offset offset = getOffset(P, dr);
      return pos + offset;
    }

    BoardTable();
    /** @param P ɤPlayerˤȤäƤ */
    template <Player P>
    Direction getLongDirection(Offset32 offset32) const
    {
      assert(offset32.isValid());
      const Offset32 blackOffset32 = offset32.blackOffset32<P>();
      Direction ret=directions[blackOffset32.index()];
      assert(isValid(ret));
      return ret;
    }
    /** @param P ɤPlayerˤȤäƤ */
    template <Player P>
    Direction getLongDirection(Position from, Position to) const
    {
      return getLongDirection<P>(Offset32(to,from));
    }
#ifndef MINIMAL
    /**
     * fromtoĹİ֤ˤ֤Υޥο
     * פƤ0
     * Ȥʤ0
     * طʤ-1
     */
    int spaceCounts(Position from,Position to) const
    {
      Offset32Wide offset32(from,to);
      return space_counts[offset32.index()];
    }
#endif
    /**
     * LongβǽΤoffsetξ, ȿ˻Ȥ offset
     * ShortoffsetξϤ켫Ȥ֤.
     * βǽΤʤ0֤
     */
    const Offset getShortOffset(Offset32 offset32) const{
      assert(offset32.isValid());
      return short_offsets[offset32.index()];
    }
    /**
     * LongβǽΤoffsetξ, ȿ˻Ȥ offset
     * KnightʳShortoffsetξϤ켫Ȥ֤.
     * Knight, βǽΤʤ0֤
     */
    const Offset getShortOffsetNotKnight(Offset32 offset32) const{
      assert(offset32.isValid());
      return short_offsets_not_knight[offset32.index()];
    }
    /**
     * p0, p1δ֤t뤫ɤ.
     * (t,p0), (t,p1)Τɤ餫8ǤˤΤ߸ƤӽФ
     */
    bool isBetween(Position t,Position p0,Position p1) const
    {
      Offset offset0=getShortOffsetNotKnight(Offset32(p0,t));
      Offset offset1=getShortOffsetNotKnight(Offset32(p1,t));
      assert(!offset0.zero() || !offset1.zero());
      return (-offset0)==offset1;
    }
  };

  extern const BoardTable Board_Table;
} // namespace osl


#endif /* BOARD_TABLE_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
