/* dfpnRecord.h
 */
#ifndef OSL_DFPNRECORD_H
#define OSL_DFPNRECORD_H

#include "osl/checkmate/proofDisproof.h"
#include "osl/move.h"
#include "osl/pieceStand.h"
namespace osl
{
  namespace checkmate
  {
    struct DfpnRecordBase
    {
      ProofDisproof proof_disproof;
      uint64_t solved;		// ֤Ū˷̤Ƚꥹ loop Ͻ
      Move best_move;
      PieceStand proof_pieces;
      unsigned int node_count, tried_oracle;
      Move last_move;		// ʰ: ̵¥롼ײ
      /** solved max */
      PieceStand proof_pieces_candidate;
      unsigned int min_pdp;	// solved min
      Position last_to;
      enum ProofPiecesType { UNSET=0, PROOF, DISPROOF };
      ProofPiecesType proof_pieces_set;
      char need_full_width, false_branch;

      DfpnRecordBase() 
	: solved(0), node_count(0), tried_oracle(0), min_pdp(ProofDisproof::PROOF_MAX),
	  proof_pieces_set(UNSET), need_full_width(false), false_branch(false)
      {
      }
    };

    struct DfpnRecord : public DfpnRecordBase
    {
      CArray<PieceStand,2> stands;

      DfpnRecord() {}
      DfpnRecord(PieceStand black, PieceStand white) { stands[BLACK] = black; stands[WHITE] = white; }

      void setFrom(const DfpnRecordBase& src) 
      {
	static_cast<DfpnRecordBase*>(this)->operator=(src);
      }
      unsigned int proof() const { return proof_disproof.proof(); }
      unsigned int disproof() const { return proof_disproof.disproof(); }
      void setProofPieces(PieceStand a) 
      {
	assert(proof_pieces_set == UNSET);
	assert((stands[BLACK] == PieceStand() && stands[WHITE] == PieceStand())
	       || stands[BLACK].isSuperiorOrEqualTo(a)
	       || stands[WHITE].isSuperiorOrEqualTo(a));
	proof_pieces_set = PROOF;
	proof_pieces = a;
      }
      void setDisproofPieces(PieceStand a) 
      {
	assert(proof_pieces_set == UNSET);
	assert((stands[BLACK] == PieceStand() && stands[WHITE] == PieceStand())
	       || stands[BLACK].isSuperiorOrEqualTo(a)
	       || stands[WHITE].isSuperiorOrEqualTo(a));
	proof_pieces_set = DISPROOF;
	proof_pieces = a;
      }
      const PieceStand proofPieces() const 
      {
	assert(proof_pieces_set == PROOF);
	return proof_pieces;
      }
      const PieceStand disproofPieces() const 
      {
	assert(proof_pieces_set == DISPROOF);
	return proof_pieces;
      }
    };
  }
}



#endif /* OSL_DFPNRECORD_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
