/* majorPiece.h
 */

#ifndef EVAL_EXPERIMENTAL_MAJORPIECE_H
#define EVAL_EXPERIMENTAL_MAJORPIECE_H

#include "osl/eval/experimental/weights.h"
#include "osl/eval/experimental/midgame.h"
#include "osl/misc/carray.h"
#include "osl/misc/carray2d.h"
#include "osl/state/numEffectState.h"
#include "osl/container/tripleInt.h"

namespace osl
{
  namespace eval
  {
    namespace experimental
    {
      template <bool Opening, Ptype MajorBasic>
      class MajorY
      {
      private:
	static CArray<int, 18> table;
	static int index(Piece piece)
	{
	  return ((piece.owner() == BLACK) ? (piece.position().y() - 1) :
		  (9 - piece.position().y())) + (piece.isPromoted() ? 9 : 0);
	}
      public:
	enum { DIM = 18 };
	static void setUp(const Weights &weights);
	static int eval(const NumEffectState &state)
	{
	  int value = 0;
	  for (int i = PtypeTraits<MajorBasic>::indexMin;
	       i < PtypeTraits<MajorBasic>::indexLimit;
	       ++i)
	  {
	    const Piece piece = state.getPieceOf(i);
	    if (piece.isOnBoard())
	      {
		if (piece.owner() == BLACK)
		  value += table[index(piece)];
	        else
		  value -= table[index(piece)];
	      }
	  }
	  return value;
	}
      };

      class RookYOpening : public MajorY<true, ROOK>
      {
      };
      class RookYEnding : public MajorY<false, ROOK>
      {
      };
      class BishopYOpening : public MajorY<true, BISHOP>
      {
      };
      class BishopYEnding : public MajorY<false, BISHOP>
      {
      };

      template <bool Opening>
      class RookPawn
      {
      public:
	enum { DIM = 1 };
	static void setUp(const Weights &weights);
	static int eval(const NumEffectState &state);
      private:
	static int weight;
      };
      class RookPawnOpening : public RookPawn<true>
      {
      };
      class RookPawnEnding : public RookPawn<false>
      {
      };

      class RookPawnY
      {
	friend class RookPawnYX;
      public:
	enum { ONE_DIM = 180, DIM = ONE_DIM * EvalStages };
	static void setUp(const Weights &weights);
	static MultiInt eval(const NumEffectState &state,
			      const CArray2d<int, 2, 9> &pawns);
      private:
	static int index(const Piece rook, const int pawn_y)
	{
	  const int rook_y =
	    (rook.owner() == BLACK ? rook.position().y() : 10 - rook.position().y());
	  return (rook_y - 1) * 10 + pawn_y + (rook.isPromoted() ? 90 : 0);
	}
	static int indexY(const Position king,
			  const Piece rook, int pawn_y)
	{
	  const int x_diff = std::abs(rook.position().x() - king.x());
	  const int rook_y =
	    (rook.owner() == BLACK ? rook.position().y() : 10 - rook.position().y());
	  return x_diff * 10 * 9 + (rook_y - 1) * 10 + pawn_y + (rook.isPromoted() ? 810 : 0);
	}
	static CArray<MultiInt, 180> table;
	static CArray<MultiInt, 1620> y_attack_table;
	static CArray<MultiInt, 1620> y_defense_table;
      };

      class RookPawnYX
      {
      public:
	enum { ONE_DIM = 1620, DIM = ONE_DIM * 2*EvalStages };
	static void setUp(const Weights &weights);
      };

      class AllMajor
      {
      public:
	enum { DIM = 1 };
	static void setUp(const Weights &weights,int stage);
	static MultiInt eval(int black_major_count)
	{
	  if (black_major_count == 4)
	    return weight;
	  else if (black_major_count == 0)
	    return -weight;

	  return MultiInt();
	}
      private:
	static MultiInt weight;
      };

      template <bool Opening>
      class MajorGoldSilverAttacked
      {
      public:
	enum { DIM = 32 };
	static void setUp(const Weights &weights);
	static int eval(const NumEffectState &state);
	static int index(const NumEffectState &state, Piece piece);
	template <Ptype PTYPE>
	static int evalOne(const NumEffectState &state);
      private:
	static CArray<int, 32> table;
      };

      class MajorGoldSilverAttackedOpening : public MajorGoldSilverAttacked<true>
      {
      };
      class MajorGoldSilverAttackedEnding : public MajorGoldSilverAttacked<false>
      {
      };

      class RookEffectBase
      {
	friend class RookEffectPiece;
      public:
	enum { ONE_DIM = 612, DIM = ONE_DIM * 2 };
	static MultiInt eval(const NumEffectState &state);
      protected:
	template<Player P>
	static MultiInt evalOne(const NumEffectState& state,
				 Position rook,
				 Position myKing,
				 Position opKing,
				 Position up,
				 Position dp,
				 Position rp,
				 Position lp,
				 bool isP);
	/**
	 * (abs_x_diff, y_diff) - ̤Ȥζޥа
	 * horizontal - ֤β
	 * is_promoted - εξ
	 */
	static int index(int abs_x_diff, int y_diff, bool horizontal, bool is_promoted)
	{
	  return y_diff + 8 + abs_x_diff * 17 + (horizontal ? 153 : 0) +
	    (is_promoted ? 306 : 0);
	}
	/**
	 * (ε)Τ
	 * (abs_x_diff, y_diff) - ˤ̤а
	 *                        abs_x_diff
	 * ptypeO - ptypeO, 򤫤ξȿž
	 * horizontal - ֤β
	 * is_promoted - εξ
	 */
	static int index0(int abs_x_diff,int y_diff,
			  PtypeO ptypeO,
			  bool horizontal, bool promoted){
	  return y_diff+8+abs_x_diff*17+(ptypeO - PTYPEO_MIN) * 17 * 9 +
	    (horizontal ? 4896 : 0) + (promoted ? 9792 : 0);
	}
	/**
	 * (ε)Τindex
	 * {attack,defense}_{u,r,l,d} ؤΥ˻Ȥ
	 * from - ΰ
	 * king - ̤ΰ
	 * ptypeO - μࡤ򤫤ξȿž
	 *          (BLACK,PTYPE_EDGE)⤢
	 * isP - εξ
	 */
	static int index1(Position king,Position from,PtypeO ptypeO,bool isP)
	{
	  int y_diff=from.y()-king.y();
	  int x_diff=from.x()-king.x();
	  return index1(x_diff,y_diff,ptypeO,isP);
	}
	/**
	 * (ε)Τindex
	 * {attack,defense}_{u,r,l,d} ؤΥ˻Ȥ
	 * (x_diff, y_diff) - ̤˸а
	 * ptypeO - μࡤ򤫤ξȿž
	 *          (BLACK,PTYPE_EDGE)⤢
	 * isP - εξ
	 */
	static int index1(int x_diff,int y_diff,PtypeO ptypeO,bool isP){
	  assert(-9 <= y_diff && y_diff <= 9);
	  assert(-9 <= x_diff && x_diff <= 9);
	  assert(getPtype((PtypeO)ptypeO)!=PTYPE_EMPTY);
	  int index=(ptypeO-PTYPEO_MIN)+32*((y_diff+9)+19*(x_diff+9+19*(isP ? 1 : 0)));
	  assert(0<=index && index<32*19*19*2);
	  return index;
	}
	/**
	 * (ε)϶ޥǤʤΤǤʬơ֥
	 * {attack,defense}_nospace ؤΥ
	 * king - ̤ΰ
	 * from - (ε)ΰ
	 * isP - εξ
	 */
        static int index2(Position king,Position from,bool isP)
	{
	  int y_diff=from.y()-king.y();
	  int x_diff=from.x()-king.x();
	  return index2(x_diff,y_diff,isP);
	}
	/**
	 * (ε)϶ޥǤʤΤǤʬơ֥
	 * {attack,defense}_nospace ؤΥ
	 * (x_diff, y_diff)  - ̤ˤƤߤ(ε)а
	 * isP - εξ
	 */
	static int index2(int x_diff,int y_diff,bool isP){
	  assert(-9 <= y_diff && y_diff <= 9);
	  assert(-9 <= x_diff && x_diff <= 9);
	  int index=(y_diff+9)+19*(x_diff+9+19*(isP ? 1 : 0));
	  assert(0<=index && index<19*19*2);
	  return index;
	}
	static CArray<MultiInt, 612> attack_table;
	static CArray<MultiInt, 612> defense_table;
	static CArray<MultiInt, 32> piece_table;
	static CArray<MultiInt, 23104> attack_u;
	static CArray<MultiInt, 23104> attack_d;
	static CArray<MultiInt, 23104> attack_l;
	static CArray<MultiInt, 23104> attack_r;
	static CArray<MultiInt, 23104> defense_u;
	static CArray<MultiInt, 23104> defense_d;
	static CArray<MultiInt, 23104> defense_l;
	static CArray<MultiInt, 23104> defense_r;
	static CArray<MultiInt, 722> attack_nospace;
	static CArray<MultiInt, 722> defense_nospace;
      };
      class RookEffect : public RookEffectBase
      {
      public:
	static void setUp(const Weights &weights,int stage);
      };

      class RookEffectPiece
      {
      public:
	enum { DIM = 32 * EvalStages };
	static void setUp(const Weights &weights);
      };
      class RookEffectPieceKingRelative : RookEffectBase
      {
      public:
	enum { ONE_DIM = 19584, DIM = ONE_DIM * 2*EvalStages };
	static void setUp(const Weights & weights);
      };

      class RookPromoteDefense : public RookEffectBase
      {
      public:
	enum { ONE_DIM = 256, DIM = ONE_DIM * EvalStages };
	static void setUp(const Weights &weights);
	static MultiInt eval(const NumEffectState &state);
      private:
	static CArray<MultiInt, 256> promote_defense_table;
      };

      class BishopEffectBase
      {
	friend class BishopEffectPiece;
      public:
	enum { ONE_DIM = 612, DIM = ONE_DIM * 2 };
	static MultiInt eval(const NumEffectState &state);
      protected:
	template<Player P>
	static MultiInt evalOne(const NumEffectState& state,
				 Position bishop,
				 Position myKing,
				 Position opKing,
				 Position ulp,
				 Position urp,
				 Position dlp,
				 Position drp,
				 bool isP);
	static int index(int x_diff, int y_diff, bool ur, bool promoted)
	{
	  if (x_diff<0)
	    ur = !ur;
	  return y_diff + 8 + std::abs(x_diff) * 17 + (ur ? 153 : 0) + (promoted ? 306 : 0);
	}
	static int index0(int x_diff, int y_diff,PtypeO ptypeO,bool ur, bool promoted)
	{
	  if (x_diff>0)
	    ur = !ur;
	  return -y_diff + 8 + std::abs(x_diff) * 17 + (ptypeO - PTYPEO_MIN) * 17 * 9 +
	    (ur ? 4896 : 0) + (promoted ? 9792 : 0);
	}
	/**
	 * γ()Τindex
	 * {attack,defense}_{ul,ur,dl,dr} ؤΥ˻Ȥ
	 * from - ΰ
	 * king - ̤ΰ
	 * ptypeO - μࡤ򤫤ξȿž
	 *          (BLACK,PTYPE_EDGE)⤢
	 * isP - Ϥξ
	 */
	static int index1(Position king,Position from,PtypeO ptypeO,bool isP)
	{
	  int y_diff=from.y()-king.y();
	  int x_diff=from.x()-king.x();
	  return index1(x_diff,y_diff,ptypeO,isP);
	}
	/**
	 * γ()Τindex
	 * {attack,defense}_{ul,ur,dl,dr} ؤΥ˻Ȥ
	 * (x_diff, y_diff) - ̤˸а
	 * ptypeO - μࡤ򤫤ξȿž
	 *          (BLACK,PTYPE_EDGE)⤢
	 * isP - Ϥξ
	 */
	static int index1(int x_diff,int y_diff,PtypeO ptypeO,bool isP){
	  assert(-9 <= y_diff && y_diff <= 9);
	  assert(-9 <= x_diff && x_diff <= 9);
	  assert(getPtype((PtypeO)ptypeO)!=PTYPE_EMPTY);
	  int index=(ptypeO-PTYPEO_MIN)+32*((y_diff+9)+19*(x_diff+9+19*(isP ? 1 : 0)));
	  assert(0<=index && index<32*19*19*2);
	  return index;
	}
	/**
	 * γ()϶ޥǤʤΤǤʬơ֥
	 * {attack,defense}_nospace ؤΥ
	 * king - ̤ΰ
	 * from - ()ΰ
	 * isP - Ϥξ
	 */
        static int index2(Position king,Position from,bool isP)
	{
	  int y_diff=from.y()-king.y();
	  int x_diff=from.x()-king.x();
	  return index2(x_diff,y_diff,isP);
	}
	/**
	 * γ()϶ޥǤʤΤǤʬơ֥
	 * {attack,defense}_nospace ؤΥ
	 * (x_diff, y_diff)  - ̤ˤƤߤ()а
	 * isP - Ϥξ
	 */
	static int index2(int x_diff,int y_diff,bool isP){
	  assert(-9 <= y_diff && y_diff <= 9);
	  assert(-9 <= x_diff && x_diff <= 9);
	  int index=(y_diff+9)+19*(x_diff+9+19*(isP ? 1 : 0));
	  assert(0<=index && index<19*19*2);
	  return index;
	}
	static CArray<MultiInt, 612> attack_table;
	static CArray<MultiInt, 612> defense_table;
	static CArray<MultiInt, 32> piece_table;
	static CArray<MultiInt, 23104> attack_ur;
	static CArray<MultiInt, 23104> attack_ul;
	static CArray<MultiInt, 23104> attack_dr;
	static CArray<MultiInt, 23104> attack_dl;
	static CArray<MultiInt, 23104> defense_ur;
	static CArray<MultiInt, 23104> defense_ul;
	static CArray<MultiInt, 23104> defense_dr;
	static CArray<MultiInt, 23104> defense_dl;
	static CArray<MultiInt, 722> attack_nospace;
	static CArray<MultiInt, 722> defense_nospace;
      };
      class BishopEffect : public BishopEffectBase
      {
      public:
	static void setUp(const Weights &weights,int stage);
      };
      class BishopEffectPiece
      {
      public:
	enum { DIM = 32*EvalStages };
	static void setUp(const Weights &weights);
      };

      class BishopEffectPieceKingRelative : BishopEffectBase
      {
      public:
	enum { ONE_DIM = 19584, DIM = ONE_DIM * 2*EvalStages };
	static void setUp(const Weights & weights);
      };

      class BishopHead
      {
	friend class BishopHeadKingRelative;
      public:
	enum { ONE_DIM = 32, DIM = ONE_DIM * EvalStages };
	static void setUp(const Weights &weights);
	static MultiInt eval(const NumEffectState &state);
      private:
	static int indexK(Player player, PtypeO ptypeO, int x_diff, int y_diff)
	{
	  if (player == WHITE)
	    {
	      ptypeO=(PtypeO)(static_cast<int>(ptypeO)^(~15));
	    }
	  if (player == WHITE)
	    {
	      y_diff = -y_diff;
	    }
	  return (ptypeOIndex(ptypeO) * 9 + x_diff) * 17 + y_diff + 8;
	}
	static CArray<MultiInt, 32> table;
	static CArray<MultiInt, 4896> king_table;
      };

      class BishopHeadKingRelative
      {
      public:
	enum { ONE_DIM = 4896, DIM = ONE_DIM * EvalStages };
	static void setUp(const Weights &weights);
      };
      class KingRookBishop
      {
      public:
	enum { ONE_DIM = 374544, DIM = ONE_DIM * EvalStages };
	static void setUp(const Weights &weights);
	template<Player King>
	static MultiInt evalOne(const NumEffectState &state);
	static MultiInt eval(const NumEffectState &state);
      private:
	static CArray<MultiInt, 374544> table;
	template <Player King>
	static int index(const Position king, const Piece rook, const Piece bishop)
	{
	  const int rook_x = std::abs(king.x() - rook.position().x());
	  const int bishop_x = std::abs(king.x() - bishop.position().x());
	  const int rook_y = (King == BLACK ? rook.position().y() - king.y() : king.y() - rook.position().y());
	  const int bishop_y = (King == BLACK ? bishop.position().y() - king.y() : king.y() - bishop.position().y());
	  return bishop_y + 8 + 17 * (bishop_x + 9 * (rook_y + 8 + 17 * (rook_x + 9 * ((bishop.owner() == King ? 1 : 0) + 2 * ((rook.owner() == King ? 1 : 0) + 2 * (2 * (bishop.isPromoted() ? 1 : 0) + (rook.isPromoted() ? 1 : 0)))))));
	}
      };

      class NumPiecesBetweenBishopAndKing
      {
	friend class NumPiecesBetweenBishopAndKingSelf;
	friend class NumPiecesBetweenBishopAndKingOpp;
	friend class NumPiecesBetweenBishopAndKingAll;
      public:
	static MultiInt eval(const NumEffectState &state);
      private:
	static void countBetween(const NumEffectState &state,
				 Position king, Piece bishop,
				 int &self_count, int &opp_count,
				 int &total_count);
	static CArray<MultiInt, 9> self_table;
	static CArray<MultiInt, 9> opp_table;
	static CArray<MultiInt, 9> all_table;
      };
      class NumPiecesBetweenBishopAndKingSelf
      {
      public:
	enum { ONE_DIM = 9, DIM = ONE_DIM * EvalStages };
	static void setUp(const Weights &weights);
      };
      class NumPiecesBetweenBishopAndKingOpp
      {
      public:
	enum { ONE_DIM = 9, DIM = ONE_DIM * EvalStages };
	static void setUp(const Weights &weights);
      };
      class NumPiecesBetweenBishopAndKingAll
      {
      public:
	enum { ONE_DIM = 9, DIM = ONE_DIM * EvalStages };
	static void setUp(const Weights &weights);
      };
    }
  }
}

#endif // EVAL_EXPERIMENTAL_MAJORPIECE_H
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
