/* mobility.h
 */

#ifndef EVAL_EXPERIMENTAL_MOBILITY_H
#define EVAL_EXPERIMENTAL_MOBILITY_H

#include "osl/eval/experimental/weights.h"
#include "osl/eval/experimental/midgame.h"
#include "osl/ptype.h"
#include "osl/misc/carray.h"
#include "osl/state/numEffectState.h"
#include "osl/container/tripleInt.h"

namespace osl
{
  namespace eval
  {
    namespace experimental
    {
      struct RookMobilityAll
      {
	template<int Sign>
	static void adjust(bool promoted, int vertical, int horizontal, MultiInt& value);
	static void eval(const NumEffectState&, MultiInt& out);
      };
      class RookMobility
      {
	static CArray<MultiInt, 18> rook_vertical_table;
	static CArray<MultiInt, 18> rook_horizontal_table;
	friend class RookMobilityAll;
      public:
	enum { DIM = 36 };
	RookMobility() { };
	static void setUp(const Weights &weights,int stage);
      };

      struct BishopMobilityAll
      {
	template<int Sign>
	static void adjust(bool promoted, int index, MultiInt& value);
	static void eval(const NumEffectState&, MultiInt& out);
      };
      class BishopMobility
      {
	friend class BishopMobilityAll;
	static CArray<MultiInt, 36> bishop_table;
      public:
	enum { DIM = 36 };
	BishopMobility() { };
	static void setUp(const Weights &weights,int stage);
      };

      struct LanceMobilityAll
      {
	template<int Sign>
	static void adjust(int index, MultiInt& value);
	static void eval(const NumEffectState&, MultiInt& out);
      };
      class LanceMobility
      {
	static CArray<MultiInt, 9> lance_table;
	friend class LanceMobilityAll;
      public:
	enum { DIM = 9 };
	LanceMobility() { };
	static void setUp(const Weights &weights,int stage);
      };
    }
  }
}
#endif // EVAL_EXPERIMENTAL_MOBILITY_H
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
