#ifndef GAME_PLAYING_SEARCHPLAYER_H
#define GAME_PLAYING_SEARCHPLAYER_H

#include "osl/game_playing/computerPlayer.h"
#include "osl/search/searchTimer.h"
#include "osl/misc/milliSeconds.h"
#include "osl/container/moveVector.h"
#include <boost/scoped_ptr.hpp>
#include <boost/shared_ptr.hpp>

namespace osl
{
  namespace misc
  {
    class RealTime;
  }
  namespace search
  {
    class CountRecorder;
    class SimpleHashTable;
    class BigramKillerMove;
    class TimeAssigned;
  }
  namespace game_playing
  {
    struct Config;
    bool operator==(const Config& l, const Config& r);
    class PVHistory;
    /**
     * MtdfPlayer  AlphaBetaPlayer ζʬ
     */
    class SearchPlayer : public ComputerPlayer
    {
    public:
      struct CheckmateSearcher;
      struct NtesukiThread;
      struct Config
      {
	int limit;
	int node_limit;
	int table_size;
	int table_record_limit;
	int initial_limit;
	int deepening_step;
	size_t total_checkmate_limit;
	int verbose;
	/** SearchBase::next_iteration_coefficient ꤹ */
	double next_iteration_coefficient;
	/** Фܡʥ/ڥʥƥ. -2ʤ1»Ƥ򤱤 */
	int draw_coef;
	bool save_pv;
	Config();
	friend bool operator==(const Config& l, const Config& r);
      };
    protected:
      Config config;
      boost::scoped_ptr<search::SimpleHashTable> table_ptr;
      boost::shared_ptr<CheckmateSearcher> checkmate_ptr;
      boost::scoped_ptr<search::CountRecorder> recorder_ptr;
      boost::scoped_ptr<search::BigramKillerMove> bigram_ptr;
      volatile bool searching;
      boost::scoped_ptr<search::SearchTimer> searcher;
      /** õ˻ߤ */
      volatile bool plan_stop;
      const MoveVector *root_ignore_moves; // acquaintance
      boost::scoped_ptr<PVHistory> pv_history;
    public:
      SearchPlayer();
      SearchPlayer(const SearchPlayer&);
      ~SearchPlayer();

      /** ;õפξ */
      void setTotalCheckmateLimit(int limit);

      void setDepthLimit(int limit, int initial_limit, int deepening_step);
      void setNodeLimit(int node_limit);
      void setTableLimit(int size, int record_limit);
      void setVerbose(int verbose=1);
      void setDrawCoef(int new_value) { config.draw_coef = new_value; }
      void setNextIterationCoefficient(double new_value);
      void enableSavePV(bool enable=true) { config.save_pv = enable; }

      /** ͭž */
      void resetRecorder(search::CountRecorder *new_recorder);

      void pushMove(Move m);
      void popMove();

      /**
       * other ζɽȼؤ
       */
      void swapTable(SearchPlayer& other);

      const search::SimpleHashTable* table() const { return table_ptr.get(); }
      const search::CountRecorder& recorder() const { return *recorder_ptr; }

      bool stopSearchNow();
      bool canStopSearch();	// ƽФάʤ褦ǰΰ const Ǥʤ
      /**
       * searchWithSecondsForThisMove ƤӽФ
       */
      const MoveWithComment selectBestMove(const GameState&, int, int, int);
      const search::TimeAssigned assignTime(const GameState& state, int limit, int elapsed,
					    int byoyomi) const;
      void saveSearchResult(const GameState&, const MoveWithComment&);
    protected:
      template <class Searcher>
      ComputerPlayer* cloneIt(const Searcher&) const;
      /** @return time consumed in milliseconds */
      const MilliSeconds::Interval setUpTable(const GameState&, int pawn_value);
      template <class Searcher>
      const MoveWithComment search(const GameState&, const search::TimeAssigned&);
      template <class Searcher>
      bool isReasonableMoveBySearch(Searcher&, Move move, int pawn_sacrifice);
      template <class Searcher>
      static int pawnValue();
      template <class Searcher>
      static int pawnValueOfTurn(Player turn);
    public:
      virtual const MoveWithComment searchWithSecondsForThisMove(const GameState&, const search::TimeAssigned&)=0;
      void setRootIgnoreMoves(const MoveVector *rim) { root_ignore_moves = rim; }
      /**
       * state ͭϤʼ򤤤Ĥ
       */
      void pickUpBestMoves(const GameState& state, MoveVector&) const;

      const Config& getConfig() const { return config; }

      static int secondsForThisMove(const GameState& state,
				    int limit, int elapsed, int byoyomi, int verboseness);
      int secondsForThisMove(const GameState& state, int limit, int elapsed, int byoyomi) const;

      void setTimeAssign(const search::TimeAssigned& new_assign);
      const MilliSeconds startTime() const;
    };

  } // namespace game_playing
} // namespace osl

#endif /* GAME_PLAYING_SEARCHPLAYER_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
