/* rzone.h
 */
#ifndef _NTESUKI_RZONE_H
#define _NTESUKI_RZONE_H
#include "osl/state/numEffectState.h"
#include "osl/ntesuki/ntesukiMoveGenerator.h"

#include "osl/ntesuki/ntesukiExceptions.h"
#include "osl/container/moveVector.h"

#include <osl/stl/vector.h>
#include <bitset>
#include <iosfwd>

using namespace osl::state;

namespace osl
{
  namespace ntesuki
  {
    class
    Rzone
    {
      typedef std::bitset<0x100> mask_t;
      mask_t mask;
    public:
      /** state ζ̤ΰ֤ mask 줿 rzone. */
      Rzone (NumEffectState state,
	     Player p)
      {
	const Position pos = state.getKingPosition(p);
	if (!pos.isOnBoard()) return;
	unsigned int index = pos.index();
	ntesuki_assert(index < Position::indexMax());
	mask.set(index);
      }

      Rzone() {}

      Rzone (const Position pos)
      {
	unsigned int index = pos.index();
	ntesuki_assert(index < Position::indexMax());
	mask.set(index);
      }

      /** rzone սǤ set Ƥ뤫Ĵ٤. */
      bool any() const
      {
	return mask.any();
      }

      /** pos  rzone  set Ƥ뤫Ĵ٤. */
      bool test(Position pos) const
      {
	return mask.test(pos.index());
      }

      /** rzone . */
      bool operator==(const Rzone rhs) const
      {
	return mask == rhs.mask;
      }

      /** rzone . */
      Rzone operator+(const Rzone rhs) const
      {
	mask_t m = mask | rhs.mask;
	return Rzone(m);
      }

      /** rzone κ. */
      Rzone operator-(const Rzone rhs) const
      {
	ntesuki_assert((rhs.mask & mask) == rhs.mask);
	mask_t m = (rhs.mask ^ mask) & mask;
	return Rzone(m);
      }

      /** rzone ι, ֤. */
      Rzone update(const Rzone rhs)
      {
	mask_t mask_orig = mask;
	mask |= rhs.mask;

	mask_t mask_diff = (mask_orig ^ mask) & mask;

	return Rzone(mask_diff);
      }
      /** rzone ν. */
      friend std::ostream& operator<<(std::ostream& os,
				      const Rzone& rzone)
      {
	return os << rzone.mask;
      }
    private:
      Rzone(mask_t _mask) : mask(_mask) {}
      
    };
  } //ntesuki
} //osl

#endif /* _NTESUKI_SEACHER_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
