/* effect5x3Table.h
 */
#ifndef _PROGRESS_EFFECT5X3_TABLE_H
#define _PROGRESS_EFFECT5X3_TABLE_H

#include "osl/ptype.h"
#include "osl/ptypeTraits.h"
#include "osl/misc/carray3d.h"
#include "osl/misc/carray2d.h"
#include "osl/misc/carray.h"
#include "osl/offset32.h"

#include <iosfwd>

namespace osl
{
  namespace progress
  {
    /**
     * 5x3ΰؤĹλϤޤȽꡥ
     * offset - 0λ̵Ȥ򼨤
     * minIndex - ΰindex
     * maxIndex - ΰ褫Фindex
     */
    struct LongEffect
    {
      Offset offset;
      unsigned short minIndex;
      unsigned short maxIndex;
    };
    /**
     * ĹϹ⡹4ʤΤǡĹ4CArrayɽ
     */
    typedef CArray<LongEffect,4> LongEffect4;

    /**
     * Effect5x3κʬ׻ǻȤơ֥.
     */
    class Effect5x3Table
    {
    public:
      enum {
	StandPAWN=1,
	StandLANCE=4,
	StandKNIGHT=8,
	StandSILVER=8,
	StandGOLD=8,
	StandBISHOP=12,
	StandROOK=12,
      };
    private:
      CArray<unsigned int,PTYPE_SIZE> onStand;
      CArray2d<unsigned int,PTYPEO_SIZE,Offset32::SIZE> shortEffect;
      CArray2d<LongEffect4,PTYPEO_SIZE,Offset32::SIZE> longEffect;
      CArray2d<LongEffect,8,Offset32::SIZE> blockEffect;
      CArray3d<unsigned int,2,Position::SIZE,5*3> attackEffect;
      CArray3d<unsigned int,2,Position::SIZE,5*3> defenseEffect;
      void setupOnStand();
      void setupShortEffect();
      void setupLongEffect();
      void setupBlockEffect();
      void setupAttackEffect();
      void setupDefenseEffect();
    public:
      Effect5x3Table();
      /**
       * PtypeȤδͿɽơ֥λ.
       * ʲνŤߤǷ׻
       * PAWN 1
       * LANCE 4
       * KNIGHT,SILVER,GOLD 8
       * BISHOP,ROOK 12
       * @param ptype - ptype
       */
      unsigned int piecesOnStand(Ptype ptype) const
      {
	return onStand[ptype];
      }
      /**
       * 5x3ΰؤûοη׻.
       * 8ܤΤ֤
       * @param ptypeO - Ĥ褦ȤptypeO
       * @param o32 - (to - from)Offset32
       */
      unsigned int countShortEffect(PtypeO ptypeO,Offset32 offset32) const
      {
	return shortEffect[ptypeO-PTYPEO_MIN][offset32.index()];
      }
      /**
       * 5x3ΰؤPtypeOꤷĹη׻.
       * 8ܤΤ֤
       * @param ptypeO - Ĥ褦ȤptypeO
       * @param o32 - (to - from)Offset32
       */
      LongEffect4 const& getLongEffect(PtypeO ptypeO,Offset32 offset32) const
      {
	return longEffect[ptypeO-PTYPEO_MIN][offset32.index()];
      }
      /**
       * 5x3ΰؤdirectionꤷĹη׻.
       * @param d - Ĺ
       * @param o32 - (to - from)Offset32
       */
      LongEffect const& getBlockEffect(Direction d,Offset32 offset32) const
      {
	assert(d<8);
	return blockEffect[d][offset32.index()];
      }
      unsigned int getAttackEffect(Player pl,Position pos,int x,int y) const
      {
	assert(pos.isOnBoard() && 0<=x && x<5 && 0<= y && y<3);
	return attackEffect[pl][pos.index()][x*3+y];
      }
      unsigned int getDefenseEffect(Player pl,Position pos,int x,int y) const
      {
	assert(pos.isOnBoard() && 0<=x && x<5 && 0<= y && y<3);
	return defenseEffect[pl][pos.index()][x*3+y];
      }
    };
    extern const Effect5x3Table Effect5x3_Table;
    std::ostream& operator<<(std::ostream& os,LongEffect const& longEffect);
  }
}
#endif /* _PROGRESS_EFFECT5X3_TABLE_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
