/* repetitionCounter.h
 */
#ifndef _RepetitionCounter_H
#define _RepetitionCounter_H

#include "osl/move.h"
#include "osl/state/hashEffectState.h"
#include "osl/hash/hashKeyStack.h"
#include "osl/stl/slist.h"
#include "osl/stl/vector.h"
#include "osl/sennichite.h"
#include <boost/scoped_ptr.hpp>
#include <cassert>
#include <iosfwd>
namespace osl
{
  /**
   * θ.
   * Ϣ³(Ʊ̤κǽ4ܤδ֤ΰλؤ꤬Τߤä)ϡϢ³򤫤Ƥۤ餱
   * http://www.computer-shogi.org/wcsc14/youryou.html
   */
  class RepetitionCounter
  {
    class Table;
    boost::scoped_ptr<Table> table;
    CArray<osl::vector<int>, 2> continuous_check;
    HashKeyStack hash_history;
    int order() const { return hash_history.size(); }
  public:
    typedef osl::slist<int> list_t;

    RepetitionCounter();
    RepetitionCounter(const RepetitionCounter& c);
    explicit RepetitionCounter(const HashEffectState& initial);
    RepetitionCounter(const NumEffectState& initial);
    ~RepetitionCounter();

  private:
    void push(const HashKey& new_key, bool is_check);
  public:    
    /**
     * state ξ֤ move (줫)ؤȤϿ
     */
    void push(const NumEffectState& state, Move move);
    /**
     * ؤζ̤Ͽ
     */
    void push(const NumEffectState& state);
    /**
     * ؤζ̤Ͽ
     */
    void push(const HashKey& key, const NumEffectState& state);
    void pop();
    void clear();
    
    const Sennichite isSennichite(const NumEffectState& state, Move move) const;
  private:
    const Sennichite isAlmostSennichiteUnsafe(int first_move) const
    {
      assert(first_move >= 0);
      const int duration = (order() - first_move) / 2;
      if (continuous_check[BLACK].back() >= duration)
	return Sennichite::BLACK_LOSE();
      if (continuous_check[WHITE].back() >= duration)
	return Sennichite::WHITE_LOSE();
      return Sennichite::DRAW();
    }    
  public:
    /**
     * ΤޤƱ򷫤֤ɤη̤ˤʤ뤫֤
     */
    const Sennichite isAlmostSennichite(const HashKey& key) const
    {
      const int first_move = getFirstMove(key);
      if (first_move < 0)
	return Sennichite::NORMAL();
      return isAlmostSennichiteUnsafe(first_move);
    }    
    /** @return pair<isAlmostSennichite, count> */
    const std::pair<Sennichite,int> distanceToSennichite(const HashKey& key) const;
    unsigned int countRepetition(const HashKey&) const;
    const list_t getRepetitions(const HashKey&) const;
    void printMatches(const HashKey& key) const;
    /**
     * key μǸϿؼֹ.
     * @return Ƥζ̤Ǥ-1 
     */
    int getLastMove(const HashKey& key) const;
    /**
     * key μǽϿؼֹ.
     * @return Ƥζ̤Ǥ-1 
     */
    int getFirstMove(const HashKey& key) const;
    int checkCount(Player attack) const { 
      assert(! continuous_check[attack].empty());
      return continuous_check[attack].back(); 
    }
    const HashKeyStack& history() const { return hash_history; }
    bool isConsistent() const;

    static bool maybeEqual(const RepetitionCounter& l, const RepetitionCounter& r);
  };

}

#endif /* _RepetitionCounter_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
