/* searchRecorder.h
 */
#ifndef _MTDF_RECORDER_H
#define _MTDF_RECORDER_H

#include "osl/player.h"
#include "osl/move.h"
#include "osl/misc/lightMutex.h"
#include <boost/scoped_ptr.hpp>
#include <iosfwd>
namespace osl
{
  class MoveLogProb;
  namespace state
  {
    class SimpleState;
  }
  namespace search
  {
    /**
     * recorder of MTDF/AlphaBeta
     *
     * destructor virtualˤ뤳ȤǴưפˤ롥
     * method virtual*ʤ*Ȥǡ㥹Ȥ̥饹Ȥ
     * 񤦤褦ˤ롥
     */
    class CountRecorder
    {
      int node_count;
      int quiescence_count;
      int checkmate_count;
#ifdef OSL_SMP
      mutable LightMutex mutex;
#endif
    public:
      CountRecorder();
      virtual ~CountRecorder();

      /** õ潪λǻȤ褦ɬ */
      void addNodeCount(int count=1) { 
#if (defined OSL_SMP) && (defined OSL_USE_RACE_DETECTOR)
	LightMutex::scoped_lock lk(mutex);
#endif
	node_count += count; 
      }
      void addQuiescenceCount(int count=1) { 
#if (defined OSL_SMP) && (defined OSL_USE_RACE_DETECTOR)
	LightMutex::scoped_lock lk(mutex);
#endif
	quiescence_count += count; 
      }
      void addCheckmateCount(int count=1) { 
#ifdef OSL_SMP
	LightMutex::scoped_lock lk(mutex);
#endif
	checkmate_count += count; 
      }
      void setCheckmateCount(int count=1) { 
#ifdef OSL_SMP
	LightMutex::scoped_lock lk(mutex);
#endif
	checkmate_count = count; 
      }

      void resetNodeCount();
      int nodeCount() const { 
#if (defined OSL_SMP) && (defined OSL_USE_RACE_DETECTOR)
	LightMutex::scoped_lock lk(mutex);
#endif
	return node_count; 
      }
      int quiescenceCount() const { 
#if (defined OSL_SMP) && (defined OSL_USE_RACE_DETECTOR)
	LightMutex::scoped_lock lk(mutex);
#endif
	return quiescence_count; 
      }
      int checkmateCount() const { 
#if (defined OSL_SMP) && (defined OSL_USE_RACE_DETECTOR)
	LightMutex::scoped_lock lk(mutex);
#endif
	return checkmate_count; 
      }
      int allNodeCount() const { 
#if (defined OSL_SMP) && (defined OSL_USE_RACE_DETECTOR)
	LightMutex::scoped_lock lk(mutex);
#endif
	return node_count + quiescence_count + checkmate_count; 
      }
      /** recordValue ȥåȤǸƤ */
      void tryMove(const MoveLogProb& /*m*/, int /*last_f*/, int /*limit*/) const {}
      /** recordValue ȥåȤǸƤ */
      void retryMove(const MoveLogProb& /*m*/, int /*last_f*/, int /*limit*/,
		     int /*retryCount*/) const {}
      /** tryMove ȥåȤǸƤ */
      void recordValue(const MoveLogProb&, int /*val*/, bool /*better_move*/,
		       int /*limit*/) const {}

      /** ˿εϿ */
      void recordTopLevelLowFail(const MoveLogProb& /* best */, int /*last_f*/) const {}
      void recordTopLevelHighFail(const MoveLogProb& /*best */, int /*last_f*/) const {}

      void tableHitLowerBound(Player, int, int /*last_f*/, int /*limit*/) const {}
      void tableHitUpperBound(Player, int, int /*last_f*/, int /*limit*/) const {}

      void tableStoreLowerBound(Player, const MoveLogProb&, int, int) const {}
      void tableStoreUpperBound(Player, const MoveLogProb&, int, int) const {}
    

      void startSearch(int /*limit*/) const {}
      /** ٤Ƥⵤˤʤ */
      virtual void finishSearch(Move best, double seconds_consumed,
				bool verbose) const;

      void recordInvalidMoveInTable(const state::SimpleState&, 
				    const MoveLogProb&, int limit) const;
      void newCategory(const char */*name*/, int /*limit*/) const {}

      /** ;̵¥롼ȯ */
      void gotoCheckmateSearch(const state::SimpleState&, int) const {}
      void backFromCheckmateSearch() const {}

      void reportCount(std::ostream&, double seconds) const;
      void reportCount(std::ostream&) const;
    };

    class SearchRecorder : public CountRecorder
    {
      class Recorder;
      /** hide implementation */
      boost::scoped_ptr<Recorder> recorder;
    public:
      explicit SearchRecorder(const char *filename="mtdf.log");
      ~SearchRecorder();

      /** ɤٿϿ뤫ؼ */
      void setLogMargin(int margin=500);

      void tryMove(const MoveLogProb& m, int last_f, int limit) const;
      void retryMove(const MoveLogProb& m, int last_f, int limit,
		     int retryCount) const;

      void recordValue(const MoveLogProb& m, int val, bool betterMove, int limit) const;

      void tableHitLowerBound(Player p, int val, int last_f, int limit) const;
      void tableHitUpperBound(Player p, int val, int last_f, int limit) const;

      void tableStoreLowerBound(Player p, const MoveLogProb& best_move, int val, int limit) const;
      void tableStoreUpperBound(Player p, const MoveLogProb& best_move, int val, int limit) const;

      void recordTopLevelLowFail(const MoveLogProb& /* best */, int last_f) const;
      void recordTopLevelHighFail(const MoveLogProb& best_move, int last_f) const;

      void startSearch(int limit) const;
      void finishSearch(Move best_move, double seconds_consumed, bool verbose) const;

      void newCategory(const char *name, int limit) const;

      void gotoCheckmateSearch(const state::SimpleState&, int nodeLimit) const;
      void backFromCheckmateSearch() const;

      /** ˥å򺮤Ȥ˻Ȥ */
      std::ostream& stream() const;
    };
  } // namespace search

  using search::CountRecorder;
  using search::SearchRecorder;
} // namespace osl


#endif /* _MTDF_RECORDER_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
