/* searchState2.h
 */
#ifndef SEARCHSTATE2_H
#define SEARCHSTATE2_H

#include "osl/search/killerMoveTable.h"
#include "osl/search/bigramKillerMove.h"
#include "osl/search/firstMoveThreatmate.h"
#include "osl/search/simpleHashRecord.h"
#if defined OSL_SMP && ! defined NEW_DFPN
#  define NEW_DFPN
#endif
#ifdef NEW_DFPN
#  include "osl/checkmate/dualDfpn.h"
#else
#  include "osl/checkmate/dualCheckmateSearcher.h"
#endif
#include "osl/checkmate/fixedDepthSearcher.h"
#include "osl/effect_util/neighboring8Direct.h"
#include "osl/state/numEffectState.h"
#include "osl/hash/hashKey.h"
#include "osl/repetitionCounter.h"
#include "osl/container/moveStack.h"
#include "osl/apply_move/applyMoveWithPath.h"
#include "osl/misc/fixedCapacityVector.h"
#include "osl/misc/align16New.h"

#include <boost/shared_ptr.hpp>
#include <boost/utility.hpp>

namespace osl
{
  namespace search
  {
    /**
     * SimpleHashRecord* stack. 
     * ƬǤrooṭ롣
     */
    class RecordStack2
    {
      static const int SEARCH_DEPTH_MAX = 64;
      FixedCapacityVector<SimpleHashRecord*, SEARCH_DEPTH_MAX> data;
    public:
      RecordStack2();
      void clear();
      void push(SimpleHashRecord *r) { data.push_back(r); }
      void pop() { assert(size() > 1); data.pop_back(); }

      SimpleHashRecord* lastRecord(unsigned int n=0) const
      {
	assert(size() > n);
	return data[size()-1-n];
      }
      SimpleHashRecord* rootRecord() const
      {
	assert(! empty());
	return data[0];
      }
      void setRootRecord(SimpleHashRecord *root) { data[0] = root; }
      void setLastRecord(SimpleHashRecord *r) { 
	assert(size() > 0);	// 1 for root
	data[size()-1] = r;
      }

      size_t size() const { return data.size(); }
      bool empty() const { return data.empty(); }
      bool hasLastRecord(unsigned int n=0) const
      {
	return size() > n;
      }

      void dump() const;
    };

    class Worker;
    /**
     * õ򤹤˶ͭ
     */
    struct SearchState2Shared : boost::noncopyable
    {
      BigramKillerMove bigram_killers;
      KillerMoveTable killer_moves;

      SearchState2Shared();
      ~SearchState2Shared();
    };
#define search_assert(x) assert(((x) || (SearchState2Core::abort())))
#define search_assert2(x, m) assert(((x) || (SearchState2Core::abort(m))))
    class AlphaBeta2ParallelCommon;
    /**
     */
    class SearchState2Core
    {
      friend class AlphaBeta2ParallelCommon;
    public:
      enum { MaxDepth = 64 };
#ifdef NEW_DFPN
      typedef DualDfpn checkmate_t;
    protected:
#else
      typedef DualCheckmateSearcher<> checkmate_t;
    private:
#endif
      NumEffectState current_state;
      checkmate_t* checkmate_searcher;
#ifdef OSL_SMP
    public:
      void setCheckmateSearcher(checkmate_t *new_checkmate) {
	checkmate_searcher = new_checkmate;
      }
    private:
#endif
      PathEncoding current_path;
      MoveStack move_history;
      int root_depth;
    protected:
      RecordStack2 record_stack;
      RepetitionCounter repetition_counter;
      boost::shared_ptr<SearchState2Shared> shared;
    public:
      typedef FixedCapacityVector<Move,MaxDepth> PVVector;
    protected:
      CArray<PVVector,MaxDepth> pv;
      enum NodeType { PvNode = 0, AllNode = 1, CutNode = -1, };
      CArray<NodeType,MaxDepth> node_type;
    public:
      /** beta cut in parallel search */
      volatile bool stop_tree;
      static CArray<int, MaxDepth> depth_node_count_quiesce;
      SearchState2Core(const NumEffectState& s, checkmate_t& checker);
      virtual ~SearchState2Core();
      int curDepth() const { return history().size() - root_depth; }

      /**
       * state ΥԡԤ.
       *
       * this->state õλ¸뤬õ exception 
       * ˲Ƥ
       */
      virtual void setState(const NumEffectState& s);
      void setHistory(const MoveStack& h);
      bool hasLastRecord(unsigned int n=0) const
      {
	return record_stack.hasLastRecord(n);
      }
      SimpleHashRecord* lastRecord(unsigned int n=0) 
      {
	return record_stack.lastRecord(n); 
      }
      const SimpleHashRecord* lastRecord(unsigned int n=0) const
      {
	return record_stack.lastRecord(n); 
      }
      SimpleHashRecord *rootRecord()
      {
	return record_stack.rootRecord();
      }
      void setCurrentRecord(SimpleHashRecord *r)
      {
	search_assert((int)record_stack.size() == curDepth()+1);
	record_stack.setLastRecord(r);
      }
      void setRootRecord(SimpleHashRecord *root)
      {
	search_assert(record_stack.size() == 1);
	search_assert(curDepth() == 0);
	record_stack.setRootRecord(root); 
      }

      // killer move
      void setKillerMove(Move best_move)
      {
	if (best_move.isPass())
	  return;
	shared->killer_moves.setMove(curDepth(), best_move);
	const Move last_move = lastMove();
	if (! last_move.isInvalid()) {
	  search_assert(best_move.player() != last_move.player());
	  shared->bigram_killers.setMove(last_move, best_move);
	}
      }
      void getBigramKillerMoves(MoveVector& moves) const
      {
	shared->bigram_killers.getMove(state(), lastMove(), moves);
#ifdef TRI_PLY_BIGRAM_KILLERS
	if (move_history.hasLastMove(3))
	  shared->bigram_killers.getMove(state(), lastMove(3), moves);
#endif
      }
      void getKillerMoves(MoveVector& moves) const
      {
	getBigramKillerMoves(moves);
	shared->killer_moves.getMove(state(), curDepth(), moves);
      }
      const BigramKillerMove& bigramKillerMove() const { 
	return shared->bigram_killers;
      }
      void setBigramKillerMove(const BigramKillerMove& killers);
      
      // doUndo
      void pushPass()
      {
	const Move pass = Move::PASS(current_state.getTurn());
	current_state.changeTurn();
	current_path.pushMove(pass);
	move_history.push(pass);
      }
      void popPass()
      {
	const Move pass = Move::PASS(alt(current_state.getTurn()));
	current_state.changeTurn();
	current_path.popMove(pass);
	move_history.pop();
      }
    private:
      /**
       * ApplyMove˹Ԥ
       */
      void pushBeforeApply(Move move)
      {
	move_history.push(move);
	record_stack.push(0);
	assert(curDepth() > 0);
	node_type[curDepth()] = static_cast<NodeType>(-node_type[curDepth()-1]);
      }
      struct Updator
      {
	const HashKey& new_hash;
	SearchState2Core *state;
	Updator(const HashKey& h, SearchState2Core *s)
	  : new_hash(h), state(s)
	{
	}
	void update()
	{
	  state->updateRepetitionCounterAfterMove(new_hash);
	}
      };
      template <class Function>
      struct UpdateWrapper : public Updator
      {
	Function& function;
	UpdateWrapper(const HashKey& h, SearchState2Core *s, Function& f)
	  : Updator(h, s), function(f)
	{
	}
	void operator()(Position to)
	{
	  update();
	  function(to);
	}
      };
      friend class Updator;
      /**
       * pushBeforeApply θ塤ApplyMove桤FunctionƤ˸ƤФ
       */
      void updateRepetitionCounterAfterMove(const HashKey& new_hash)
      {
	repetition_counter.push(new_hash, current_state);
      }
      /**
       * ApplyMoveθ˹Ԥ
       */
      void popAfterApply()
      {
	record_stack.pop();
	repetition_counter.pop();
	move_history.pop();
      }
    public:
      /**
       * ޤȤdoUndo
       */
      template <Player P, class Function>
      void doUndoMoveOrPass(const HashKey& new_hash,
			    Move move, Function& f)
      {
	pushBeforeApply(move);
	UpdateWrapper<Function> wrapper(new_hash, this, f);
	ApplyMoveWithPath<P>::doUndoMoveOrPass(current_state, current_path, 
					       move, wrapper);
	popAfterApply();
      }
      void makeMove(Move move)	// for debug
      {
	HashKey new_hash = currentHash().newHashWithMove(move);
	pushBeforeApply(move);
	ApplyMoveOfTurn::doMove(current_state, move);
	updateRepetitionCounterAfterMove(new_hash);
      }
      
      const Move lastMove(int i=1) const { return move_history.lastMove(i); }
      const MoveStack& history() const { return move_history; }
      const RecordStack2& recordHistory() const { return record_stack; }
      const PathEncoding& path() const { return current_path; }
      const NumEffectState& state() const { return current_state; }
      const checkmate_t& checkmateSearcher() const { return *checkmate_searcher; }
      const RepetitionCounter& repetitionCounter() const { 
	return repetition_counter;
      }
      const HashKey& currentHash() const
      {
	return repetition_counter.history().top();
      }

      /**
       * ̲ doUndo , hash 򹹿ʤ
       */
      template <Player P, class Function>
      void doUndoMoveLight(Move move, Function& f)
      {
	ApplyMoveWithPath<P>::doUndoMoveOrPass
	  (static_cast<NumEffectState&>(current_state), current_path, 
	   move, f);
      }

      template <Player P>
      bool isLosingState(int node_limit)
      {
	search_assert(P == path().turn());
	const bool lose = 
	  checkmate_searcher->isLosingState<P>
	  (node_limit, current_state, currentHash(), path(), lastMove());
	return lose;
      }
    public:
      template <Player P>
      bool isWinningState(int node_limit, Move& checkmate_move, 
			  AttackOracleAges& age)
      {
	search_assert(P == path().turn());
#ifdef NEW_DFPN
	const bool win = checkmate_searcher->isWinningState<P>
	  (node_limit, current_state, currentHash(), path(), checkmate_move, lastMove());
#else
	const bool win = checkmate_searcher->isWinningState<P>
	  (node_limit, current_state, currentHash(), path(), checkmate_move, age,
	   lastMove());
#endif
	return win;
      }
      /** FixedDepthSearcher Ƥ */
      template <Player P>
      bool isWinningStateShort(int depth, Move& checkmate_move)
      {
	checkmate::FixedDepthSearcher searcher(current_state);
	const ProofDisproof pdp=searcher.hasCheckmateMove<P>(depth, checkmate_move);
	return pdp.isCheckmateSuccess();
      }
      /**
       * P μ֤Pζ̤˵ͤäƤ뤫ɤ
       */
      template <Player P>
      bool isThreatmateState(int node_limit, Move& threatmate_move, 
			     AttackOracleAges& age)
      {
	search_assert(P == path().turn());
	current_state.changeTurn();
	HashKey threatmate_hash = currentHash();
	threatmate_hash.changeTurn();
	const PathEncoding threatmate_path(current_state.getTurn());
	const Player Opponent = PlayerTraits<P>::opponent;
#ifdef NEW_DFPN
	const bool threatmate 
	  = checkmate_searcher->template isWinningState<Opponent>
	  (node_limit, current_state, 
	   threatmate_hash, threatmate_path, threatmate_move, Move::PASS(P));
#else
	const bool threatmate 
	  = checkmate_searcher->template isWinningState<Opponent>
	  (node_limit, current_state, 
	   threatmate_hash, threatmate_path, threatmate_move, age);
#endif
	current_state.changeTurn();
	return threatmate;
      }
      template <Player P>
      bool isThreatmateStateShort(int depth, Move& threatmate_move)
      {
	search_assert(P == path().turn());
	current_state.changeTurn();

	const Player Opponent = PlayerTraits<P>::opponent;

	checkmate::FixedDepthSearcher searcher(current_state);
	const ProofDisproof pdp
	  = searcher.hasCheckmateMove<Opponent>(depth, threatmate_move);

	current_state.changeTurn();
	return pdp.isCheckmateSuccess();
      }
      bool abort() const;
      virtual bool abort(Move) const;

      bool tryThreatmate() const 
      {
	const Move last_move = lastMove();
	if (! last_move.isNormal())
	  return false;
	const Position king = state().getKingPosition(state().getTurn());
	if (curDepth() == 1)
	  return FirstMoveThreatmate::isMember(last_move, king);
	return Neighboring8Direct::hasEffect
	  (state(), last_move.ptypeO(), last_move.to(), king);

      }

      void makePV(Move m) 
      {
	const int depth = curDepth();
	makePV(pv[depth], m, pv[depth+1]);
      }
      void initPV() 
      {
	const int depth = curDepth();
	pv[depth].clear();
      }
      void makePV(PVVector& parent, Move m, PVVector& pv) const;
      /** turn ¦Ϣ³ǵͤƨƤ */
      int countCheckAfterThreatmate(Player turn, int depth=1) const
      {
	assert(((depth % 2) && state().getTurn() == turn)
	       || ((depth % 2) == 0 && state().getTurn() != turn));
	int result = 0;
	for (int i=depth;; i+=2) {
	  if (! hasLastRecord(i) || ! lastRecord(i))
	    break;
	  if (lastRecord(i)->qrecord.threatmate.status(turn).status()
	      != ThreatmateState::CHECK_AFTER_THREATMATE)
	    break;
	  ++result;
	}
	return result;
      }
      int countCheckAfterThreatmateSacrifice(Player turn, int depth=1) const
      {
	assert(((depth % 2) && state().getTurn() == turn)
	       || ((depth % 2) == 0 && state().getTurn() != turn));
	assert(depth > 0);
	int result = 0;
	for (int i=depth;; i+=2) {
	  if (! hasLastRecord(i) || ! lastRecord(i))
	    break;
	  if (lastRecord(i)->qrecord.threatmate.status(turn).status()
	      != ThreatmateState::CHECK_AFTER_THREATMATE)
	    break;
	  if (lastMove(i-1).capturePtype() != PTYPE_EMPTY)
	    ++result;
	}
	return result;
      }
    };

#undef search_assert    
#undef search_assert2
#define search_assert(x) assert((x) || SearchState2Core::abort())
#define search_assert2(x, m) assert((x) || SearchState2Core::abort(m))
    /**
     * SearchFramework Τtemplate parameter ޤޤʤʬ.
     */
    class SearchState2 : public SearchState2Core
#if OSL_WORDSIZE == 32
		       , public misc::Align16New
#endif
    {
    public:
      /** õ䡤ؼǤΨι⤤꤬äȤ̵뤹ϰ */
      static const int ReSearchLimitMargin = 80;
    protected:
      int root_limit;
      int cur_limit;
    public:
      SearchState2(const NumEffectState& s, checkmate_t& checker);
      virtual ~SearchState2();

      void setState(const NumEffectState& s);
      void setKillerMove(Move best_move)
      {
	if (best_move.isPass())
	  return;
	SearchState2Core::setKillerMove(best_move);
      }
      
      int curLimit() const { return cur_limit; }

      bool abort(Move) const;

    protected:
      /**
       * root  limitͤõϤ뤳Ȥ
       */
      void setRoot(int limit)
      {
	root_limit = limit;
	cur_limit = limit;
      }
      void addLimit(int limit) { cur_limit += limit; search_assert(cur_limit >= 0); }
      void subLimit(int limit) { cur_limit -= limit; search_assert(cur_limit >= 0); }

      /** μΤƶϢ³2롼פޤǿ 
       * @param history_max ʾվʤ
       */
      int countSacrificeCheck2(int history_max) const;
      /** debug  */
      void checkPointSearchAllMoves();
    };
  } // namespace search
} // namespace osl

#undef search_assert
#undef search_assert2

#endif /* SEARCHSTATE2_H */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
