/* dfpn.cc
 */
#include "osl/checkmate/dfpn.h"
#include "osl/checkmate/dfpnRecord.h"
#include "osl/checkmate/immediateCheckmate.h"
#include "osl/checkmate/fixedDepthSearcher.h"
#include "osl/checkmate/fixedDepthSearcher.tcc"
#include "osl/checkmate/libertyEstimator.h"
#include "osl/checkmate/pieceCost.h"
#include "osl/checkmate/disproofPieces.h"
#include "osl/checkmate/oracleAdjust.h"
#include "osl/checkmate/pawnCheckmateMoves.h"
#include "osl/move_generator/escape_.h"
#include "osl/move_generator/addEffect_.h"
#include "osl/move_action/store.h"
#include "osl/move_action/safeFilter.h"
#include "osl/move_action/checkFilter.h"
#include "osl/move_classifier/check_.h"
#include "osl/move_classifier/moveAdaptor.h"
#include "osl/move_classifier/pawnDropCheckmate.h"
#include "osl/apply_move/applyMoveWithPath.h"
#include "osl/container/moveVector.h"
#include "osl/stl/hash_map.h"
#include "osl/stl/slist.h"
#include "osl/stat/ratio.h"
#include <boost/tuple/tuple.hpp>
#include <boost/tuple/tuple_comparison.hpp>
#include <iostream>
#include <bitset>

#define GRAND_PARENT_SIMULATION
#define GRAND_PARENT_DELAY

#define INITIAL_DOMINANCE
/** root ڤ */
#define ROOT_PROOF_TOL 65536ul*1024
/** root ڤȿڿ */
#define ROOT_DISPROOF_TOL 65536ul*1024
#define DELAY_UPWARD
#define CHECKMATE_D2
#define PROOF_AVERAGE
#define DISPROOF_AVERAGE

#define KAKINOKI_FALSE_BRANCH_SEARCH
#define CHECKMATE_A3_GOLD
#define CEHCKMATE_A3_SIMULLATION

// ܤ줿ؼ꤬ѤߤϿ϶ֹ˰¸Τա
#define MEMORIZE_SOLVED_IN_BITSET

// #define DFPN_STAT

static const int UpwardWeight = 2, SacrificeBlockCount = 0, LongDropCount = 1;
static const unsigned int min_record_node_count = 1;

#ifdef MEMORIZE_SOLVED_IN_BITSET
static const unsigned int InitialDominanceProofMax = 25;
#else
static const unsigned int InitialDominanceProofMax = 20;
#endif
static const unsigned int InitialDominanceDisproofMax = 110;

#ifndef NDEBUG
static size_t timer = 0;
#endif
/* ------------------------------------------------------------------------- */

namespace osl
{
  namespace checkmate
  {
    struct SimpleTwinList : slist<PathEncoding>
    {
    };

    struct DfpnPathRecord
    {
      bool visiting;
      /** distance from root */
      int distance;
      SimpleTwinList twin_list;
      DfpnPathRecord() : visiting(false), distance(Dfpn::MaxDepth)
      {
      }
    };
    template <bool Enabled=true>
    struct DfpnVisitLock : boost::noncopyable
    {
      DfpnPathRecord *record;
      DfpnVisitLock(DfpnPathRecord *r) : record(r)
      {
	if (! Enabled) return;
	assert(! record->visiting);
	record->visiting = true;
      }
      ~DfpnVisitLock()
      {
	if (! Enabled) return;
	assert(record->visiting);
	record->visiting = false;
      }
    };
    enum LoopToDominance { NoLoop=0, BadAttackLoop };
    struct DfpnPathList : public slist<std::pair<PieceStand, DfpnPathRecord> >
    {
    private:
      template <Player Attack>
      iterator find(PieceStand black, LoopToDominance& loop)
      {
	loop = NoLoop;
	iterator ret = end();
	for (iterator p=begin(); p!=end(); ++p) {
	  if (p->first == black) {
	    assert(p->second.distance != Dfpn::MaxDepth);
	    ret = p;
	    if (loop || p->second.visiting) break;
	  } 
	  if (! p->second.visiting)
	    continue;
	  if (p->first.isSuperiorOrEqualTo(black)) {
	    if (Attack == BLACK) {
	      loop = BadAttackLoop;
	      if (ret != end()) break;
	    } 
	  } 
	  else if (black.isSuperiorOrEqualTo(p->first)) {  
	    if (Attack == WHITE) {
	      loop = BadAttackLoop;
	      if (ret != end()) break;
	    }	    
	  }
	}
	return ret;
      }
    public:
      template <Player Attack>
      DfpnPathRecord *allocate(PieceStand black, int depth, LoopToDominance& loop)
      {
	iterator ret = find<Attack>(black, loop);
	if (ret != end()) {
	  ret->second.distance = std::min(depth, ret->second.distance);
	  return &(ret->second);
	}
	push_front(std::make_pair(black, DfpnPathRecord()));
	DfpnPathRecord *record = &(begin()->second);
	assert(record->distance == Dfpn::MaxDepth);
	record->distance = depth;
	return record;
      }
      const DfpnPathRecord *probe(PieceStand black) const
      {
	for (const_iterator p=begin(); p!=end(); ++p) {
	  if (p->first == black)
	    return &(p->second);
	}
	return 0;
      }
    };
    class DfpnPathTable 
    {
      typedef hash_map<SignatureKey, DfpnPathList> table_t;
      table_t table;
    public:
      template <Player Attack>
      DfpnPathRecord *allocate(const HashKey& key, int depth, LoopToDominance& loop) 
      {
	DfpnPathList& l = table[key.getSignatureKey()];
	return l.allocate<Attack>(key.blackStand(), depth, loop);
      }
      const DfpnPathRecord *probe(const HashKey& key) const
      {
	table_t::const_iterator p = table.find(key.getSignatureKey());
	if (p == table.end()) 
	  return 0;
	return p->second.probe(key.blackStand());
      }
      void clear() { table.clear(); }
    };

    int attackProofCost(Player attacker, const NumEffectState& state, Move move)
    {
      int proof = 0;
      if (move.capturePtype() == PTYPE_EMPTY)
      {
	const Position from=move.from(), to=move.to();
	const int a = (state.countEffect(attacker,to) 
		       + (from.isPieceStand() ? 1 : 0));
	int d = state.countEffect(alt(attacker),to);
	if (a <= d)
	{
	  const Ptype ptype = move.ptype();
	  proof = PieceCost::attack_sacrifice_cost[ptype];
	  if ((d >= 2) && (a == d))	// ɲȤ줿Ȥ
	    proof /= 2;
	}
      }
      return proof;
    }
  }
}

/* ------------------------------------------------------------------------- */
struct osl::checkmate::Dfpn::NodeBase
{
  // input
  HashKey hash_key;
  PathEncoding path;
  ProofDisproof threshold;
  Move moved;
  PieceStand white_stand;
  // work or output
  DfpnRecord record;
  DfpnPathRecord *path_record;
};

struct osl::checkmate::Dfpn::Node : NodeBase
{
  MoveVector moves;
  FixedCapacityVector<int,Move::MaxUniqMoves> proof_cost; // only attack
  FixedCapacityVector<DfpnRecord,Move::MaxUniqMoves> children;
  FixedCapacityVector<const DfpnPathRecord*,Move::MaxUniqMoves> children_path;
  CArray<HashKey,Move::MaxUniqMoves> hashes;
  size_t visit_time;

  const PieceStand nextWhiteStand(Player P, Move move) const
  {
    assert(move.player() == P);
    return (P == WHITE) ? white_stand.nextStand(P, move) : white_stand;
  }
  void clear()
  {
    moves.clear();
    proof_cost.clear();
    children.clear();
    children_path.clear();
  }
  void allocate(int n)
  {
    while (n--) {
      proof_cost.push_back(0);
      children.push_back(DfpnRecord());
      children_path.push_back(0);
    }
  }
  void setLoopDetection()
  {
    assert(! (record.proof_disproof.isFinal()
	      && ! record.proof_disproof.isLoopDetection()));
    record.proof_disproof = ProofDisproof(1,1);
    path_record->twin_list.push_front(path);
  }
  const PathEncoding newPath(int c) const
  {
    PathEncoding n = path;
    n.pushMove(moves[c]);
    return n;
  }
  bool isLoop(int c) const
  {
    if (! children_path[c] || children[c].proof_disproof.isFinal())
      return false;
    if (children_path[c]->visiting)
      return true;
    const PathEncoding p = newPath(c);
    const SimpleTwinList& tl = children_path[c]->twin_list;
    return std::find(tl.begin(), tl.end(), p) != tl.end();
  }
  void setCheckmateAttack(Player attack, int best_i)
  {
    DfpnRecord& child = children[best_i];
    assert(child.proof_disproof.isCheckmateSuccess());
    record.proof_disproof = child.proof_disproof;
    record.best_move = moves[best_i];
    const PieceStand proof_pieces 
      = ProofPieces::attack(child.proofPieces(), record.best_move,
			    record.stands[attack]);
    record.setProofPieces(proof_pieces);
  }
  void setNoCheckmateDefense(Player attack, int best_i)
  {
    DfpnRecord& child = children[best_i];
    assert(child.proof_disproof.isCheckmateFail());
    assert(! child.proof_disproof.isLoopDetection());
    record.proof_disproof = child.proof_disproof;
    record.best_move = moves[best_i];
    const PieceStand disproof_pieces 
      = DisproofPieces::defense(child.disproofPieces(), record.best_move,
				record.stands[alt(attack)]);
    record.setDisproofPieces(disproof_pieces);
  }
  void setCheckmateDefense(Player attack, const NumEffectState& state) 
  {
    assert(moves.size());
    assert(record.proof_disproof.isCheckmateSuccess());
    PieceStand result = record.proof_pieces_candidate;
    const Player defender = alt(attack);
    if (! effect_util::UnblockableCheck::isMember(defender, state))
      ProofPiecesUtil::addMonopolizedPieces(state, attack, record.stands[attack],
					    result);
    record.setProofPieces(result);
  }
  void setNoCheckmateAttack(Player attack, const NumEffectState& state)
  {
    assert(moves.size());
    assert(record.proof_disproof.isCheckmateFail());
    assert(! record.proof_disproof.isLoopDetection());
    PieceStand result = record.proof_pieces_candidate;
    ProofPiecesUtil::addMonopolizedPieces(state, alt(attack), record.stands[alt(attack)],
					  result);
    record.setDisproofPieces(result);
  }
  void setCheckmateChildInDefense(size_t i) 
  {
    assert(children[i].proof_disproof.isCheckmateSuccess());
#ifdef MEMORIZE_SOLVED_IN_BITSET
    record.solved |= (1ull<<i);
#endif
    record.min_pdp = std::min(record.min_pdp, children[i].proof_disproof.disproof());
    record.proof_pieces_candidate
      = record.proof_pieces_candidate.max(children[i].proofPieces());
  }
  void setNoCheckmateChildInAttack(size_t i) 
  {
    assert(children[i].proof_disproof.isCheckmateFail());
#ifdef MEMORIZE_SOLVED_IN_BITSET
    record.solved |= (1ull<<i);
#endif
    record.min_pdp = std::min(record.min_pdp, children[i].proof_disproof.proof());
    record.proof_pieces_candidate
      = record.proof_pieces_candidate.max(children[i].disproofPieces());
  }
};

struct osl::checkmate::Dfpn::Tree
{
  Node node[MaxDepth];
  NumEffectState state;
  int depth;
  Tree() : state(SimpleState(HIRATE))
  {
  }
  bool inCheck(Player P) const
  {
    return state.hasEffectBy(alt(P), state.getKingPosition(P));
  }
  const Piece king(Player P) const { return state.getKingPiece(P); }
  void newVisit(Player P, Move move, const HashKey& next_hash)
  {
    assert(P == move.player());
    const Node& node = this->node[depth];
    assert(next_hash == node.hash_key.newHashWithMove(move));
    Node& next = this->node[depth+1];
    next.moved = move;
    next.white_stand = node.nextWhiteStand(P, move);
    next.path = node.path;
    if (1 || next.hash_key != next_hash || next.moves.size() <= 1 /* clear oracle move */) {
      next.clear();
      next.hash_key = next_hash;
    }
  }
  void setNoCheckmateChildInAttack(size_t best_i) 
  {
    Node &node = this->node[depth];
    node.setNoCheckmateChildInAttack(best_i);
  }
  void setNoCheckmateDefense(Player attack, int best_i)
  {
    Node &node = this->node[depth];
    node.setNoCheckmateDefense(attack, best_i);
  }
  void dump(int lines, int depth=0)
  {
#ifndef NDEBUG
    if (depth == 0)
      depth = this->depth;
    for (int i=0; i<=depth; ++i) {
      std::cerr << node[i].moved << "\n";
    }
    Node &node = this->node[depth];
    std::cerr << "time " << node.visit_time << " (" << timer << ") here " << lines << "\n" << state;
    std::cerr << " false-branch? " << (bool)node.record.false_branch << "\n";
#ifdef MEMORIZE_SOLVED_IN_BITSET
    std::cerr << " solved " << std::bitset<32>(node.record.solved) << "\n";
#endif
    std::cerr << " last_to " << node.record.last_to << "\n";
    for (size_t i=0; i<node.moves.size(); ++i) {
      std::cerr << node.moves[i]
		<< " " << node.children[i].proof_disproof
		<< " " << node.children[i].best_move
		<< " " << node.children_path[i]
		<< " " << node.children[i].node_count
		<< "\n";
    }
    std::cerr << node.record.proof_disproof << " " << node.record.best_move << "\n";
    std::cerr << "path " << node.path << " twins ";
    if (node.path_record) {
      const SimpleTwinList& l = node.path_record->twin_list;
      for (SimpleTwinList::const_iterator p=l.begin(); p!=l.end(); ++p)
	std::cerr << *p << " ";
    }
    std::cerr << "\n";
#endif
  }
};

/* ------------------------------------------------------------------------- */

struct osl::checkmate::DfpnTable::List : public slist<DfpnRecord> 
{
#ifdef OSL_SMP
  mutable Mutex mutex;
#endif
  List() {}
  List(const List& src) : slist<DfpnRecord>(src) {}

  typedef slist<DfpnRecord> list_t;
  template <Player Attack>
  const DfpnRecord probe(const HashKey& key, PieceStand white_stand) const;
  template <Player Attack>
  const DfpnRecord findProofOracle(const HashKey& key, PieceStand white_stand, Move last_move) const;
  template <Player Attack>
  void showProofOracles(const HashKey& key, PieceStand white_stand, Move last_move) const;
  bool store(const DfpnRecord& value)
  {
    for (list_t::iterator p=begin(); p!=end(); ++p) {
      if (p->stands[BLACK] == value.stands[BLACK]) {
	*p = value;
	return false;
      }
    }
    push_front(value);
    return true;
  }
};
template <osl::Player A>
const osl::checkmate::DfpnRecord osl::checkmate::DfpnTable::
List::probe(const HashKey& key, PieceStand white_stand) const
{
  DfpnRecord result(key.blackStand(), white_stand);
  const PieceStand attack_stand = (A == BLACK) ? key.blackStand() : white_stand;
  const PieceStand defense_stand = (A == BLACK) ? white_stand : key.blackStand();
#ifdef INITIAL_DOMINANCE
  unsigned int proof_ll = 1, disproof_ll = 1;
#endif
  for (list_t::const_iterator p=begin(); p!=end(); ++p) {
    if (p->stands[BLACK] == key.blackStand()) {
      result = *p;
      if (result.proof_disproof.isFinal())
	break;
      continue;
    }
    if (p->proof_disproof.isCheckmateSuccess()) {
      if (attack_stand.isSuperiorOrEqualTo(p->proofPieces())) {
	result.setFrom(*p);
	break;
      }
    }
    else if (p->proof_disproof.isCheckmateFail()) {
      if (defense_stand.isSuperiorOrEqualTo(p->disproofPieces())) {
	result.setFrom(*p);
	break;
      }
    }
#ifdef INITIAL_DOMINANCE
    else if (p->stands[A].isSuperiorOrEqualTo(attack_stand)) {
      proof_ll = std::max(proof_ll, p->proof());
    }
    else if (attack_stand.isSuperiorOrEqualTo(p->stands[A])) {
      disproof_ll = std::max(disproof_ll, p->disproof());
    }
#endif
  }
#ifdef INITIAL_DOMINANCE
  if (result.proof_disproof == ProofDisproof(1,1)) {
      result.proof_disproof = ProofDisproof(std::min(proof_ll, InitialDominanceProofMax), 
					    std::min(disproof_ll, InitialDominanceDisproofMax));
  }
#endif
  return result;
}

template <osl::Player A>
const osl::checkmate::DfpnRecord osl::checkmate::DfpnTable::
List::findProofOracle(const HashKey& key, PieceStand white_stand, Move last_move) const
{
  const PieceStand attack_stand = (A == BLACK) ? key.blackStand() : white_stand;
  DfpnRecord result(key.blackStand(), white_stand);
  for (list_t::const_iterator p=begin(); p!=end(); ++p) {
    if (! p->proof_disproof.isCheckmateSuccess()) 
      continue;
    if (attack_stand.isSuperiorOrEqualTo(p->proofPieces())) {
      result.setFrom(*p);
      if (p->last_move == last_move)
	break;
    }
  }
  return result;
}

template <osl::Player A>
void osl::checkmate::DfpnTable::
List::showProofOracles(const HashKey& key, PieceStand white_stand, Move last_move) const
{
  std::cerr << "search proof oracles after " << last_move << " size " << size() << "\n";
  const PieceStand attack_stand = (A == BLACK) ? key.blackStand() : white_stand;
  for (list_t::const_iterator p=begin(); p!=end(); ++p) {
    if (! p->proof_disproof.isCheckmateSuccess()) 
      continue;
    if (attack_stand.isSuperiorOrEqualTo(p->proofPieces())) {
      std::cerr << p->last_move << " " << p->best_move << " " << p->node_count << " " << p->proofPieces()
		<< " " << p->stands[BLACK] << " " << p->stands[WHITE] << "\n";
    }
  }
}

struct osl::checkmate::DfpnTable::Table : public hash_map<SignatureKey, List> 
{
  Player attack;
  explicit Table(Player a=BLACK) : attack(a) {}
};

osl::checkmate::
DfpnTable::DfpnTable(Player attack) 
  : table(new Table[DIVSIZE]), total_size(0)
{
  setAttack(attack);
}

osl::checkmate::
DfpnTable::DfpnTable() 
  : table(new Table[DIVSIZE]), total_size(0)
{
}
osl::checkmate::
DfpnTable::~DfpnTable()
{
}
void osl::checkmate::
DfpnTable::showStats() const
{
  if (size()) {
    std::cerr << "total " << total_size << "\n";
    for (int i=0; i<DIVSIZE; ++i)
      std::cerr << "DfpnTable " << i << " " << table[i].size() << "\n";
  }
}

void osl::checkmate::
DfpnTable::setAttack(Player a) 
{
  assert(size() == 0);
  for (int i=0; i<DIVSIZE; ++i)
    table[i].attack = a;
}

osl::Player osl::checkmate::
DfpnTable::attack() const
{
  return table[0].attack;
}

template <osl::Player Attack>
const osl::checkmate::DfpnRecord osl::checkmate::
DfpnTable::probe(const HashKey& key, PieceStand white_stand) const
{
  const int i=keyToIndex(key);
#ifdef OSL_SMP
  Mutex::scoped_lock lk(mutex[i]);
#endif
  assert(table[i].attack == Attack);
  Table::const_iterator p = table[i].find(key.getSignatureKey());
  if (p == table[i].end())
    return DfpnRecord(key.blackStand(), white_stand);
  return p->second.probe<Attack>(key, white_stand);
}

const osl::checkmate::DfpnRecord osl::checkmate::
DfpnTable::probe(const HashKey& key, PieceStand white_stand) const
{
  if (table[0].attack == BLACK)
    return probe<BLACK>(key, white_stand);
  else
    return probe<WHITE>(key, white_stand);
}
template <osl::Player Attack>
const osl::checkmate::DfpnRecord osl::checkmate::
DfpnTable::findProofOracle(const HashKey& key, PieceStand white_stand, Move last_move) const
{
  const int i=keyToIndex(key);
#ifdef OSL_SMP
  Mutex::scoped_lock lk(mutex[i]);
#endif
  assert(table[i].attack == Attack);
  Table::const_iterator p = table[i].find(key.getSignatureKey());
  if (p == table[i].end()) 
    return DfpnRecord(key.blackStand(), white_stand);
  return p->second.findProofOracle<Attack>(key, white_stand, last_move);
}

template <osl::Player Attack>
void osl::checkmate::
DfpnTable::showProofOracles(const HashKey& key, PieceStand white_stand, Move last_move) const
{
  const int i=keyToIndex(key);
#ifdef OSL_SMP
  Mutex::scoped_lock lk(mutex[i]);
#endif
  assert(table[i].attack == Attack);
  Table::const_iterator p = table[i].find(key.getSignatureKey());
  if (p == table[i].end()) 
    return;
  return p->second.showProofOracles<Attack>(key, white_stand, last_move);
}

void osl::checkmate::
DfpnTable::store(const HashKey& key, const DfpnRecord& value)
{
  const int i=keyToIndex(key);
#ifdef OSL_SMP
  Mutex::scoped_lock lk(mutex[i]);
#endif
  assert(key.blackStand() == value.stands[BLACK]);
  assert(! value.proof_disproof.isLoopDetection());
  List& l = table[i][key.getSignatureKey()];
  if (l.store(value)) {
#ifdef OSL_USE_RACE_DETECTOR
    LightMutex::scoped_lock lk(root_mutex);
    // __sync_fetch_and_add() ?
#endif
    total_size += 1;
  }
}
void osl::checkmate::
DfpnTable::clear()
{
  total_size = 0;
  for (int i=0; i<DIVSIZE; ++i) {
#ifdef OSL_SMP
    Mutex::scoped_lock lk(mutex[i]);
#endif
    table[i].clear();
  }
}

size_t osl::checkmate::
DfpnTable::size() const
{
  return total_size;
}

/* ------------------------------------------------------------------------- */

template <osl::Player P>
struct osl::checkmate::Dfpn::CallAttack
{
  Dfpn *search;
  CallAttack(Dfpn *s) : search(s)
  {
  }
  void operator()(Position) const 
  {
    search->attack<P>();
  }
};

template <osl::Player P>
struct osl::checkmate::Dfpn::CallDefense
{
  Dfpn *search;
  CallDefense(Dfpn *s) : search(s)
  {
  }
  void operator()(Position) const 
  {
    search->defense<P>();
  }
};

/* ------------------------------------------------------------------------- */


osl::checkmate::Dfpn::Dfpn()
  : table(0), tree(new Tree), path_table(new DfpnPathTable), blocking_verify(true)
{
}
osl::checkmate::Dfpn::~Dfpn()
{
}

void osl::checkmate::Dfpn::setIllegal(const HashKey& key)
{
  // path_table DualDfpnǥꥢΤǤϸǤϤޤʤ
  LoopToDominance dummy;
  DfpnPathRecord *record = (table->attack() == BLACK)
    ? path_table->allocate<BLACK>(key, 0, dummy)
    : path_table->allocate<WHITE>(key, 0, dummy);
  record->visiting = true;

  // 餬
  DfpnRecord result(key.blackStand(), PieceStand() /* XXX: white_stand */);
  result.proof_disproof = ProofDisproof::NoCheckmate();
  table->store(key, result);
}

const osl::checkmate::ProofDisproof 
osl::checkmate::
Dfpn::hasCheckmateMove(NumEffectState& state, const HashKey& key,
		       const PathEncoding& path, size_t limit, Move& best_move, Move last_move)
{
  PieceStand dummy;
  return hasCheckmateMove(state, key, path, limit, best_move, dummy, last_move);
}

const osl::checkmate::ProofDisproof 
osl::checkmate::
Dfpn::hasCheckmateMove(NumEffectState& state, const HashKey& key,
		       const PathEncoding& path, size_t limit, Move& best_move, PieceStand& proof_pieces,
		       Move last_move)
{
  assert(table);
  if (! table)
    return ProofDisproof();
  path_table->clear();
  
  tree->state = state;
  node_count = tree->depth = 0;
  node_count_limit = limit;

  Node& root = tree->node[0];
  root.hash_key = key;
  root.path = path;
  root.clear();
  root.threshold = ProofDisproof(ROOT_PROOF_TOL, ROOT_DISPROOF_TOL);
  root.white_stand = PieceStand(WHITE, state);
  root.moved = last_move;

  try {
    if (state.getTurn() == BLACK)
      attack<BLACK>();
    else
      attack<WHITE>();
  }
  catch (DepthLimitReached&) {
    return ProofDisproof();
  }
  if (root.path_record
      && (std::find(root.path_record->twin_list.begin(), root.path_record->twin_list.end(), path)
	  != root.path_record->twin_list.end()))
    return ProofDisproof::LoopDetection();
  best_move = root.record.best_move;
  if (root.record.proof_disproof.isCheckmateSuccess())
    proof_pieces = root.record.proofPieces();
  return root.record.proof_disproof;
}

const osl::checkmate::ProofDisproof 
osl::checkmate::
Dfpn::tryProof(NumEffectState& state, const HashKey& key,
	       const PathEncoding& path, const ProofOracle& oracle, size_t oracle_id, Move& best_move,
	       Move last_move)
{
  return tryProofMain<true>(state, key, path, oracle, oracle_id, best_move, last_move);
}
const osl::checkmate::ProofDisproof 
osl::checkmate::
Dfpn::tryProofLight(NumEffectState& state, const HashKey& key,
		    const PathEncoding& path, const ProofOracle& oracle, size_t oracle_id, Move& best_move,
		    Move last_move)
{
  return tryProofMain<false>(state, key, path, oracle, oracle_id, best_move, last_move);
}

template <bool UseTable>
const osl::checkmate::ProofDisproof 
osl::checkmate::
Dfpn::tryProofMain(NumEffectState& state, const HashKey& key,
		   const PathEncoding& path, const ProofOracle& oracle, size_t oracle_id, Move& best_move,
		   Move last_move)
{
  assert(table);
  if (! table)
    return ProofDisproof();
  path_table->clear();
  
  tree->state = state;
  node_count = tree->depth = 0;
  node_count_limit = 1000000;	// large enough

  Node& root = tree->node[0];
  root.hash_key = key;
  root.path = path;
  root.clear();
  root.threshold = ProofDisproof(ROOT_PROOF_TOL, ROOT_DISPROOF_TOL);
  root.white_stand = PieceStand(WHITE, state);
  root.moved = last_move;

  root.record = (state.getTurn() == BLACK) 
    ? table->probe<BLACK>(root.hash_key, root.white_stand)
    : table->probe<WHITE>(root.hash_key, root.white_stand);
  if (root.record.proof_disproof.isFinal() || root.record.tried_oracle > oracle_id) {
    best_move = root.record.best_move;
    return root.record.proof_disproof;
  }

  try {
    if (state.getTurn() == BLACK)
      proofOracleAttack<BLACK,UseTable>(oracle);
    else
      proofOracleAttack<WHITE,UseTable>(oracle);
  }
  catch (DepthLimitReached&) {
    return ProofDisproof();
  }
  if (UseTable && root.path_record
      && (std::find(root.path_record->twin_list.begin(), root.path_record->twin_list.end(), path)
	  != root.path_record->twin_list.end()))
    return ProofDisproof::LoopDetection();
  best_move = root.record.best_move;
  root.record.tried_oracle = oracle_id+1;
  if (UseTable) {
    root.record.last_move = last_move;
    table->store(root.hash_key, root.record);
  }
  return root.record.proof_disproof;
}


const osl::checkmate::ProofDisproof
osl::checkmate::
Dfpn::hasEscapeMove(NumEffectState& state, 
		    const HashKey& key, const PathEncoding& path, 
		    size_t limit, Move last_move)
{
  assert(table);
  if (! state.hasEffectBy(alt(state.getTurn()), state.getKingPosition(state.getTurn())))
    return ProofDisproof::NoCheckmate();
  if (! table)
    return ProofDisproof();
  path_table->clear();
  tree->state = state;
  node_count = tree->depth = 0;
  node_count_limit = limit;

  Node& root = tree->node[0];
  root.hash_key = key;
  root.path = path;
  root.moved = last_move;
  root.clear();
  root.threshold = ProofDisproof(ROOT_PROOF_TOL, ROOT_DISPROOF_TOL);
  root.white_stand = PieceStand(WHITE, state);
  
  try {
    if (state.getTurn() == BLACK)
      defense<WHITE>();
    else
      defense<BLACK>();

    if (root.record.need_full_width) {
      root.clear();
      if (state.getTurn() == BLACK)
	defense<WHITE>();
      else
	defense<BLACK>();
    }
  }
  catch (DepthLimitReached&) {
    return ProofDisproof();
  }
  if (root.record.proof_disproof == ProofDisproof::NoEscape()
      && last_move.isNormal() && last_move.isDrop() && last_move.ptype() == PAWN)
    return ProofDisproof::PawnCheckmate();
  if (root.path_record) {
    const SimpleTwinList& tl = root.path_record->twin_list;
    if (std::find(tl.begin(), tl.end(), root.path) != tl.end())
      return ProofDisproof::LoopDetection();
  }
  return root.record.proof_disproof;
}

namespace osl
{
  namespace 
  {
    typedef boost::tuple<int,int,int,int> tuple_t;
    template <Player Turn>
    struct move_compare
    {
      const NumEffectState *state;
      move_compare(const NumEffectState& s) : state(&s)
      {
	assert(Turn == state->getTurn());
      }
      tuple_t convert(Move m) const
      {
	const int a = state->countEffect(Turn, m.to()) + m.isDrop();
	const int d = state->countEffect(alt(Turn), m.to());
	return boost::make_tuple(a > d, m.isPromote(), m.to().index(), m.from().index());
      }
      bool operator()(Move l, Move r) const
      {
	return convert(l) > convert(r);
      }
    };
  }
}

void osl::checkmate::
Dfpn::sortAlternative(const NumEffectState& state, MoveVector& moves)
{
#ifdef MEMORIZE_SOLVED_IN_BITSET
  if (state.getTurn() == BLACK)
    std::sort(moves.begin(), moves.end(), move_compare<BLACK>(state));
  else
    std::sort(moves.begin(), moves.end(), move_compare<WHITE>(state));
#endif
}

template <osl::Player Turn>
void osl::checkmate::
Dfpn::sort(const NumEffectState& state, MoveVector& moves)
{
#ifdef MEMORIZE_SOLVED_IN_BITSET
  // std::sort(moves.begin(), moves.end());
  int last_sorted = 0, cur = 0; 
  Ptype last_ptype = PTYPE_EMPTY;
  for (;(size_t)cur < moves.size(); ++cur) {
    if (moves[cur].isDrop() || moves[cur].oldPtype() == last_ptype)
      continue;
    std::sort(moves.begin()+last_sorted, moves.begin()+cur, move_compare<Turn>(state));
    last_sorted = cur;
    last_ptype = moves[cur].oldPtype();
  }
  std::sort(moves.begin()+last_sorted, moves.begin()+cur, move_compare<Turn>(state));
#endif
}

template <osl::Player P> 
void osl::checkmate::
Dfpn::generateCheck(const NumEffectState& state, MoveVector& moves)
{
  assert(moves.empty());
  if (state.inCheck(P))
  {
    using namespace osl::move_classifier;
    MoveVector escape;
    move_action::Store store(escape);
    move_generator::Escape<P,move_action::Store>::
      generateKingEscape(state, store);
    for (size_t i=0; i<escape.size(); ++i)
    {
      if (MoveAdaptor<Check<P> >::isMember(state, escape[i]))
	moves.push_back(escape[i]);
    }
  }
  else
  {
    move_action::Store store(moves);
    typedef move_action::SafeFilter<P,move_action::Store> safe_filter_t;
    safe_filter_t safe_filter(state,store);
    move_generator::AddEffect<P,true>::generate
      (state,state.getKingPiece(alt(P)).position(),safe_filter);
  }
  sort<P>(state, moves);
}

// P Ϲ¦
template <osl::Player P> 
void osl::checkmate::
Dfpn::attack()
{
  assert(! tree->inCheck(alt(P)));
  Node& node = tree->node[tree->depth];
#if (! defined NDEBUG) && (! defined OSL_USE_RACE_DETECTOR)
  node.visit_time = ++timer;
#endif
  const int my_distance = tree->depth ? tree->node[tree->depth-1].path_record->distance+1 : node.path.getDepth();
  LoopToDominance loop;
  DfpnVisitLock<> lk(node.path_record = path_table->allocate<P>(node.hash_key, my_distance, loop));
  DfpnRecord& record = node.record;
  record = DfpnRecord();
  if (loop == BadAttackLoop) {
    node.setLoopDetection();
    return;
  }
  assert(node.white_stand == PieceStand(WHITE, tree->state));
  const size_t node_count_org = node_count++;
#ifndef CHECKMATE_D2
  if (! tree->inCheck(P)
      && ImmediateCheckmate::hasCheckmateMove<P>(tree->state, record.best_move)) {
    PieceStand proof_pieces;	// Note: ImmediateCheckmate ɬפʲȤʤȤ˰¸
    if (record.best_move.isDrop())
      proof_pieces.add(record.best_move.ptype());
    record.setProofPieces(proof_pieces);
    record.proof_disproof = ProofDisproof::Checkmate();
    return;
  }
#endif
  assert(tree->depth + 2 < MaxDepth);
  if (tree->depth + 2 >= MaxDepth)
    throw DepthLimitReached();
  record = table->probe<P>(node.hash_key, node.white_stand);
  assert(record.stands[BLACK] == node.hash_key.blackStand());
  assert(record.stands[WHITE] == node.white_stand);
  if (record.proof_disproof.isFinal())
    return;
  if (tree->depth == 0 && node_count_limit <= 50 && record.node_count >= node_count_limit)
    return;
  if (tree->depth == 0
#ifdef CHECKMATE_A3_GOLD
      || (record.proof_disproof == ProofDisproof(1,1) && tree->state.hasPieceOnStand<GOLD>(P)
	  && (tree->king(alt(P)).position().x() <= 3 || tree->king(alt(P)).position().x() >= 7
	      || tree->king(alt(P)).position().positionForBlack<P>().y() <= 3))
#endif
    )
  {
#ifdef DFPN_STAT
    static stat::Ratio oracle_success("a3-gold");
#endif
    FixedDepthSearcher fixed_searcher(tree->state);
    PieceStand proof_pieces;
    ProofDisproof pdp = fixed_searcher.hasCheckmateMove<P>(2, record.best_move, proof_pieces);
    ++node_count;
#ifdef DFPN_STAT
    oracle_success.add(pdp.isCheckmateSuccess());
#endif
    if (pdp.isCheckmateSuccess()) {
      record.node_count++;
      record.proof_disproof = pdp;
      record.setProofPieces(proof_pieces);
      record.last_move = node.moved;
      table->store(node.hash_key, record);
      return;
    }
  }
  // move generation
  generateCheck<P>(tree->state, node.moves);
  if (node.moves.empty()) {
    record.setDisproofPieces(DisproofPieces::leaf(tree->state, alt(P), 
						  record.stands[alt(P)]));
    record.proof_disproof = ProofDisproof::NoCheckmate();
    return;
  }
  // probe all
#ifdef PROOF_AVERAGE
  int frontier_count = 0, sum_frontier_proof = 0;
#endif
  assert(node.children.empty());
  {
    node.allocate(node.moves.size());
    const King8Info info_modified 
      = Edge_Table.resetEdgeFromLiberty(alt(P), tree->king(alt(P)).position(), tree->state.king8Info(alt(P)));
    for (size_t i=0; i<node.moves.size(); ++i) {
#ifdef MEMORIZE_SOLVED_IN_BITSET
      if (record.solved & (1ull << i))
	continue;
#endif
      const HashKey& new_key = node.hashes[i] = node.hash_key.newHashWithMove(node.moves[i]);
      node.children[i] = table->probe<P>(new_key, node.nextWhiteStand(P, node.moves[i]));
      if (node.children[i].proof_disproof == ProofDisproof(1,1)) {
	unsigned int proof, disproof;
	LibertyEstimator::attackH(P, tree->state, info_modified, 
				  node.moves[i], proof, disproof);
	node.children[i].proof_disproof = ProofDisproof(proof, disproof);
      }
#ifdef PROOF_AVERAGE
      if (node.children[i].node_count == 0) {
	++frontier_count;
	sum_frontier_proof += node.children[i].proof();
      }
#endif
      if (node.children[i].proof_disproof == ProofDisproof::NoEscape()
	  && node.moves[i].isDrop() && node.moves[i].ptype() == PAWN) {
	node.children[i].proof_disproof = ProofDisproof::PawnCheckmate();
#ifdef MEMORIZE_SOLVED_IN_BITSET
	record.solved |= (1ull << i);
#endif
	record.min_pdp = std::min(record.min_pdp, (unsigned int)ProofDisproof::PAWN_CHECK_MATE_PROOF);
      }
      else if (node.children[i].proof_disproof.isCheckmateFail())
	tree->setNoCheckmateChildInAttack(i);
      else if (node.children[i].proof_disproof.isCheckmateSuccess()) {
	record.node_count += node_count - node_count_org;
	node.setCheckmateAttack(P,i);
	record.last_move = node.moved;
	table->store(node.hash_key, record);
	return;
      }      
      node.children_path[i] = path_table->probe(new_key);
      node.proof_cost[i] = attackProofCost(P, tree->state, node.moves[i]);
    }
  }

  assert(node.children.size() == node.moves.size());
#ifdef PROOF_AVERAGE
  const size_t proof_average = frontier_count ? sum_frontier_proof/frontier_count : 1;
#else
  const size_t proof_average = 1;
#endif
  // main loop
  for (int loop=0; true; ++loop) {
    unsigned int min_proof=record.min_pdp, min_proof2=record.min_pdp;
    size_t sum_disproof = 0, max_disproof = 0, next_i=node.children.size();
    size_t max_drop_disproof_rook = 0, max_drop_disproof_bishop = 0, max_drop_disproof_lance = 0;
    int max_children_depth = 0, upward_count = 0;
    for (size_t i=0; i<node.children.size(); ++i) {
#ifdef MEMORIZE_SOLVED_IN_BITSET
      if (record.solved & (1ull << i))
	continue;
#endif
      size_t proof = node.children[i].proof();
      size_t disproof = node.children[i].disproof();
      if (node.children_path[i]) {	
	if (node.isLoop(i)) {
	  node.children[i].proof_disproof = ProofDisproof::LoopDetection();
	  assert(proof < ProofDisproof::LOOP_DETECTION_PROOF);
	  proof = ProofDisproof::LOOP_DETECTION_PROOF;
	  disproof = 0;
	} 
	else if (! node.children[i].proof_disproof.isFinal()) {
	  max_children_depth = std::max(max_children_depth, node.children_path[i]->distance);
#ifdef DELAY_UPWARD
	  if (node.children_path[i]->distance <= node.path_record->distance) {
	    max_disproof = std::max(max_disproof, disproof);
	    ++upward_count;
	    disproof = UpwardWeight;
	    // disproof = 0;
	  }
	  else
#endif
	    if (node.moves[i].isDrop())
	    {
	      const EffectContent e
		= Ptype_Table.getEffect(node.moves[i].ptypeO(),
					Offset32(tree->king(alt(P)).position(), node.moves[i].to()));
	      if (! e.hasUnblockableEffect()) {
		size_t *target = &max_drop_disproof_lance;
		if (node.moves[i].ptype() == ROOK)
		  target = &max_drop_disproof_rook;
		else if (node.moves[i].ptype() == BISHOP)
		  target = &max_drop_disproof_bishop;
		*target = std::max(*target, disproof);
		disproof = LongDropCount;
	      }
	    }
	} // ! isFinal
      }
      else {
	max_children_depth = node.path_record->distance+1;
      }
      if (proof && disproof) 
	proof += node.proof_cost[i];
      if (proof < min_proof || (proof == min_proof && disproof && disproof < node.children[next_i].disproof())) {
	min_proof2 = min_proof;
	min_proof = proof;
	next_i = i;
      } else if (proof < min_proof2) {
	min_proof2 = proof;
      }
      sum_disproof += disproof;
    }
    sum_disproof += max_drop_disproof_rook + max_drop_disproof_bishop + max_drop_disproof_lance;
    if (LongDropCount) {
      if (max_drop_disproof_rook)   sum_disproof -= LongDropCount;
      if (max_drop_disproof_bishop) sum_disproof -= LongDropCount;
      if (max_drop_disproof_lance)  sum_disproof -= LongDropCount;
    }
    if (upward_count) {
      // sum_disproof += upward_count*UpwardWeight;
      sum_disproof = std::max(sum_disproof, max_disproof);
    }
    if (node.path_record->distance >= max_children_depth) {
      node.path_record->distance = max_children_depth-1;
    }
    if (min_proof >= node.threshold.proof()
	|| sum_disproof >= node.threshold.disproof()
	|| next_i >= node.children.size()
	|| node_count + min_proof >= node_count_limit) {
      record.proof_disproof = ProofDisproof(min_proof, sum_disproof);
      if (record.proof_disproof.isLoopDetection())
	node.setLoopDetection();
      else if (record.proof_disproof.isCheckmateFail()) {
	node.setNoCheckmateAttack(P, tree->state);
      }
      record.node_count += node_count - node_count_org;
      table->store(node.hash_key, record);
      return;
    }
#ifdef MEMORIZE_SOLVED_IN_BITSET
    assert(! (record.solved & (1ull << next_i)));
#endif
    record.best_move = node.moves[next_i];
    tree->newVisit(P, node.moves[next_i], node.hashes[next_i]);
    Node& next = tree->node[tree->depth+1];
    next.threshold = ProofDisproof(std::min(min_proof2+proof_average, (size_t)node.threshold.proof())
				   - node.proof_cost[next_i], 
				   node.threshold.disproof()
				   - (sum_disproof - node.children[next_i].disproof()));
    CallDefense<P> helper(this);
    tree->depth += 1;
    ApplyMoveWithPath<P>::doUndoMove(tree->state, next.path, next.moved, helper);
    tree->depth -= 1;

    node.children[next_i] = next.record;
    node.children_path[next_i] = next.path_record;
    if (next.record.proof_disproof == ProofDisproof::NoEscape()
	&& next.moved.isDrop() && next.moved.ptype() == PAWN)
      node.children[next_i].proof_disproof = ProofDisproof::PawnCheckmate();
    if (node.children[next_i].proof_disproof.isCheckmateSuccess()) {
      node.setCheckmateAttack(P,next_i);
      record.node_count += node_count - node_count_org;
      record.last_move = node.moved;
      table->store(node.hash_key, record);
      return;
    }      
    else if (next.record.proof_disproof.isCheckmateFail()
	     && ! next.record.proof_disproof.isLoopDetection())
      tree->setNoCheckmateChildInAttack(next_i);
    min_proof = std::min(min_proof2, node.children[next_i].proof());
    if (min_proof < ProofDisproof::PROOF_LIMIT
	&& node_count + min_proof >= node_count_limit) {
      record.proof_disproof = ProofDisproof(min_proof, sum_disproof);
      record.node_count += node_count - node_count_org;
      table->store(node.hash_key, record);
      return;
    }
  }
}

template <osl::Player P> 
void osl::checkmate::
Dfpn::generateEscape(const NumEffectState& state, bool need_full_width, 
		     Position last_to, MoveVector& moves)
{
  assert(moves.empty());
#ifdef GRAND_PARENT_DELAY
  const bool delay_node = last_to != Position() 
      && state.hasEffectBy(alt(P), last_to)
      && (state.hasEffectNotBy(alt(P), state.getKingPiece(alt(P)), last_to)
	  || ! state.hasEffectBy(P, last_to));
  if (delay_node)
  {
    MoveVector all;
    move_action::Store store(all);
    move_generator::Escape<PlayerTraits<P>::opponent,move_action::Store,true>::
      generateKingEscape(state, store);
    for (size_t i=0; i<all.size(); ++i) {
      if (all[i].to() == last_to) {
	moves.push_back(all[i]);
      }
    }
#ifdef MEMORIZE_SOLVED_IN_BITSET
    sort<PlayerTraits<P>::opponent>(state, moves);
#endif
  }
  else 
#endif
  {
    move_action::Store store(moves);
    move_generator::Escape<PlayerTraits<P>::opponent,move_action::Store,true>::
      generateKingEscape(state, store);
#ifdef MEMORIZE_SOLVED_IN_BITSET
    sort<PlayerTraits<P>::opponent>(state, moves);
#endif
  }

  if (need_full_width) {
    MoveVector others;
    move_action::Store store(others);
    move_generator::Escape<PlayerTraits<P>::opponent,move_action::Store,false>::
      generateKingEscape(state, store);
#ifdef MEMORIZE_SOLVED_IN_BITSET
    sort<PlayerTraits<P>::opponent>(state, others);
#endif
    const int org_size = moves.size();
    for (size_t i=0; i<others.size(); ++i) {
      if (std::find(moves.begin(), moves.begin()+org_size, others[i]) == moves.begin()+org_size)
	moves.push_back(others[i]);
    }
  }
  // TODO: ͤᲦ
}

bool osl::checkmate::
Dfpn::grandParentSimulationSuitable() const
{
#ifdef GRAND_PARENT_SIMULATION
  Node& node = tree->node[tree->depth];
  if (tree->depth >= 2) {
    const Node& parent = tree->node[tree->depth-1];
    const Node& gparent = tree->node[tree->depth-2];
    const Move alm = node.moved; // attacker's last move
    const Move dlm = parent.moved; // defense's last move
    const Move alm2 = gparent.moved; // attacker's second-last move
    if (dlm.isNormal() && alm.to() == dlm.to() && dlm.capturePtype() == PTYPE_EMPTY
	&& alm2.isNormal() && alm2.to() == alm.from()) {
      return true;
    }
  }
#endif
  return false;
}

template <osl::Player P> 
void osl::checkmate::
Dfpn::defense()
{
  Node& node = tree->node[tree->depth];
#if (! defined NDEBUG) && (! defined OSL_USE_RACE_DETECTOR)
  node.visit_time = ++timer;
#endif
  const int my_distance = tree->depth ? tree->node[tree->depth-1].path_record->distance+1 : node.path.getDepth();
  LoopToDominance loop;
  DfpnVisitLock<> lk(node.path_record = path_table->allocate<P>(node.hash_key, my_distance, loop));
  DfpnRecord& record = node.record;
  if (loop == BadAttackLoop) {
    record = DfpnRecord();
    node.setLoopDetection();
    return;
  }
  const size_t node_count_org = node_count++;
  assert(tree->inCheck(alt(P)));
  assert(node.white_stand == PieceStand(WHITE, tree->state));

  record = table->probe<P>(node.hash_key, node.white_stand);
  assert(record.stands[BLACK] == node.hash_key.blackStand());
  assert(record.stands[WHITE] == node.white_stand);
  if (record.proof_disproof.isFinal())
    return;
  const bool grand_parent_simulation = grandParentSimulationSuitable();
  if (record.last_to == Position())
    record.last_to = grand_parent_simulation ? node.moved.to() : tree->king(alt(P)).position();
  const Position grand_parent_delay_last_to
    = (record.last_to != tree->king(alt(P)).position()) ? record.last_to : Position();

  generateEscape<P>(tree->state, record.need_full_width, grand_parent_delay_last_to, node.moves);
  if (node.moves.empty() && ! record.need_full_width) {
    record.need_full_width = true;
    generateEscape<P>(tree->state, record.need_full_width, grand_parent_delay_last_to, node.moves);
  }
  if (node.moves.empty()) {
    record.setProofPieces(ProofPieces::leaf(tree->state, P, record.stands[P]));
    record.proof_disproof = ProofDisproof::NoEscape();
    return;
  }
  // probe all
#ifdef DISPROOF_AVERAGE
  int frontier_count = 0, sum_frontier_disproof = 0;
#endif
  assert(node.children.empty());
  {
    node.allocate(node.moves.size());
    for (size_t i=0;i <node.moves.size(); ++i) {
#ifdef MEMORIZE_SOLVED_IN_BITSET
      if (record.solved & (1ull << i))
	continue;
#endif
      const HashKey& new_key = node.hashes[i] = node.hash_key.newHashWithMove(node.moves[i]);
      node.children[i] = table->probe<P>(new_key, node.nextWhiteStand(alt(P), node.moves[i]));
      if (node.children[i].proof_disproof.isCheckmateSuccess()) {
	node.setCheckmateChildInDefense(i);	
      }
#ifdef CHECKMATE_D2
      else if (node.children[i].proof_disproof == ProofDisproof(1,1)) {
	FixedDepthSearcher fixed_searcher(tree->state);
	PieceStand proof_pieces;
	Move check_move;
	node.children[i].proof_disproof
	  = fixed_searcher.hasEscapeByMove<P>(node.moves[i], 0, check_move, proof_pieces);
	++node_count;
	if (node.children[i].proof_disproof.isCheckmateSuccess()) {	  
	  node.children[i].best_move = check_move;
	  node.children[i].setProofPieces(proof_pieces);
	  node.children[i].node_count++;
	  node.setCheckmateChildInDefense(i);
	}
	else {
	  if (node.children[i].proof_disproof.isCheckmateFail()) {
	    node.children[i].proof_disproof = ProofDisproof(1,1);
	    if (i) {
	      node.moves[0] = node.moves[i];
	      node.children[0] = node.children[i];
	      node.children_path[0] = node.children_path[i];
	      const int old_size = (int)node.moves.size();
	      for (int j=1; j<old_size; ++j) {
		node.moves.pop_back();
		node.children.pop_back();
		node.children_path.pop_back();
	      }
	    }
	    break;
	  }
#ifdef DISPROOF_AVERAGE
	  ++frontier_count;
	  sum_frontier_disproof += node.children[i].proof_disproof.disproof();
#endif
	}
	// ++node_count;
      }
#endif
      if (node.children[i].proof_disproof.isCheckmateFail()) {
	tree->setNoCheckmateDefense(P, i);
	table->store(node.hash_key, record);
	return;
      }      
      node.children_path[i] = path_table->probe(new_key);
      if (node.isLoop(i)) {
	node.setLoopDetection();
	return;
      }
#ifdef GRAND_PARENT_SIMULATION
      if (grand_parent_simulation && node.children[i].proof_disproof == ProofDisproof(1,1)) {
	const Node& gparent = tree->node[tree->depth-2];
	size_t gi=std::find(gparent.moves.begin(), gparent.moves.end(), node.moves[i]) - gparent.moves.begin();
	if (gi < gparent.moves.size() 
	    && (
#ifdef MEMORIZE_SOLVED_IN_BITSET
	      (gparent.record.solved & (1ull<<gi))
	      || 
#endif
	      gparent.children[gi].proof_disproof.isCheckmateSuccess())) {
	  grandParentSimulation<P>(i, gparent, gi);
	  if (node.children[i].proof_disproof.isCheckmateSuccess())
	    node.setCheckmateChildInDefense(i);
	}
      }
#endif
    }
    if (record.need_full_width==1) {
      record.need_full_width++;
      for (size_t i=0;i <node.moves.size(); ++i) {
	if ((
#ifdef MEMORIZE_SOLVED_IN_BITSET
	      (record.solved & (1ull<<i))
	      || 
#endif
	     (i >= 64 && node.children[i].proof_disproof.isCheckmateSuccess()))
	    && node.moves[i].isDrop()) {
	  blockingSimulation<P>(i, ProofOracle(node.hash_key.newHashWithMove(node.moves[i]), 
					       node.nextWhiteStand(alt(P), node.moves[i])));
	}
      }
    }
  }
  assert(node.children.size() == node.moves.size());
#ifdef DISPROOF_AVERAGE
  const size_t disproof_average = frontier_count ? sum_frontier_disproof/frontier_count : 1;
#else
  const size_t disproof_average = 1;
#endif
  // main loop
  for (int loop=0; true; ++loop) {
    unsigned int min_disproof=record.min_pdp, min_disproof2=record.min_pdp;
    size_t sum_proof = 0, max_upward_proof = 0, max_drop_proof = 0, next_i=node.children.size();
    int max_children_depth = 0, upward_count = 0;
#ifdef KAKINOKI_FALSE_BRANCH_SEARCH
    size_t max_proof = 0;
    bool false_branch_candidate = !record.false_branch;
#endif
    for (size_t i=0; i<node.children.size(); ++i) {
#ifdef MEMORIZE_SOLVED_IN_BITSET
      if (record.solved & (1ull << i))
	continue;
#endif
      size_t disproof = node.children[i].disproof();
      size_t proof = node.children[i].proof();
      if (node.children_path[i]) {
	if (node.isLoop(i)) {
	  node.setLoopDetection();
	  return;
	}
	if (! node.children[i].proof_disproof.isFinal()) {
	  max_children_depth = std::max(max_children_depth, node.children_path[i]->distance);
#ifdef DELAY_UPWARD
	  if (node.children_path[i]->distance <= node.path_record->distance) {
	    max_upward_proof = std::max(max_upward_proof , proof);
	    ++upward_count;
	    proof = UpwardWeight;
	    // proof = 0;
	  }
	  else
#endif
	    if (node.moves[i].isDrop() && !tree->state.hasEffectBy(alt(P), node.moves[i].to())) {
	      max_drop_proof = std::max(max_drop_proof, proof);
	      proof = SacrificeBlockCount;
	    }
	}
      }
      else {
	max_children_depth = node.path_record->distance+1;
      }
      if (disproof < min_disproof
	  || (disproof == min_disproof && proof && proof < node.children[next_i].proof())) {
	min_disproof2 = min_disproof;
	min_disproof = disproof;
	next_i = i;
      } else if (disproof < min_disproof2) {
	min_disproof2 = disproof;
      }
#ifdef KAKINOKI_FALSE_BRANCH_SEARCH
      if (false_branch_candidate && ! node.children[i].proof_disproof.isFinal()
	  && (node.children[i].node_count == 0
	      || ! node.children[i].best_move.isNormal()
	      || ! (node.moves[i].ptype() == KING && node.moves[i].capturePtype() == PTYPE_EMPTY)))
	false_branch_candidate = false;
      max_proof = std::max(max_proof, proof);
#endif
      sum_proof += proof;
    }
#ifdef KAKINOKI_FALSE_BRANCH_SEARCH
    if (false_branch_candidate) {
      record.false_branch = true;
      HashKey goal;
      for (size_t i=0; i<node.children.size(); ++i) {
	if (
#ifdef MEMORIZE_SOLVED_IN_BITSET
	  (record.solved & (1ull << i))
	  || 
#endif
	  node.children[i].proof_disproof.isFinal())
	  continue;
	HashKey key = node.hashes[i];
	key = key.newHashWithMove(node.children[i].best_move);
	if (goal == HashKey()) {
	  goal = key;
	  continue;
	}
	if (goal != key) {
	  record.false_branch = false;
	  break;
	}
      }
    }
    if (record.false_branch)
      sum_proof = max_proof;
#endif
    sum_proof += max_drop_proof;
    if (SacrificeBlockCount && max_drop_proof) 
      sum_proof -= SacrificeBlockCount;
    if (upward_count) {
      // sum_proof += upward_count*UpwardWeight;
      sum_proof = std::max(sum_proof, max_upward_proof);
    }
    if (node.path_record->distance >= max_children_depth) {
      node.path_record->distance = max_children_depth-1;
    }
    if (min_disproof >= node.threshold.disproof()
	|| sum_proof >= node.threshold.proof()
	|| next_i >= node.children.size()
	|| node_count + sum_proof >= node_count_limit) {
      record.proof_disproof = ProofDisproof(sum_proof, min_disproof);
      if (record.proof_disproof.isLoopDetection())
	node.setLoopDetection();
      else if (record.proof_disproof.isCheckmateSuccess()) {
	if (blocking_verify && ! record.need_full_width) {
	  record.need_full_width = 1;
	  record.proof_disproof = ProofDisproof(1,1);
	  table->store(node.hash_key, record);
	  return;
	}
	node.setCheckmateDefense(P, tree->state);
      }
      record.node_count += node_count - node_count_org;
      if (record.proof_disproof.isFinal() || record.node_count >= min_record_node_count) {
	table->store(node.hash_key, record);
      }
      return;
    }
#ifdef MEMORIZE_SOLVED_IN_BITSET
    assert(! (record.solved & (1ull << next_i)));
#endif
    tree->newVisit(alt(P), node.moves[next_i], node.hashes[next_i]);
    Node& next = tree->node[tree->depth+1];
    next.threshold = ProofDisproof(node.threshold.proof()
				   - (sum_proof - node.children[next_i].proof()),
				   std::min(min_disproof2+disproof_average,
					    (size_t)node.threshold.disproof()));
    CallAttack<P> helper(this);
    tree->depth += 1;
    ApplyMoveWithPath<PlayerTraits<P>::opponent>::doUndoMove(tree->state, next.path, node.moves[next_i], helper);
    tree->depth -= 1;

    node.children[next_i] = next.record;
    node.children_path[next_i] = next.path_record;
    if (next.record.proof_disproof.isCheckmateFail()) {
      if (record.proof_disproof.isLoopDetection())
	node.setLoopDetection();
      else
	tree->setNoCheckmateDefense(P, next_i);
      record.node_count += node_count - node_count_org;
      table->store(node.hash_key, record);
      return;
    }      
    if (next.record.proof_disproof.isCheckmateSuccess())
      node.setCheckmateChildInDefense(next_i);
    if (node_count >= node_count_limit) {
      record.proof_disproof = ProofDisproof(sum_proof, min_disproof);
      record.node_count += node_count - node_count_org;
      if (record.node_count >= min_record_node_count)
	table->store(node.hash_key, record);
      return;
    }
    if (next.moved.isDrop() && next.record.proof_disproof.isCheckmateSuccess()) {
      blockingSimulation<P>(next_i, ProofOracle(next.hash_key, next.white_stand));
    }
  }
}

/* ------------------------------------------------------------------------- */
template <osl::Player P, bool UseTable>
struct osl::checkmate::Dfpn::CallProofOracleAttack
{
  Dfpn *search;
  ProofOracle oracle;
  CallProofOracleAttack(Dfpn *s, const ProofOracle& o) : search(s), oracle(o)
  {
  }
  void operator()(Position) const 
  {
    search->proofOracleAttack<P,UseTable>(oracle);
  }
};

template <osl::Player P, bool UseTable>
struct osl::checkmate::Dfpn::CallProofOracleDefense
{
  Dfpn *search;
  ProofOracle oracle;
  CallProofOracleDefense(Dfpn *s, const ProofOracle& o) : search(s), oracle(o)
  {
  }
  void operator()(Position) const 
  {
    search->proofOracleDefense<P,UseTable>(oracle);
  }
};

template <osl::Player P, bool UseTable> 
void osl::checkmate::
Dfpn::proofOracleAttack(const ProofOracle& key)
{
  assert(! tree->inCheck(alt(P)));
  const int my_distance = (UseTable && tree->depth) ? (tree->node[tree->depth-1].path_record->distance+1) : 0;
  Node& node = tree->node[tree->depth];
  DfpnRecord& record = node.record;
  LoopToDominance loop;
  DfpnVisitLock<UseTable> lk((node.path_record = (UseTable ? path_table->allocate<P>(node.hash_key, my_distance, loop) : 0)));
  if (UseTable && loop == BadAttackLoop) {
    record = DfpnRecord();
    node.setLoopDetection();
    return;
  }
  assert(node.white_stand == PieceStand(WHITE, tree->state));
  const size_t node_count_org = node_count++;

  assert(tree->depth + 2 < MaxDepth);
  if (tree->depth + 2 >= MaxDepth)
    throw DepthLimitReached();
  record = table->probe<P>(node.hash_key, node.white_stand);
  if (record.proof_disproof.isFinal())
    return;
#ifdef CEHCKMATE_A3_SIMULLATION  
  if (record.node_count == 0)
  {
#ifdef DFPN_STAT
    static stat::Ratio oracle_success("a3-simulation");
#endif
    FixedDepthSearcher fixed_searcher(tree->state);
    PieceStand proof_pieces;
    ProofDisproof pdp = fixed_searcher.hasCheckmateMove<P>(2, record.best_move, proof_pieces);
    ++node_count;
#ifdef DFPN_STAT
    oracle_success.add(pdp.isCheckmateSuccess());
#endif
    if (pdp.isCheckmateSuccess()) {
      record.proof_disproof = pdp;
      record.setProofPieces(proof_pieces);
      record.node_count++;
      return;
    }
  }
#endif
#ifndef NDEBUG
  if (tree->depth > 1000) {
    std::cerr << tree->state;
    node.hash_key.dumpContents(std::cerr);
    std::cerr << "\n";
    table->showProofOracles<P>(key.key, key.white_stand, node.moved);
  }
#endif
  DfpnRecord oracle = table->findProofOracle<P>(key.key, key.white_stand, node.moved);
  if (! oracle.proof_disproof.isCheckmateSuccess() || ! oracle.best_move.isNormal())
    return;
  const Move check_move = OracleAdjust::attack(tree->state, oracle.best_move);
  if (! check_move.isNormal() || ! key.traceable(P, check_move))
    return;
  
  node.allocate(1);
  node.moves.clear();
  node.moves.push_back(check_move);
  const HashKey new_key = node.hash_key.newHashWithMove(node.moves[0]);
  if (UseTable) {
    node.children[0] = table->probe<P>(new_key, node.nextWhiteStand(P, node.moves[0]));
    node.children_path[0] = path_table->probe(new_key);
    if (node.isLoop(0))
      return;
  } else {
    node.children[0] = DfpnRecord();
    node.children_path[0] = 0;
  }

  if (! UseTable || ! node.children[0].proof_disproof.isFinal()) {
    Node& next = tree->node[tree->depth+1];
    tree->newVisit(P, node.moves[0], new_key);

    CallProofOracleDefense<P,UseTable> helper(this, key.newOracle(P, check_move));
    tree->depth += 1;
    ApplyMoveWithPath<P>::doUndoMove(tree->state, next.path, next.moved, helper);
    tree->depth -= 1;
    node.children[0] = next.record;
    node.children_path[0] = next.path_record;

    if (next.record.proof_disproof == ProofDisproof::NoEscape()
	&& next.moved.isDrop() && next.moved.ptype() == PAWN)
      node.children[0].proof_disproof = ProofDisproof::PawnCheckmate();
  }  
  if (node.children[0].proof_disproof.isCheckmateSuccess()) {
    node.setCheckmateAttack(P,0);
    record.node_count += node_count - node_count_org;
    if (UseTable) {
      record.last_move = node.moved;
      table->store(node.hash_key, record);
    }
  }
}

template <osl::Player P, bool UseTable> 
void osl::checkmate::
Dfpn::proofOracleDefense(const ProofOracle& key)
{
  const int my_distance = (UseTable && tree->depth) ? (tree->node[tree->depth-1].path_record->distance+1) : 0;
  Node& node = tree->node[tree->depth];
  LoopToDominance loop;
  DfpnVisitLock<UseTable> lk((node.path_record = (UseTable ? path_table->allocate<P>(node.hash_key, my_distance, loop) : 0)));
  DfpnRecord& record = node.record;
  if (UseTable && loop == BadAttackLoop) {
    record = DfpnRecord();
    node.setLoopDetection();
    return;
  }
  if (! UseTable && tree->depth >= 4) {
    if (tree->node[tree->depth-4].hash_key == node.hash_key
	|| (tree->depth >= 6 && tree->node[tree->depth-6].hash_key == node.hash_key)) {
      record = DfpnRecord();
      return;
    }
  }
  const size_t node_count_org = node_count++;
  assert(node.white_stand == PieceStand(WHITE, tree->state));
  if (! tree->inCheck(alt(P)) || tree->inCheck(P)) {
    record = DfpnRecord();
    record.proof_disproof = ProofDisproof::NoCheckmate();
    return;
  }

  record = table->probe<P>(node.hash_key, node.white_stand);
  if (record.proof_disproof.isFinal())
    return;
  // move generation
  const bool grand_parent_simulation = grandParentSimulationSuitable();
  if (record.last_to == Position())
    record.last_to = grand_parent_simulation ? node.moved.to() : tree->king(alt(P)).position();
  const Position grand_parent_delay_last_to
    = (record.last_to != tree->king(alt(P)).position()) ? record.last_to : Position();
  generateEscape<P>(tree->state, true, grand_parent_delay_last_to, node.moves);
  if (node.moves.empty()) {
    record.setProofPieces(ProofPieces::leaf(tree->state, P, record.stands[P]));
    record.proof_disproof = ProofDisproof::NoEscape();
    return;
  }
  DfpnRecord oracle = table->findProofOracle<P>(key.key, key.white_stand);
  if (! oracle.proof_disproof.isCheckmateSuccess())
    return;

  // probe all
  assert(node.children.empty());
  {
    node.allocate(node.moves.size());
    for (size_t i=0;i <node.moves.size(); ++i) {
#ifdef MEMORIZE_SOLVED_IN_BITSET
      if (record.solved & (1ull << i))
	continue;
#endif
      const HashKey& new_key = node.hashes[i] = node.hash_key.newHashWithMove(node.moves[i]);
      node.children[i] = UseTable ? table->probe<P>(new_key, node.nextWhiteStand(alt(P), node.moves[i])) : DfpnRecord();
#ifdef CHECKMATE_D2
      if (node.record.node_count == 0 && node.children[i].node_count == 0) {
	FixedDepthSearcher fixed_searcher(tree->state);
	PieceStand proof_pieces;
	Move check_move;
	DfpnRecord old_child = node.children[i];
	node.children[i].proof_disproof
	  = fixed_searcher.hasEscapeByMove<P>(node.moves[i], 0, check_move, proof_pieces);
	if (node.children[i].proof_disproof.isCheckmateSuccess()) {	  
	  node.children[i].best_move = check_move;
	  node.children[i].setProofPieces(proof_pieces);
	}
	else {
	  if (node.children[i].proof_disproof.isCheckmateFail())
	    node.children[i].proof_disproof = ProofDisproof(1,1);
	}
	++node_count;
      }
#endif
      if (node.children[i].proof_disproof.isCheckmateFail()) {
	tree->setNoCheckmateDefense(P, i);
	if (UseTable)
	  table->store(node.hash_key, record);
	return;
      }      
      node.children_path[i] = UseTable ? path_table->probe(new_key) : 0;
    }
  }
  assert(node.children.size() == node.moves.size());
  if (UseTable) {
    for (size_t i=0; i<node.children.size(); ++i) {
      if (node.isLoop(i)) {
	node.setLoopDetection();
	return;
      }
    }
  }
  unsigned int sum_proof=0, min_disproof=record.min_pdp;
  int num_proof = 0;
  for (size_t next_i=0; next_i<node.children.size(); ++next_i) {
#ifdef MEMORIZE_SOLVED_IN_BITSET
    if (record.solved & (1ull << next_i))
      continue;
#endif
    if (node.children[next_i].proof_disproof.isCheckmateSuccess()) {
      min_disproof = std::min(min_disproof, node.children[next_i].disproof());
      continue;
    }
    if (! key.traceable(alt(P), node.moves[next_i])) {
      ++sum_proof;
      min_disproof = 1;
      if (! UseTable)
	break;
      continue;
    }
    assert(! node.isLoop(next_i));
    Node& next = tree->node[tree->depth+1];
#ifdef MEMORIZE_SOLVED_IN_BITSET
    assert(! (record.solved & (1ull << next_i)));
#endif
    tree->newVisit(alt(P), node.moves[next_i], node.hashes[next_i]);

    CallProofOracleAttack<P,UseTable> helper(this, key.newOracle(alt(P), node.moves[next_i]));
    tree->depth += 1;
    ApplyMoveWithPath<PlayerTraits<P>::opponent>::doUndoMove(tree->state, next.path, node.moves[next_i], helper);
    tree->depth -= 1;

    node.children[next_i] = next.record;
    node.children_path[next_i] = next.path_record;
    if (next.record.proof_disproof.isCheckmateFail()) {
      if (record.proof_disproof.isLoopDetection())
	node.setLoopDetection();
      else
	tree->setNoCheckmateDefense(P, next_i);
      record.node_count += node_count - node_count_org;
      if (UseTable)
	table->store(node.hash_key, record);
      return;
    }
    if (next.record.proof_disproof.isCheckmateSuccess()) {
      node.setCheckmateChildInDefense(next_i);
      ++num_proof;
    }
    sum_proof += next.record.proof();
    min_disproof = std::min(min_disproof, next.record.disproof());
    if (sum_proof && ! UseTable)
      break;
  }
  if (sum_proof == 0) {
    node.record.proof_disproof = ProofDisproof(sum_proof, min_disproof);
    node.setCheckmateDefense(P, tree->state);
  }
}

template <osl::Player P> 
void osl::checkmate::
Dfpn::blockingSimulation(int oracle_i, const ProofOracle& oracle)
{
#ifdef DFPN_STAT
  static stat::Ratio oracle_success("blocking proof");
#endif
  Node& node = tree->node[tree->depth];
  Node& next = tree->node[tree->depth+1];
  const Move oracle_move = node.moves[oracle_i];
  const Position to = oracle_move.to();
  for (size_t i=0; i<node.moves.size(); ++i) {
#ifdef MEMORIZE_SOLVED_IN_BITSET
    if (node.record.solved & (1ull << i))
      continue;
#endif
    if (node.isLoop(i))
      break;
    if (node.children[i].proof_disproof.isFinal() || node.moves[i].to() != to)
      continue;
    if (! oracle.traceable(alt(P), node.moves[i]))
      continue;
#ifdef MEMORIZE_SOLVED_IN_BITSET
    assert(! (node.record.solved & (1ull << i)));
#endif
    tree->newVisit(alt(P), node.moves[i], node.hashes[i]);
    CallProofOracleAttack<P,true> helper(this, oracle);

    tree->depth += 1;
    ApplyMoveWithPath<PlayerTraits<P>::opponent>::doUndoMove(tree->state, next.path, node.moves[i], helper);
    tree->depth -= 1;

    node.children[i] = next.record;
    node.children_path[i] = next.path_record;    

#ifdef DFPN_STAT
    oracle_success.add(next.record.proof_disproof.isCheckmateSuccess());
#endif
  }
}

template <osl::Player P> 
void osl::checkmate::
Dfpn::grandParentSimulation(int cur_i, const Node& gparent, int gp_i)
{
#ifdef DFPN_STAT
  static stat::Ratio oracle_success("grandparent proof", true);
#endif
  Node& node = tree->node[tree->depth];
  Node& next = tree->node[tree->depth+1];

  const Move move = gparent.moves[gp_i];
  assert(move == node.moves[cur_i]);
  const HashKey& oracle_hash = (gparent.record.solved & (1ull << gp_i)) 
    ? gparent.hash_key.newHashWithMove(move)
    : gparent.hashes[gp_i];
  const ProofOracle oracle(oracle_hash, gparent.nextWhiteStand(alt(P), move));

  tree->newVisit(alt(P), node.moves[cur_i], node.hashes[cur_i]);
  CallProofOracleAttack<P,true> helper(this, oracle);

  tree->depth += 1;
  ApplyMoveWithPath<PlayerTraits<P>::opponent>::doUndoMove(tree->state, next.path, move, helper);
  tree->depth -= 1;

  node.children[cur_i] = next.record;
  node.children_path[cur_i] = next.path_record;    
#ifdef DFPN_STAT
  oracle_success.add(next.record.proof_disproof.isCheckmateSuccess());
#endif
}

// debug
int osl::checkmate::
Dfpn::distance(const HashKey& key) const
{
  const DfpnPathRecord *record = path_table->probe(key);
  if (record)
    return record->distance;
  return -1;
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
