#include "osl/eval/experimental/testEval.h"
#include "osl/eval/experimental/piecePair.h"
#include "osl/eval/experimental/kingTable.h"
#include "osl/eval/experimental/majorPiece.h"
#include "osl/eval/experimental/minorPiece.h"
#include "osl/eval/experimental/mobility.h"
#include "osl/eval/experimental/pieceStand.h"
#include "osl/eval/experimental/pin.h"
#include "osl/eval/experimental/king8.h"
#include "osl/eval/experimental/progress.h"
#include "osl/effect_util/pin.h"
#include "osl/misc/random.h"
#include "osl/oslConfig.h"
#include "osl/pieceStand.h"

#include <fstream>

bool osl::eval::experimental::TestEval::initialized_flag = false;
osl::eval::experimental::Weights
osl::eval::experimental::TestEval::piece_pair_weights;

namespace
{
#ifndef MINIMAL
  template <class Eval>
  static void setRandomOne()
  {
    osl::eval::experimental::Weights weights(Eval::DIM);
    for (size_t i = 0; i < weights.dimension(); ++i)
    {
      weights.setValue(i, (osl::misc::random() % 1024)-512);
    }
    Eval::setUp(weights);
  }
  template <class Eval>
  static void setRandomOne(int stage)
  {
    osl::eval::experimental::Weights weights(Eval::DIM);
    for (size_t i = 0; i < weights.dimension(); ++i)
    {
      weights.setValue(i, (osl::misc::random() % 1024)-512);
    }
    Eval::setUp(weights, stage);
  }
#endif
  template <class Eval>
  static int setUpOne(std::istream &is)
  {
    int read_count = 0;
    osl::eval::experimental::Weights weights(Eval::DIM);
    // std::cerr << typeid(Eval).name() << " " << Eval::DIM << "\n";
    for (size_t i = 0; i < weights.dimension(); ++i)
    {
      int val;
      is >> val;
      weights.setValue(i, val);
      ++read_count;
    }
    Eval::setUp(weights);
    return read_count;
  }
  template <class Eval>
  static int setUpOne(std::istream &is,int stage)
  {
    int read_count = 0;
    osl::eval::experimental::Weights weights(Eval::DIM);
    // std::cerr << typeid(Eval).name() << " " << Eval::DIM << "\n";
    for (size_t i = 0; i < weights.dimension(); ++i)
    {
      int val;
      is >> val;
      weights.setValue(i, val);
      ++read_count;
    }
    Eval::setUp(weights,stage);
    return read_count;
  }
}

bool osl::eval::experimental::TestEval::setUp(const char *filename)
{
  if (initialized_flag)
    return true;

  std::ifstream is(filename);
  Weights weights(PTYPE_SIZE);
  int read_count = 0;

  // flat
  for (int i = 0; i < PTYPE_SIZE; ++i)
  {
    int val;
    is >> val;
    weights.setValue(i, val);
    ++read_count;
  }
  PieceEval::setUp(weights);

  PiecePair::init();
  piece_pair_weights.resetDimension(PiecePair::DIM);
  for (size_t i = 0; i < piece_pair_weights.dimension(); ++i)
  {
    int val;
    is >> val;
    piece_pair_weights.setValue(i, val);
    ++read_count;
  }
  PiecePair::sanitize(piece_pair_weights);
  PiecePair::compile(piece_pair_weights);

  read_count += setUpOne<King25EffectAttack>(is);
  read_count += setUpOne<King25EffectYAttack>(is);
  read_count += setUpOne<ProgressBonus>(is);
  read_count += setUpOne<ProgressAttackDefense>(is);
  read_count += setUpOne<ProgressAttackDefenseAll>(is);

  // opening
  read_count += setUpOne<PieceStand>(is,0);
  read_count += setUpOne<Pin>(is,0);
  read_count += setUpOne<King25EffectEachBothOpening>(is);
  read_count += setUpOne<PawnDrop>(is,0);
  read_count += setUpOne<NoPawnOnStand>(is,0);
  read_count += setUpOne<GoldRetreat>(is,0);
  read_count += setUpOne<SilverRetreat>(is,0);
  read_count += setUpOne<KnightAdvance>(is,0);
  read_count += setUpOne<AllMajor>(is,0);
  read_count += setUpOne<KingXBlocked>(is,0);
  read_count += setUpOne<KingXBlockedY>(is,0);
  read_count += setUpOne<AllGold>(is,0);
  read_count += setUpOne<PtypeX>(is,0);
  read_count += setUpOne<PtypeY>(is,0);
  read_count += setUpOne<AnagumaEmpty>(is,0);
  read_count += setUpOne<NonPawnPieceStand>(is,0);
  read_count += setUpOne<King25EffectDefense>(is,0);
  read_count += setUpOne<King25EffectYDefense>(is,0);
  read_count += setUpOne<RookMobility>(is,0);
  read_count += setUpOne<BishopMobility>(is,0);
  read_count += setUpOne<LanceMobility>(is,0);
  read_count += setUpOne<RookEffect>(is,0);
  read_count += setUpOne<BishopEffect>(is,0);
  read_count += setUpOne<PawnAdvance>(is,0);
  read_count += setUpOne<PawnDropY>(is,0);
  read_count += setUpOne<KnightCheck>(is,0);

  // midgame
  read_count += setUpOne<PieceStand>(is,1);
  read_count += setUpOne<Pin>(is,1);
  read_count += setUpOne<King25EffectEachBothMidgame>(is);
  read_count += setUpOne<PawnDrop>(is,1);
  read_count += setUpOne<NoPawnOnStand>(is,1);
  read_count += setUpOne<GoldRetreat>(is,1);
  read_count += setUpOne<SilverRetreat>(is,1);
  read_count += setUpOne<KnightAdvance>(is,1);
  read_count += setUpOne<AllMajor>(is,1);
  read_count += setUpOne<KingXBlocked>(is,1);
  read_count += setUpOne<KingXBlockedY>(is,1);
  read_count += setUpOne<AllGold>(is,1);
  read_count += setUpOne<PtypeX>(is,1);
  read_count += setUpOne<PtypeY>(is,1);
  read_count += setUpOne<AnagumaEmpty>(is,1);
  read_count += setUpOne<NonPawnPieceStand>(is,1);
  read_count += setUpOne<King25EffectDefense>(is,1);
  read_count += setUpOne<King25EffectYDefense>(is,1);
  read_count += setUpOne<RookMobility>(is,1);
  read_count += setUpOne<BishopMobility>(is,1);
  read_count += setUpOne<LanceMobility>(is,1);
  read_count += setUpOne<RookEffect>(is,1);
  read_count += setUpOne<BishopEffect>(is,1);
  read_count += setUpOne<PawnAdvance>(is,1);
  read_count += setUpOne<PawnDropY>(is,1);
  read_count += setUpOne<KnightCheck>(is,1);

#ifdef EVAL_QUAD
  // midgame2
  read_count += setUpOne<PieceStand>(is,2);
  read_count += setUpOne<Pin>(is,2);
  read_count += setUpOne<King25EffectEachBothMidgame2>(is);
  read_count += setUpOne<PawnDrop>(is,2);
  read_count += setUpOne<NoPawnOnStand>(is,2);
  read_count += setUpOne<GoldRetreat>(is,2);
  read_count += setUpOne<SilverRetreat>(is,2);
  read_count += setUpOne<KnightAdvance>(is,2);
  read_count += setUpOne<AllMajor>(is,2);
  read_count += setUpOne<KingXBlocked>(is,2);
  read_count += setUpOne<KingXBlockedY>(is,2);
  read_count += setUpOne<AllGold>(is,2);
  read_count += setUpOne<PtypeX>(is,2);
  read_count += setUpOne<PtypeY>(is,2);
  read_count += setUpOne<AnagumaEmpty>(is,2);
  read_count += setUpOne<NonPawnPieceStand>(is,2);
  read_count += setUpOne<King25EffectDefense>(is,2);
  read_count += setUpOne<King25EffectYDefense>(is,2);
  read_count += setUpOne<RookMobility>(is,2);
  read_count += setUpOne<BishopMobility>(is,2);
  read_count += setUpOne<LanceMobility>(is,2);
  read_count += setUpOne<RookEffect>(is,2);
  read_count += setUpOne<BishopEffect>(is,2);
  read_count += setUpOne<PawnAdvance>(is,2);
  read_count += setUpOne<PawnDropY>(is,2);
  read_count += setUpOne<KnightCheck>(is,2);
#endif

  // endgame
  read_count += setUpOne<PieceStand>(is,EndgameIndex);
  read_count += setUpOne<Pin>(is,EndgameIndex);
  read_count += setUpOne<King25EffectEachBothEnding>(is);
  read_count += setUpOne<PawnDrop>(is,EndgameIndex);
  read_count += setUpOne<NoPawnOnStand>(is,EndgameIndex);
  read_count += setUpOne<GoldRetreat>(is,EndgameIndex);
  read_count += setUpOne<SilverRetreat>(is,EndgameIndex);
  read_count += setUpOne<KnightAdvance>(is,EndgameIndex);
  read_count += setUpOne<AllMajor>(is,EndgameIndex);
  read_count += setUpOne<KingXBlocked>(is,EndgameIndex);
  read_count += setUpOne<KingXBlockedY>(is,EndgameIndex);
  read_count += setUpOne<AllGold>(is,EndgameIndex);
  read_count += setUpOne<PtypeX>(is,EndgameIndex);
  read_count += setUpOne<PtypeY>(is,EndgameIndex);
  read_count += setUpOne<AnagumaEmpty>(is,EndgameIndex);
  read_count += setUpOne<NonPawnPieceStand>(is,EndgameIndex);
  read_count += setUpOne<King25EffectDefense>(is,EndgameIndex);
  read_count += setUpOne<King25EffectYDefense>(is,EndgameIndex);
  read_count += setUpOne<RookMobility>(is,EndgameIndex);
  read_count += setUpOne<BishopMobility>(is,EndgameIndex);
  read_count += setUpOne<LanceMobility>(is,EndgameIndex);
  read_count += setUpOne<RookEffect>(is,EndgameIndex);
  read_count += setUpOne<BishopEffect>(is,EndgameIndex);
  read_count += setUpOne<PawnAdvance>(is,EndgameIndex);
  read_count += setUpOne<PawnDropY>(is,EndgameIndex);
  read_count += setUpOne<KnightCheck>(is,EndgameIndex);

  // both
  read_count += setUpOne<KingPieceRelative>(is,0);
  read_count += setUpOne<KingPieceRelative>(is,1);
#ifdef EVAL_QUAD
  read_count += setUpOne<KingPieceRelative>(is,2);
#endif
  read_count += setUpOne<KingPieceRelative>(is,EndgameIndex);
  read_count += setUpOne<NonPawnPieceStandCombination>(is);
  read_count += setUpOne<NonPawnPieceStandTurn>(is);
  read_count += setUpOne<King25EffectEachXY>(is);
  read_count += setUpOne<RookPawnY>(is);
  read_count += setUpOne<RookEffectPiece>(is);
  read_count += setUpOne<BishopEffectPiece>(is);
  read_count += setUpOne<PieceStandY>(is);
  read_count += setUpOne<RookEffectPieceKingRelative>(is);
  read_count += setUpOne<BishopEffectPieceKingRelative>(is);
  read_count += setUpOne<RookPawnYX>(is);
  read_count += setUpOne<PawnPtypeOPtypeO>(is);
  read_count += setUpOne<CanCheckNonPawnPieceStandCombination>(is);
  read_count += setUpOne<PromotedMinorPieces>(is);
  read_count += setUpOne<KingPieceRelativeNoSupport>(is);
  read_count += setUpOne<NonPawnAttacked>(is);
  read_count += setUpOne<PtypeYY>(is);
  read_count += setUpOne<PawnPtypeOPtypeOY>(is);
  read_count += setUpOne<PawnDropX>(is);
  read_count += setUpOne<King3Pieces>(is);
  read_count += setUpOne<King3PiecesXY>(is);
  read_count += setUpOne<King25EffectEachKXY>(is);
  read_count += setUpOne<BishopHead>(is);
  read_count += setUpOne<BishopHeadKingRelative>(is);
  read_count += setUpOne<KnightCheckY>(is);
  read_count += setUpOne<KnightHead>(is);
  read_count += setUpOne<RookPromoteDefense>(is);
  read_count += setUpOne<PawnDropPawnStand>(is);
  read_count += setUpOne<PawnDropPawnStandX>(is);
  read_count += setUpOne<PawnDropPawnStandY>(is);
  read_count += setUpOne<King25Effect2>(is);
  read_count += setUpOne<King25EffectY2>(is);
  read_count += setUpOne<KnightHeadOppPiecePawnOnStand>(is);
  read_count += setUpOne<KingXBothBlocked>(is);
  read_count += setUpOne<KingXBothBlockedY>(is);
  read_count += setUpOne<KingRookBishop>(is);
  read_count += setUpOne<PromotedMinorPiecesY>(is);
  read_count += setUpOne<King25EffectSupported>(is);
  read_count += setUpOne<King25EffectSupportedY>(is);
  read_count += setUpOne<NonPawnAttackedKingRelative>(is);
  read_count += setUpOne<NonPawnAttackedPtype>(is);
  read_count += setUpOne<PtypeCount>(is);
  read_count += setUpOne<KingXBlocked3>(is);
  read_count += setUpOne<KingXBlocked3Y>(is);
  read_count += setUpOne<PtypeCountXY>(is);
  read_count += setUpOne<PtypeCountXYAttack>(is);
  read_count += setUpOne<LanceEffectPieceKingRelative>(is);
  read_count += setUpOne<KingMobility>(is);
  read_count += setUpOne<KingMobilitySum>(is);
  read_count += setUpOne<PtypeYPawnY>(is);
  read_count += setUpOne<GoldAndSilverNearKing>(is);
  read_count += setUpOne<PtypeCombination>(is);
  read_count += setUpOne<PieceStandCombinationBoth>(is);
  read_count += setUpOne<King25BothSide>(is);
  read_count += setUpOne<King25BothSideX>(is);
  read_count += setUpOne<King25BothSideY>(is);
  read_count += setUpOne<GoldAndSilverNearKingCombination>(is);
  read_count += setUpOne<KingMobilityWithRook>(is);
  read_count += setUpOne<KingMobilityWithBishop>(is);
  read_count += setUpOne<NumPiecesBetweenBishopAndKingSelf>(is);
  read_count += setUpOne<NumPiecesBetweenBishopAndKingOpp>(is);
  read_count += setUpOne<NumPiecesBetweenBishopAndKingAll>(is);
  read_count += setUpOne<King25Effect3>(is);

  initialized_flag = is;
  if (!initialized_flag)
  {
    std::cerr << "Failed to load TestEval data " << read_count << std::endl;
  }
  return initialized_flag;
}

bool osl::eval::experimental::TestEval::setUp()
{
  std::string filename = OslConfig::home();
#ifdef EVAL_QUAD
  filename += "/data/test-eval-quad.txt";
#else
  filename += "/data/test-eval.txt";
#endif
  return setUp(filename.c_str());  
}

osl::eval::experimental::
TestEval::TestEval(const NumEffectState &state)
  : progress(state)
{
  assert(initialized_flag);
  
  pawns.fill(0);
  black_pawn_count = 0;
  turn = state.getTurn();
  for (int i = PtypeTraits<PAWN>::indexMin;
       i < PtypeTraits<PAWN>::indexLimit; ++i)
  {
    const Piece pawn = state.getPieceOf(i);
    if (pawn.owner() == BLACK)
      ++black_pawn_count;
    if (pawn.isOnBoard() && !pawn.isPromoted())
      pawns[pawn.owner()][pawn.position().x() - 1] =
	pawn.position().y();
  }
  black_major_count = 0;
  black_gold_count = 0;
  for (int i = PtypeTraits<ROOK>::indexMin;
       i < PtypeTraits<ROOK>::indexLimit; ++i)
  {
    if (state.getPieceOf(i).owner() == BLACK)
      ++black_major_count;
  }
  for (int i = PtypeTraits<BISHOP>::indexMin;
       i < PtypeTraits<BISHOP>::indexLimit; ++i)
  {
    if (state.getPieceOf(i).owner() == BLACK)
      ++black_major_count;
  }
  for (int i = PtypeTraits<GOLD>::indexMin;
       i < PtypeTraits<GOLD>::indexLimit; ++i)
  {
    if (state.getPieceOf(i).owner() == BLACK)
      ++black_gold_count;
  }
  updateGoldSilverNearKing(state);

  ptype_count.fill(0);
  ptypeo_mask=0u;
  ptype_board_count.fill(0);
  for (int i = 0; i < Piece::SIZE; ++i)
  {
    const Piece piece = state.getPieceOf(i);
    if (piece.ptype() == KING)
      continue;
    ++ptype_count[piece.owner()][piece.ptype()];
    ptypeo_mask |= 1<<(piece.ptypeO()-PTYPEO_MIN);
    if (piece.isOnBoard())
      ++ptype_board_count[piece.owner()][piece.ptype()];
  }
  non_pawn_stand_count.fill(0);
  for (size_t i = 0; i < osl::PieceStand::order.size(); ++i)
  {
    const Ptype ptype = osl::PieceStand::order[i];
    if (ptype == PAWN)
      continue;
    non_pawn_stand_count[BLACK] +=
      state.countPiecesOnStand(osl::BLACK, ptype);
    non_pawn_stand_count[WHITE] += 
      state.countPiecesOnStand(osl::WHITE, ptype);
  }
  pin_mask[WHITE] = state.pin(WHITE);
  pin_mask[BLACK] = state.pin(BLACK);
  progress_independent_value = PieceEval::eval(state);
  piece_stand_value = PieceStand::eval(state);
  piece_pair_value = PiecePair::eval(state, piece_pair_weights);
  RookMobilityAll::eval(state, rook_mobility);
  BishopMobilityAll::eval(state, bishop_mobility);
  LanceMobilityAll::eval(state, lance_mobility);
  recalculated_value = ProgressBonus::eval(progress.progress16(BLACK),
					   progress.progress16(WHITE));
  recalculated_value += ProgressAttackDefense::eval(
    progress.progressAttack(BLACK), progress.progressDefense(WHITE),
    progress.progressAttack(WHITE), progress.progressDefense(BLACK)) +
    ProgressAttackDefenseAll::eval(
      progress.progressAttack(BLACK), progress.progressDefense(WHITE),
      progress.progressAttack(WHITE), progress.progressDefense(BLACK));

  knight_advance = KnightAdvance::eval(state);

  rook_effect = RookEffectBase::eval(state);
  bishop_effect = BishopEffectBase::eval(state);

  King25EffectEachBoth::eval(state, king25_effect_each);

  King25EffectBoth::countEffectAndPiecesBoth<BLACK>(state, effect25[WHITE],
						    effect25_supported[WHITE],
						    black_attack_effect, black_attack_piece,
						    white_defense_effect, white_defense_piece,
						    black_attack_supported_piece,
    white_vertical);
  King25EffectBoth::countEffectAndPiecesBoth<WHITE>(state, effect25[BLACK],
						    effect25_supported[BLACK],
						    white_attack_effect, white_attack_piece,
						    black_defense_effect, black_defense_piece,
						    white_attack_supported_piece,
						    black_vertical);
  recalculated_value +=
    King25EffectAttack::eval(state,
			     black_attack_effect,
			     black_attack_piece,
			     white_attack_effect, white_attack_piece);
  recalculated_value +=
    King25EffectYAttack::eval(state,
			      black_attack_effect,
			      black_attack_piece,
			      white_attack_effect, white_attack_piece);
  kingx_blocked = KingXBothBlocked::eval(state);
  {
    int black_attack_effect2 = std::min(black_attack_effect, 63);
    int white_attack_effect2 = std::min(white_attack_effect, 63);
    int black_stand_count = state.countPiecesOnStand<ROOK>(BLACK) +
      state.countPiecesOnStand<BISHOP>(BLACK) +
      state.countPiecesOnStand<GOLD>(BLACK) +
      state.countPiecesOnStand<SILVER>(BLACK);
    int white_stand_count = state.countPiecesOnStand<ROOK>(WHITE) +
      state.countPiecesOnStand<BISHOP>(WHITE) +
      state.countPiecesOnStand<GOLD>(WHITE) +
      state.countPiecesOnStand<SILVER>(WHITE);
    MultiInt result = King25Effect2::eval(state,
							black_attack_effect2,
							black_attack_piece,
							white_attack_effect2,
							white_attack_piece,
							black_stand_count,
							white_stand_count);
    MultiInt result_y = King25EffectY2::eval(state,
					      black_attack_effect2,
					      black_attack_piece,
					      white_attack_effect2,
					      white_attack_piece,
					      black_stand_count,
					      white_stand_count);
    MultiInt result_supported =
      King25EffectSupported::eval(black_attack_piece,
				  white_attack_piece,
				  black_attack_supported_piece,
				  white_attack_supported_piece);
    MultiInt result_supported_y =
      King25EffectSupportedY::eval(black_attack_piece,
				   white_attack_piece,
				   black_attack_supported_piece,
				   white_attack_supported_piece,
				   state.getKingPosition<BLACK>().y(),
				   state.getKingPosition<WHITE>().y());

    recalculated_stage_value = result;
    recalculated_stage_value += result_y + result_supported + result_supported_y;
    king_rook_bishop[BLACK]=KingRookBishop::evalOne<BLACK>(state);
    king_rook_bishop[WHITE]=KingRookBishop::evalOne<WHITE>(state);
    recalculated_stage_value+=king_rook_bishop[BLACK]-king_rook_bishop[WHITE];
    recalculated_stage_value+=KingXBlocked3::eval(state);
  }

  kingx_blocked += KingXBlocked::eval(state)+KingXBlockedY::eval(state);
  const MultiInt silver_retreat = SilverRetreatBoth::eval(state);
  const MultiInt gold_retreat = GoldRetreatBoth::eval(state);
  recalculated_stage_value += knight_advance;
  recalculated_stage_value += silver_retreat + gold_retreat;
  recalculated_stage_value += AllGold::eval(black_gold_count);
  recalculated_stage_value += AllMajor::eval(black_major_count);
  recalculated_stage_value += 
    King25EffectDefense::eval(state,black_defense_effect,black_defense_piece,
			      white_defense_effect, white_defense_piece);
  recalculated_stage_value += 
    King25EffectYDefense::eval(state,
			       black_defense_effect,
			       black_defense_piece,
			       white_defense_effect, white_defense_piece);
  recalculated_stage_value += AnagumaEmpty::eval(state);
  recalculated_stage_value += kingx_blocked[BLACK] + kingx_blocked[WHITE];

  recalculated_stage_value += NoPawnOnStand::eval(state, black_pawn_count);
  recalculated_stage_value += NonPawnPieceStand::eval(non_pawn_stand_count[BLACK], non_pawn_stand_count[WHITE]);
  recalculated_stage_value += Pin::eval(state, pin_mask[BLACK], pin_mask[WHITE]);
  recalculated_stage_value += KingMobility::eval(state) + KingMobilitySum::eval(state);
  recalculated_stage_value += GoldAndSilverNearKing::eval(state,
							  gs_near_king_count);
  recalculated_stage_value += PtypeCombination::eval(ptypeo_mask);
  recalculated_stage_value += PieceStandCombinationBoth::eval(state);
  king25_both_side[BLACK]=King25BothSide::evalOne<BLACK>(state,black_vertical);
  king25_both_side[WHITE]=King25BothSide::evalOne<WHITE>(state,white_vertical);
  recalculated_stage_value += king25_both_side[BLACK]-king25_both_side[WHITE];

  king_table_value = KingPieceRelative::eval(state);

  pawn_drop = PawnDropBoth::eval(state);

  ptypex = PtypeX::eval(state);

  ptypey = PtypeY::eval(state);

  can_check[BLACK] =
    CanCheckNonPawnPieceStandCombination::canCheck<BLACK>(state);
  can_check[WHITE] =
    CanCheckNonPawnPieceStandCombination::canCheck<WHITE>(state);
  piece_stand_combination = NonPawnPieceStandCombination::eval(state,
							       can_check);
  NonPawnPieceStandTurn::eval(state, piece_stand_turn);
  rook_pawn = RookPawnY::eval(state, pawns);
  piece_stand_y = PieceStandY::eval(state);

  pawn_advance = PawnAdvance::eval(state);
  knight_check = KnightCheck::eval(state);
  pawn_ptypeo = PawnPtypeOPtypeO::eval(state);

  promoted_minor_piece = PromotedMinorPieces::eval(state);

  effected_mask[BLACK] =
    effected_mask_for_attacked[BLACK] =
    state.effectedMask(BLACK);
  effected_mask[WHITE] =
    effected_mask_for_attacked[WHITE] =
    state.effectedMask(WHITE);
  mask_t black_ppawn =
    effected_mask_for_attacked[BLACK].selectBit<PAWN>() &
    state.promotedPieces().getMask<PAWN>();
  mask_t white_ppawn =
    effected_mask_for_attacked[WHITE].selectBit<PAWN>() &
    state.promotedPieces().getMask<PAWN>();
  effected_mask_for_attacked[BLACK].clearBit<PAWN>();
  effected_mask_for_attacked[WHITE].clearBit<PAWN>();
  effected_mask_for_attacked[BLACK].orMask(PtypeFuns<PAWN>::indexNum, black_ppawn);
  effected_mask_for_attacked[WHITE].orMask(PtypeFuns<PAWN>::indexNum, white_ppawn);
  nosupport = KingPieceRelativeNoSupport::eval(state);
  NonPawnAttacked::eval(state, non_pawn_attacked);
  NonPawnAttackedPtype::eval(state, attacked_mask, non_pawn_attacked_ptype);
  knight_head = KnightHead::eval(state);

  ptype_yy = PtypeYY::eval(state);
  king3pieces = King3Pieces::eval(state);
  bishop_head = BishopHead::eval(state);
  rook_promote_defense = RookPromoteDefense::eval(state);
  PtypeCount::eval(state, ptype_count, ptype_board_count, ptype_count_value);
  lance_effect_piece = LanceEffectPieceKingRelative::eval(state);
  ptype_y_pawn_y = PtypeYPawnY::eval(state, pawns);
  bishop_and_king = NumPiecesBetweenBishopAndKing::eval(state);
  recalculated_stage_value += King25Effect3::eval(state, effect25);
  invalidateCache();
}

int osl::eval::experimental::
TestEval::expect(const NumEffectState &state, Move move) const
{
  if (move.isPass())
    return value();
  int value;
  if(move.player()==BLACK)
    value = PieceEval::evalWithUpdate<BLACK>(state, move, progress_independent_value);
  else
    value = PieceEval::evalWithUpdate<WHITE>(state, move, progress_independent_value);

#ifdef USE_TEST_PROGRESS
  return roundUp(value * TestProgress::maxProgress() +
		 openingValue() * (TestProgress::maxProgress() - progress.progress()) + 
		 endgameValue() * progress.progress());
#else
  return roundUp(value * 16 +
		 openingValue() * (16 - progress.progress16().value()) + 
		 endgameValue() * progress.progress16().value());
#endif
}

void osl::eval::experimental::
TestEval::update(const NumEffectState &new_state, Move last_move)
{
  turn = alt(turn);
  if (last_move.isPass())
  {
    invalidateCache();
    return;
  }
  if(last_move.player()==BLACK)
    updateSub<BLACK>(new_state,last_move);
  else
    updateSub<WHITE>(new_state,last_move);
}
template<osl::Player P>
void osl::eval::experimental::
TestEval::updateSub(const NumEffectState &new_state, Move last_move)
{
  assert(last_move.player()==P);
  const Position opp_king =
    new_state.getKingPosition<PlayerTraits<P>::opponent>();
  const Position self_king =
    new_state.getKingPosition<P>();
  Ptype captured = last_move.capturePtype();
  if (captured != PTYPE_EMPTY)
  {
    Ptype base = unpromote(captured);
    if (base == PAWN)
    {
      if (P == BLACK)
	++black_pawn_count;
      else
	--black_pawn_count;
    }
    else
    {
      ++non_pawn_stand_count[P];
    }
    if (captured == PAWN)
    {
      pawns[alt(P)][last_move.to().x() - 1] = 0;
    }
    if (isMajorBasic(base))
    {
      if (P == BLACK)
	++black_major_count;
      else
	--black_major_count;
    }
    if (base == GOLD)
    {
      if (P == BLACK)
	++black_gold_count;
      else
	--black_gold_count;
    }
    if (base == GOLD || base == SILVER)
    {
      const int y_diff = std::abs(last_move.to().y() - opp_king.y());
      const int x_diff = std::abs(last_move.to().x() - opp_king.x());
      if (y_diff <= 2 && x_diff <= 3)
      {
	--gs_near_king_count[PlayerTraits<P>::opponent][std::max(x_diff, y_diff) - 1];
      }
    }
  }
  {
    const Ptype base_ptype = unpromote(last_move.ptype());
    if (base_ptype == GOLD || base_ptype == SILVER)
    {
      if (!last_move.isDrop())
      {
	const int y_diff = std::abs(last_move.from().y() - self_king.y());
	const int x_diff = std::abs(last_move.from().x() - self_king.x());
	if (y_diff <= 2 && x_diff <= 3)
	{
	  --gs_near_king_count[P][std::max(x_diff, y_diff) - 1];
	}
      }
      {
	const int y_diff = std::abs(last_move.to().y() - self_king.y());
	const int x_diff = std::abs(last_move.to().x() - self_king.x());
	if (y_diff <= 2 && x_diff <= 3)
	{
	  ++gs_near_king_count[P][std::max(x_diff, y_diff) - 1];
	}
      }
    }
    if (base_ptype == KING)
    {
      updateGoldSilverNearKing(new_state);
    }
  }
  if (last_move.isDrop() && last_move.ptype() != PAWN)
  {
    --non_pawn_stand_count[P];
  }
  if (last_move.ptype() == PPAWN && last_move.isPromote())
  {
    pawns[P][last_move.from().x() - 1] = 0;
  }
  if (last_move.ptype() == PAWN)
  {
    pawns[P][last_move.to().x() - 1] = last_move.to().y();
  }
  const Position kb = new_state.getKingPosition<BLACK>(), kw = new_state.getKingPosition<WHITE>();
  {
    const BoardMask mask = new_state.changedEffects();
    const bool update_black = mask.anyInRange(Board_Mask_Table5x5.mask(kw)); // black attack to white
    const bool update_white = mask.anyInRange(Board_Mask_Table5x5.mask(kb));
    if (update_black ||
	(effect25_supported[WHITE] & new_state.effectedMask(BLACK)) !=
	effect25_supported[WHITE] ||
	(~effect25_supported[WHITE] & effect25[WHITE] & ~new_state.effectedMask(BLACK)) !=
	(~effect25_supported[WHITE] & effect25[WHITE])){
      King25EffectBoth::countEffectAndPiecesBoth<BLACK>(
	new_state, effect25[WHITE], effect25_supported[WHITE],
	black_attack_effect, black_attack_piece,
	white_defense_effect, white_defense_piece,
	black_attack_supported_piece, white_vertical);
      king25_both_side[WHITE]=King25BothSide::evalOne<WHITE>(new_state,white_vertical);
    }
    if (update_white ||
	(effect25_supported[BLACK] & new_state.effectedMask(WHITE)) !=
	effect25_supported[BLACK] ||
	(~effect25_supported[BLACK] & effect25[BLACK] & ~new_state.effectedMask(WHITE)) !=
	(~effect25_supported[BLACK] & effect25[BLACK])){
      King25EffectBoth::countEffectAndPiecesBoth<WHITE>(
	new_state, effect25[BLACK], effect25_supported[BLACK],
	white_attack_effect, white_attack_piece,
	black_defense_effect, black_defense_piece,
	white_attack_supported_piece, black_vertical);
      king25_both_side[BLACK]=King25BothSide::evalOne<BLACK>(new_state,black_vertical);
    }
  }
#ifdef USE_TEST_PROGRESS
  progress.updateSub<P>(new_state, last_move);
#else
  progress.update(new_state, last_move);
#endif

  pin_mask[WHITE] = new_state.pin(WHITE);
  pin_mask[BLACK] = new_state.pin(BLACK);
  progress_independent_value =
    PieceEval::evalWithUpdate<P>(new_state, last_move, progress_independent_value);
  piece_stand_value =
    PieceStand::evalWithUpdate<P>(new_state, last_move,
				      piece_stand_value);
  if (new_state.longEffectChanged<ROOK>())
  {
    RookMobilityAll::eval(new_state, rook_mobility);
    rook_effect = RookEffectBase::eval(new_state);
  }
  else if( last_move.ptype() == KING)
  {
    rook_effect = RookEffectBase::eval(new_state);
  }
  if (new_state.longEffectChanged<BISHOP>())
  {
    BishopMobilityAll::eval(new_state, bishop_mobility);
    bishop_effect = BishopEffectBase::eval(new_state);
  }
  else if (last_move.ptype() == KING)
  {
    bishop_effect = BishopEffectBase::eval(new_state);
  }
  if (new_state.longEffectChanged<LANCE>() || last_move.ptype() == KING)
  {
    LanceMobilityAll::eval(new_state, lance_mobility);
    lance_effect_piece = LanceEffectPieceKingRelative::eval(new_state);
  }

  if (new_state.anyEffectChanged<KNIGHT>()) {
    knight_advance = KnightAdvance::eval(new_state);
  }
  KingXBlockedBoth::evalWithUpdateBang(new_state, last_move, kingx_blocked);
  const MultiInt silver_retreat = SilverRetreatBoth::eval(new_state);
  const MultiInt gold_retreat = GoldRetreatBoth::eval(new_state);
  recalculated_stage_value = silver_retreat+gold_retreat;
  recalculated_stage_value += AllGold::eval(black_gold_count);
  recalculated_stage_value += AllMajor::eval(black_major_count);
  
  recalculated_value = ProgressBonus::eval(progress.progress16(BLACK),
					    progress.progress16(WHITE));
  recalculated_value += ProgressAttackDefense::eval(
    progress.progressAttack(BLACK), progress.progressDefense(WHITE),
    progress.progressAttack(WHITE), progress.progressDefense(BLACK)) +
    ProgressAttackDefenseAll::eval(
      progress.progressAttack(BLACK), progress.progressDefense(WHITE),
      progress.progressAttack(WHITE), progress.progressDefense(BLACK));
  King25EffectEachBoth::evalWithUpdate(new_state, last_move,
				       king25_effect_each);
  
  recalculated_value +=
    King25EffectAttack::eval(new_state,
			     black_attack_effect,
			     black_attack_piece,
			     white_attack_effect, white_attack_piece);
  recalculated_value +=
    King25EffectYAttack::eval(new_state,
			      black_attack_effect,
			      black_attack_piece,
			      white_attack_effect, white_attack_piece);

  recalculated_stage_value += 
    King25EffectDefense::eval(new_state,black_defense_effect,black_defense_piece,
			      white_defense_effect, white_defense_piece);
  recalculated_stage_value += 
    King25EffectYDefense::eval(new_state,
			       black_defense_effect,
			       black_defense_piece,
			       white_defense_effect, white_defense_piece);
  recalculated_stage_value += knight_advance;
  recalculated_stage_value += AnagumaEmpty::eval(new_state);
  recalculated_stage_value += kingx_blocked[BLACK] + kingx_blocked[WHITE];
  recalculated_stage_value += NoPawnOnStand::eval(new_state, black_pawn_count);
  recalculated_stage_value += NonPawnPieceStand::eval(non_pawn_stand_count[BLACK], non_pawn_stand_count[WHITE]);
  recalculated_stage_value += Pin::eval(new_state, pin_mask[BLACK], pin_mask[WHITE]);
  recalculated_stage_value += KingMobility::eval(new_state) + KingMobilitySum::eval(new_state);
  recalculated_stage_value += GoldAndSilverNearKing::eval(new_state,
							  gs_near_king_count);
  recalculated_stage_value += PieceStandCombinationBoth::eval(new_state);
  
  {
    int black_attack_effect2 = std::min(black_attack_effect, 63);
    int white_attack_effect2 = std::min(white_attack_effect, 63);
    int black_stand_count = new_state.countPiecesOnStand<ROOK>(BLACK) +
      new_state.countPiecesOnStand<BISHOP>(BLACK) +
      new_state.countPiecesOnStand<GOLD>(BLACK) +
      new_state.countPiecesOnStand<SILVER>(BLACK);
    int white_stand_count = new_state.countPiecesOnStand<ROOK>(WHITE) +
      new_state.countPiecesOnStand<BISHOP>(WHITE) +
      new_state.countPiecesOnStand<GOLD>(WHITE) +
      new_state.countPiecesOnStand<SILVER>(WHITE);
    MultiInt result = King25Effect2::eval(new_state,
							 black_attack_effect2,
							 black_attack_piece,
							 white_attack_effect2,
							 white_attack_piece,
							 black_stand_count,
							 white_stand_count);
    MultiInt result_y = King25EffectY2::eval(new_state,
							    black_attack_effect2,
							    black_attack_piece,
							    white_attack_effect2,
							    white_attack_piece,
							    black_stand_count,
							    white_stand_count);
    MultiInt result_supported =
      King25EffectSupported::eval(black_attack_piece,
				  white_attack_piece,
				  black_attack_supported_piece,
				  white_attack_supported_piece);
    MultiInt result_supported_y =
      King25EffectSupportedY::eval(black_attack_piece,
				   white_attack_piece,
				   black_attack_supported_piece,
				   white_attack_supported_piece,
				   new_state.getKingPosition<BLACK>().y(),
				   new_state.getKingPosition<WHITE>().y());
    recalculated_stage_value += result;
    recalculated_stage_value += result_y + result_supported + result_supported_y;
    if(isMajorNonPieceOK(last_move.ptype()) || 
       isMajorNonPieceOK(last_move.capturePtype())){ // rook or bishop
      king_rook_bishop[BLACK]=KingRookBishop::evalOne<BLACK>(new_state);
      king_rook_bishop[WHITE]=KingRookBishop::evalOne<WHITE>(new_state);
    }
    else if(last_move.ptype() == KING){
      king_rook_bishop[P]=KingRookBishop::evalOne<P>(new_state);
    }
    recalculated_stage_value+=king_rook_bishop[BLACK]-king_rook_bishop[WHITE];
    recalculated_stage_value += KingXBlocked3::eval(new_state);
  recalculated_stage_value += king25_both_side[BLACK]-king25_both_side[WHITE];
  }
  {
    // optimization memo: 
    //  king_table_opening_value =
    //    king_opening_table.evalWithUpdate(new_state,
    //				      last_move, king_table_opening_value);
    //  king_table_ending_value =
    //    king_ending_table.evalWithUpdate(new_state,
    //				      last_move, king_table_ending_value);
    king_table_value = KingPieceRelative::evalWithUpdate<P>
      (new_state, last_move, king_table_value);
  }
  piece_pair_value = PiecePair::evalWithUpdateCompiled(new_state,
						       last_move,
						       piece_pair_value);
  pawn_drop = PawnDropBoth::evalWithUpdate<P>(new_state,
					   last_move, pawn_drop);

  ptypex = PtypeX::evalWithUpdate<P>(new_state, last_move, ptypex);
  ptypey = PtypeY::evalWithUpdate<P>(new_state, last_move, ptypey);
  CArray<bool, 2> can_check_new;
  can_check_new[BLACK] =
    CanCheckNonPawnPieceStandCombination::canCheck<BLACK>(new_state);
  can_check_new[WHITE] =
    CanCheckNonPawnPieceStandCombination::canCheck<WHITE>(new_state);
  piece_stand_combination =
    NonPawnPieceStandCombination::evalWithUpdate(new_state,
						 last_move,
						 piece_stand_combination,
						 can_check,
						 can_check_new);
  can_check = can_check_new;
  NonPawnPieceStandTurn::evalWithUpdateBang<P>(new_state,
					    last_move,
					    piece_stand_turn);
  rook_pawn = RookPawnY::eval(new_state, pawns);
  piece_stand_y = PieceStandY::evalWithUpdate<P>(new_state, last_move,
						 piece_stand_y);
  PawnAdvanceAll::evalWithUpdateBang<P>(new_state,
					last_move,
					pawn_advance);

  knight_check = KnightCheck::eval(new_state);
  pawn_ptypeo = PawnPtypeOPtypeO::template evalWithUpdate<P>(new_state, last_move,
 						 pawns,
 						 pawn_ptypeo);

  promoted_minor_piece =
    PromotedMinorPieces::evalWithUpdate(new_state,
					last_move,
					promoted_minor_piece);

  nosupport = KingPieceRelativeNoSupport::evalWithUpdate(new_state, last_move,
							 effected_mask,
							 nosupport);
  NonPawnAttacked::evalWithUpdateBang<P>(new_state,
				      last_move,
				      effected_mask_for_attacked,
				      non_pawn_attacked);
  NonPawnAttackedPtype::evalWithUpdateBang<P>(
    new_state, last_move, effected_mask_for_attacked,
    attacked_mask, non_pawn_attacked_ptype);
  effected_mask[BLACK] =
    effected_mask_for_attacked[BLACK] =
    new_state.effectedMask(BLACK);
  effected_mask[WHITE] =
    effected_mask_for_attacked[WHITE] =
    new_state.effectedMask(WHITE);
  mask_t black_ppawn =
    effected_mask_for_attacked[BLACK].selectBit<PAWN>() &
    new_state.promotedPieces().getMask<PAWN>();
  mask_t white_ppawn =
    effected_mask_for_attacked[WHITE].selectBit<PAWN>() &
    new_state.promotedPieces().getMask<PAWN>();
  effected_mask_for_attacked[BLACK].clearBit<PAWN>();
  effected_mask_for_attacked[WHITE].clearBit<PAWN>();
  effected_mask_for_attacked[BLACK].orMask(PtypeFuns<PAWN>::indexNum, black_ppawn);
  effected_mask_for_attacked[WHITE].orMask(PtypeFuns<PAWN>::indexNum, white_ppawn);

  ptype_yy = PtypeYY::evalWithUpdate(new_state, last_move, ptype_yy);
  king3pieces = King3Pieces::evalWithUpdate(new_state, last_move, king3pieces);
  bishop_head = BishopHead::eval(new_state);
  knight_head = KnightHead::eval(new_state);
  rook_promote_defense = RookPromoteDefense::eval(new_state);
  PtypeCount::evalWithUpdateBang<P>(new_state,
				 last_move, ptype_count, ptype_board_count,
				 ptype_count_value,ptypeo_mask);
  PtypeYPawnY::evalWithUpdateBang<P>(new_state, last_move,pawns, ptype_y_pawn_y);
  recalculated_stage_value += PtypeCombination::eval(ptypeo_mask);
  bishop_and_king = NumPiecesBetweenBishopAndKing::eval(new_state);
  recalculated_stage_value += King25Effect3::eval(new_state, effect25);
  invalidateCache();
}

#ifndef MINIMAL
osl::eval::experimental::TestEvalDebugInfo
osl::eval::experimental::TestEval::debugInfo(const NumEffectState &state)
{
  TestEvalDebugInfo debug_info;
  debug_info.values[TestEvalDebugInfo::EVAL] = value();
  debug_info.values[TestEvalDebugInfo::OPENING] = openingValue();
  debug_info.values[TestEvalDebugInfo::ENDGAME] = endgameValue();
  debug_info.values[TestEvalDebugInfo::PROGRESS_INDEPENDENT] = progressIndependentValue();
  debug_info.values[TestEvalDebugInfo::PROGRESS] = progress16().value();
  debug_info.values[TestEvalDebugInfo::PIECE] = progress_independent_value;
  debug_info.values[TestEvalDebugInfo::ROOK_MOBILITY] = 0; // XXX progress dependent now
  debug_info.values[TestEvalDebugInfo::BISHOP_MOBILITY] = 0; // XXX
  debug_info.values[TestEvalDebugInfo::LANCE_MOBILITY] = 0; // XXX
  int black_attack_effect, black_attack_piece, black_defense_effect, black_defense_piece,
    white_attack_effect, white_attack_piece, white_defense_effect, white_defense_piece;
  CArray<int, 5> black_vertical, white_vertical;
  King25EffectBoth::countEffectAndPiecesBoth<BLACK>(state, effect25[WHITE],
						    effect25_supported[WHITE],
						    black_attack_effect, black_attack_piece,
						    white_defense_effect, white_defense_piece,
						    black_attack_supported_piece,
						    white_vertical);
  King25EffectBoth::countEffectAndPiecesBoth<WHITE>(state, effect25[BLACK],
						    effect25_supported[BLACK],
						    white_attack_effect, white_attack_piece,
						    black_defense_effect, black_defense_piece,
						    white_attack_supported_piece,
						    black_vertical);
  debug_info.values[TestEvalDebugInfo::KING25_EFFECT] =
    King25EffectBoth::eval(state,
			   black_attack_effect,
			   black_attack_piece,
			   white_attack_effect, white_attack_piece,
			   black_defense_effect, black_defense_piece,
			   white_defense_effect, white_defense_piece) +
    King25EffectY::eval(state,
			black_attack_effect,
			black_attack_piece,
			white_attack_effect, white_attack_piece,
			black_defense_effect, black_defense_piece,
			white_defense_effect, white_defense_piece);
  debug_info.values[TestEvalDebugInfo::PIECE_PAIR] = piece_pair_value;
  debug_info.values[TestEvalDebugInfo::KING_PIECE_OPENING] = king_table_value[0];
  debug_info.values[TestEvalDebugInfo::KING_PIECE_ENDING] = king_table_value[EndgameIndex];
  debug_info.values[TestEvalDebugInfo::PIECE_STAND_OPENING] = piece_stand_value[0];
  debug_info.values[TestEvalDebugInfo::PIECE_STAND_ENDING] = piece_stand_value[EndgameIndex];
  debug_info.values[TestEvalDebugInfo::KING25_EACH_OPENING] = king25_effect_each[BLACK][0] + king25_effect_each[WHITE][0];
  debug_info.values[TestEvalDebugInfo::KING25_EACH_ENDING] = king25_effect_each[WHITE][EndgameIndex] + king25_effect_each[WHITE][EndgameIndex];
  debug_info.values[TestEvalDebugInfo::PAWN_DROP_ENDING] = pawn_drop[0];
  debug_info.values[TestEvalDebugInfo::PAWN_DROP_ENDING] = pawn_drop[EndgameIndex];
  debug_info.values[TestEvalDebugInfo::PTYPE_X_OPENING] = ptypex[0];
  debug_info.values[TestEvalDebugInfo::PTYPE_X_ENDING] = ptypex[EndgameIndex];
  debug_info.values[TestEvalDebugInfo::PTYPE_Y_OPENING] = ptypey[0];
  debug_info.values[TestEvalDebugInfo::PTYPE_Y_ENDING] = ptypey[EndgameIndex];
  debug_info.values[TestEvalDebugInfo::NO_PAWN_OPENING] =
    NoPawnOnStand::eval(state, black_pawn_count)[0];
  debug_info.values[TestEvalDebugInfo::NO_PAWN_ENDING] =
    NoPawnOnStand::eval(state, black_pawn_count)[EndgameIndex];
  debug_info.values[TestEvalDebugInfo::GOLD_RETREAT_OPENING] =
    GoldRetreat::eval(state)[0];
  debug_info.values[TestEvalDebugInfo::GOLD_RETREAT_ENDING] =
    GoldRetreat::eval(state)[EndgameIndex];
  debug_info.values[TestEvalDebugInfo::SILVER_RETREAT_OPENING] =
    SilverRetreat::eval(state)[0];
  debug_info.values[TestEvalDebugInfo::SILVER_RETREAT_ENDING] =
    SilverRetreat::eval(state)[EndgameIndex];
  debug_info.values[TestEvalDebugInfo::KNIGHT_BLOCKED_OPENING] =
    knight_advance[0];
  debug_info.values[TestEvalDebugInfo::KNIGHT_BLOCKED_ENDING] =
    knight_advance[EndgameIndex];
  debug_info.values[TestEvalDebugInfo::KINGX_BLOCKED_OPENING] =
    kingx_blocked[BLACK][0] + kingx_blocked[WHITE][0];
  debug_info.values[TestEvalDebugInfo::KINGX_BLOCKED_ENDING] =
    kingx_blocked[BLACK][EndgameIndex] + kingx_blocked[WHITE][EndgameIndex];
  debug_info.values[TestEvalDebugInfo::PIN_OPENING] =
    Pin::eval(state,
	      pin_mask[BLACK],
	      pin_mask[WHITE])[0];
  debug_info.values[TestEvalDebugInfo::PIN_ENDING] =
    Pin::eval(state,
	      pin_mask[BLACK],
	      pin_mask[WHITE])[EndgameIndex];
  debug_info.values[TestEvalDebugInfo::ALL_GOLD_OPENING] =
    AllGold::eval(black_gold_count)[0];
  debug_info.values[TestEvalDebugInfo::ALL_GOLD_ENDING] =
    AllGold::eval(black_gold_count)[EndgameIndex];
  debug_info.values[TestEvalDebugInfo::ANAGUMA_EMPTY_OPENING] =
    AnagumaEmpty::eval(state)[0];
  debug_info.values[TestEvalDebugInfo::ANAGUMA_EMPTY_ENDING] =
    AnagumaEmpty::eval(state)[EndgameIndex];
  debug_info.values[TestEvalDebugInfo::STAND_EXCEPT_PAWN_OPENING] =
    NonPawnPieceStand::eval(non_pawn_stand_count[BLACK], non_pawn_stand_count[WHITE])[0];
  debug_info.values[TestEvalDebugInfo::STAND_EXCEPT_PAWN_ENDING] =
    NonPawnPieceStand::eval(non_pawn_stand_count[BLACK], non_pawn_stand_count[WHITE])[EndgameIndex];
  return debug_info;
}

#define DEBUGPRINT(x) std::cerr << "  " << #x << " " << x << "\n"
void osl::eval::experimental::TestEval::
debug() const
{
  DEBUGPRINT(king_table_value[0]);
  DEBUGPRINT(piece_stand_value[0]);
  DEBUGPRINT(king25_effect_each[BLACK][0] + king25_effect_each[WHITE][0]);
  DEBUGPRINT(ptypex[0]);
  DEBUGPRINT(ptypey[0]);
  DEBUGPRINT(rook_mobility[0]);
  DEBUGPRINT(bishop_mobility[0]);
  DEBUGPRINT(lance_mobility[0]);
  DEBUGPRINT(rook_effect[0]);
  DEBUGPRINT(bishop_effect[0]);
  DEBUGPRINT(piece_stand_combination[0]); 
  DEBUGPRINT(piece_stand_turn[turn][0]);
  DEBUGPRINT(rook_pawn[0]);
  DEBUGPRINT(pawn_drop[0]);
  DEBUGPRINT(piece_stand_y[0]);
  DEBUGPRINT(knight_check[0]);
  DEBUGPRINT(pawn_advance[0]);
  DEBUGPRINT(pawn_ptypeo[0]);
  DEBUGPRINT(promoted_minor_piece[0]);
  DEBUGPRINT(nosupport[0]);
  DEBUGPRINT(non_pawn_attacked[turn][0]);
  DEBUGPRINT(ptype_yy[0]);
  DEBUGPRINT(king3pieces[0]);
  DEBUGPRINT(recalculated_stage_value[0]);
}

void osl::eval::experimental::TestEval::
setRandom()
{
  setRandomOne<King25EffectAttack>();
  setRandomOne<King25EffectYAttack>();
  // setRandomOne<ProgressBonus>(); // not suitable for random test
  setRandomOne<ProgressAttackDefense>();
  // setRandomOne<ProgressAttackDefenseAll>(); // not suitable for random test

  // opening
  setRandomOne<PieceStand>(0);
  setRandomOne<Pin>(0);
  setRandomOne<King25EffectEachBothOpening>();
  setRandomOne<PawnDrop>(0);
  setRandomOne<NoPawnOnStand>(0);
  setRandomOne<GoldRetreat>(0);
  setRandomOne<SilverRetreat>(0);
  setRandomOne<KnightAdvance>(0);
  setRandomOne<AllMajor>(0);
  setRandomOne<KingXBlocked>(0);
  setRandomOne<KingXBlockedY>(0);
  setRandomOne<AllGold>(0);
  setRandomOne<PtypeX>(0);
  setRandomOne<PtypeY>(0);
  setRandomOne<AnagumaEmpty>(0);
  setRandomOne<NonPawnPieceStand>(0);
  setRandomOne<King25EffectDefense>(0);
  setRandomOne<King25EffectYDefense>(0);
  setRandomOne<RookMobility>(0);
  setRandomOne<BishopMobility>(0);
  setRandomOne<LanceMobility>(0);
  setRandomOne<RookEffect>(0);
  setRandomOne<BishopEffect>(0);
  setRandomOne<PawnAdvance>(0);
  setRandomOne<PawnDropY>(0);
  setRandomOne<KnightCheck>(0);

  // midgame
  setRandomOne<PieceStand>(1);
  setRandomOne<Pin>(1);
  setRandomOne<King25EffectEachBothMidgame>();
  setRandomOne<PawnDrop>(1);
  setRandomOne<NoPawnOnStand>(1);
  setRandomOne<GoldRetreat>(1);
  setRandomOne<SilverRetreat>(1);
  setRandomOne<KnightAdvance>(1);
  setRandomOne<AllMajor>(1);
  setRandomOne<KingXBlocked>(1);
  setRandomOne<KingXBlockedY>(1);
  setRandomOne<AllGold>(1);
  setRandomOne<PtypeX>(1);
  setRandomOne<PtypeY>(1);
  setRandomOne<AnagumaEmpty>(1);
  setRandomOne<NonPawnPieceStand>(1);
  setRandomOne<King25EffectDefense>(1);
  setRandomOne<King25EffectYDefense>(1);
  setRandomOne<RookMobility>(1);
  setRandomOne<BishopMobility>(1);
  setRandomOne<LanceMobility>(1);
  setRandomOne<RookEffect>(1);
  setRandomOne<BishopEffect>(1);
  setRandomOne<PawnAdvance>(1);
  setRandomOne<PawnDropY>(1);
  setRandomOne<KnightCheck>(1);

#ifdef EVAL_QUAD
  // midgame2
  setRandomOne<PieceStand>(2);
  setRandomOne<Pin>(2);
  setRandomOne<King25EffectEachBothEnding>();
  setRandomOne<PawnDrop>(2);
  setRandomOne<NoPawnOnStand>(2);
  setRandomOne<GoldRetreat>(2);
  setRandomOne<SilverRetreat>(2);
  setRandomOne<KnightAdvance>(2);
  setRandomOne<AllMajor>(2);
  setRandomOne<KingXBlocked>(2);
  setRandomOne<KingXBlockedY>(2);
  setRandomOne<AllGold>(2);
  setRandomOne<PtypeX>(2);
  setRandomOne<PtypeY>(2);
  setRandomOne<AnagumaEmpty>(2);
  setRandomOne<NonPawnPieceStand>(2);
  setRandomOne<King25EffectDefense>(2);
  setRandomOne<King25EffectYDefense>(2);
  setRandomOne<RookMobility>(2);
  setRandomOne<BishopMobility>(2);
  setRandomOne<LanceMobility>(2);
  setRandomOne<RookEffect>(2);
  setRandomOne<BishopEffect>(2);
  setRandomOne<PawnAdvance>(2);
  setRandomOne<PawnDropY>(2);
  setRandomOne<KnightCheck>(2);
#endif
  // endgame
  setRandomOne<PieceStand>(EndgameIndex);
  setRandomOne<Pin>(EndgameIndex);
  setRandomOne<King25EffectEachBothMidgame>();
  setRandomOne<PawnDrop>(EndgameIndex);
  setRandomOne<NoPawnOnStand>(EndgameIndex);
  setRandomOne<GoldRetreat>(EndgameIndex);
  setRandomOne<SilverRetreat>(EndgameIndex);
  setRandomOne<KnightAdvance>(EndgameIndex);
  setRandomOne<AllMajor>(EndgameIndex);
  setRandomOne<KingXBlocked>(EndgameIndex);
  setRandomOne<KingXBlockedY>(EndgameIndex);
  setRandomOne<AllGold>(EndgameIndex);
  setRandomOne<PtypeX>(EndgameIndex);
  setRandomOne<PtypeY>(EndgameIndex);
  setRandomOne<AnagumaEmpty>(EndgameIndex);
  setRandomOne<NonPawnPieceStand>(EndgameIndex);
  setRandomOne<King25EffectDefense>(EndgameIndex);
  setRandomOne<King25EffectYDefense>(EndgameIndex);
  setRandomOne<RookMobility>(EndgameIndex);
  setRandomOne<BishopMobility>(EndgameIndex);
  setRandomOne<LanceMobility>(EndgameIndex);
  setRandomOne<RookEffect>(EndgameIndex);
  setRandomOne<BishopEffect>(EndgameIndex);
  setRandomOne<PawnAdvance>(EndgameIndex);
  setRandomOne<PawnDropY>(EndgameIndex);
  setRandomOne<KnightCheck>(EndgameIndex);

  // both
  setRandomOne<KingPieceRelative>(0);
  setRandomOne<KingPieceRelative>(1);
#ifdef EVAL_QUAD
  setRandomOne<KingPieceRelative>(2);
#endif
  setRandomOne<KingPieceRelative>(EndgameIndex);
  setRandomOne<NonPawnPieceStandCombination>();
  setRandomOne<NonPawnPieceStandTurn>();
  setRandomOne<King25EffectEachXY>();
  setRandomOne<RookPawnY>();
  setRandomOne<RookEffectPiece>();
  setRandomOne<BishopEffectPiece>();
  setRandomOne<PieceStandY>();
  setRandomOne<RookEffectPieceKingRelative>();
  setRandomOne<BishopEffectPieceKingRelative>();
  setRandomOne<RookPawnYX>();
  setRandomOne<PawnPtypeOPtypeO>();
  setRandomOne<CanCheckNonPawnPieceStandCombination>();
  setRandomOne<PromotedMinorPieces>();
  setRandomOne<KingPieceRelativeNoSupport>();
  setRandomOne<NonPawnAttacked>();
  setRandomOne<PtypeYY>();
  setRandomOne<PawnPtypeOPtypeOY>();
  setRandomOne<PawnDropX>();
  setRandomOne<King3Pieces>();
  setRandomOne<King3PiecesXY>();
  setRandomOne<King25EffectEachKXY>();
  setRandomOne<BishopHead>();
  setRandomOne<BishopHeadKingRelative>();
  setRandomOne<KnightCheckY>();
  setRandomOne<KnightHead>();
  setRandomOne<RookPromoteDefense>();
  setRandomOne<PawnDropPawnStand>();
  setRandomOne<PawnDropPawnStandX>();
  setRandomOne<PawnDropPawnStandY>();
  setRandomOne<King25Effect2>();
  setRandomOne<King25EffectY2>();
  setRandomOne<KnightHeadOppPiecePawnOnStand>();
  setRandomOne<KingXBothBlocked>();
  setRandomOne<KingXBothBlockedY>();
  setRandomOne<KingRookBishop>();
  setRandomOne<PromotedMinorPiecesY>();
  setRandomOne<King25EffectSupported>();
  setRandomOne<King25EffectSupportedY>();
  setRandomOne<NonPawnAttackedKingRelative>();
  setRandomOne<NonPawnAttackedPtype>();
  setRandomOne<PtypeCount>();
  setRandomOne<KingXBlocked3>();
  setRandomOne<KingXBlocked3Y>();
  setRandomOne<PtypeCountXY>();
  setRandomOne<PtypeCountXYAttack>();
  setRandomOne<LanceEffectPieceKingRelative>();
  setRandomOne<KingMobility>();
  setRandomOne<KingMobilitySum>();

}
#endif

// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
