#include "osl/record/checkDuplicate.h"
#include "osl/apply_move/applyMove.h"
#include <boost/foreach.hpp>
#include <boost/format.hpp>
#include <iostream>

osl::hash::HashKey osl::record::
CheckDuplicate::getLastState(const osl::vector<osl::Move>& moves)
{
  osl::SimpleState state(osl::HIRATE);
  osl::hash::HashKey hval(state);

  BOOST_FOREACH(const osl::Move& move, moves)
  {
    osl::ApplyMoveOfTurn::doMove(state, move);
    hval = hval.newHashWithMove(move);
    assert(state.isConsistent(true));
  }

  return hval;
}

osl::record::CheckDuplicate::DUPLICATE_RESULT osl::record::
CheckDuplicate::regist(const osl::vector<osl::Move>& moves)
{
  const osl::hash::HashKey hval = 
    osl::record::CheckDuplicate::getLastState(moves);
  return regist(hval, moves);
}

osl::record::CheckDuplicate::DUPLICATE_RESULT osl::record::
CheckDuplicate::regist(const osl::hash::HashKey& key, 
                       const osl::vector<osl::Move>& moves)
{
  ++regist_counter;

  keymap_t::iterator rs = keys.find(key);
  if (rs == keys.end())
  {
    // not found. i.e. a new key
    moves_t new_value;
    new_value.push_back(moves);
    keys.insert(std::make_pair(key, new_value));
    return NO_DUPLICATE;
  }
  else
  {
    moves_t& values = rs->second;
    moves_t::iterator match = values.begin();
    while (match != values.end())
    {
      if (*match++ == moves)
        break;
    } // while
    if (match == values.end())
    {
      // new moves
      ++duplicated_hash_counter;
      values.push_back(moves);
      return HASH_DUPLICATE;
    }
    else
    {
      // hit 
      ++duplicated_moves_counter;
      return MOVES_DUPLICATE;
    }
  }
}

void osl::record::
CheckDuplicate::print(std::ostream& out) const
{
  std::cout << boost::format("Trials %d, Unique %d, Duplicates Hash %d, Duplicated moves %d\n")
               % regist_counter 
               % keys.size() 
               % duplicated_hash_counter 
               % duplicated_moves_counter;
}

// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
