/* dotAnalyzer.cc
 */
#include "osl/search/analyzer/dotAnalyzer.h"
#include "osl/search/analyzer/dotWriter.h"
#include "osl/search/simpleHashTable.h"
#include "osl/search/simpleHashRecord.h"
#include "osl/hash/hashKey.h"

osl::search::analyzer::DotAnalyzer::
DotAnalyzer(const SimpleHashTable& t, std::ostream& os)
    : table(t), writer(new DotWriter(os))
{
}

osl::search::analyzer::DotAnalyzer::
~DotAnalyzer()
{
}

void osl::search::analyzer::DotAnalyzer::
resetWriter(LogWriter *new_writer)
{
  writer.reset(new_writer);
}

void osl::search::analyzer::DotAnalyzer::
analyzeSequence(const HashKey& root, int limit, 
		const Move *first, const Move *last)
{
  const SimpleHashRecord *record = table.find(root);
  if (! record) 
    return;

  writer->showNode(root.turn(), record, limit, 
		   LogWriter::IMPORTANT);
  const SearchMoveSet& moves = record->moves();
  for (; first != last; ++first) 
  {
    const SearchMove *move = moves.find(*first);
    if (! move)
      break;
    const SimpleHashRecord *next_record = move->record;
    if (! next_record) 
      break;
    const MoveLogProb ml = move->moveLogProb();
    writer->showArc(record, next_record, ml, true);
    record = next_record;
    writer->showNode(ml.getMove().player(), record, limit, LogWriter::IMPORTANT);
  }

  analyze(root, limit);
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
