/* dotAnalyzerAll.cc
 */
#include "osl/search/analyzer/dotAnalyzerAll.h"
#include "osl/search/analyzer/dotWriter.h"
#include "osl/search/simpleHashTable.h"
#include "osl/search/simpleHashRecord.h"
#include "osl/hash/hashKey.h"
#include "osl/moveLogProb.h"

osl::search::analyzer::DotAnalyzerAll::
DotAnalyzerAll(const SimpleHashTable& t, std::ostream& o)
  : DotAnalyzer(t, o)
{
}

osl::search::analyzer::DotAnalyzerAll::
~DotAnalyzerAll()
{
}


void osl::search::analyzer::DotAnalyzerAll::
visit(const HashKey& key, int limit,
      const SimpleHashRecord *from, const MoveLogProb& move)
{
  const SimpleHashRecord *record = table.find(key);
  if (! record) 
    return;
  
  if (move.validMove()) 
  {
    limit -= move.getLogProb();
    if (limit < 0)
      return;
  }
  if (from) 
    writer->showArc(from, record, move, false);

  const SearchMoveSet& moves = record->moves();
  const MoveLogProb& bestMove = record->bestMove().moveLogProb();
  if (! moves.empty())
  {
    writer->showNode(key.turn(), record, limit, 
		     LogWriter::NORMAL);
    SearchMoveSet::const_range r(moves);
    for (SearchMoveSet::const_iterator p=r.first; p!=r.last; ++p)
    {
      assert(p->getMove().isPass() 
	     || (key.isPlayerOfTurn(BLACK)
		 == (p->getMove().player() == BLACK)));
      const HashKey newKey = key.newHashWithMove(p->getMove());
      if (p->getMove() == bestMove.getMove())
	visit(newKey, limit, record, bestMove);
      else
	visit(newKey, limit, record, p->moveLogProb());
    }
  }
  else
  {
    writer->showNodeQuiescence(key.turn(), record, limit, 
			       LogWriter::NORMAL);
#if 0
    const QuiescenceRecord& qrecord = record->qrecord;
    for (size_t i=0; i<qrecord.moves.size(); ++i)
    {
      const Move move = qrecord.moves[i];
      assert(key.isPlayerOfTurn(BLACK)
	     == (getPlayer(move) == BLACK));
      const HashKey newKey = key.newHashWithMove(move);
      visitQuiescence(newKey, limit, record, move);
    }
#endif
  }
}

void osl::search::analyzer::DotAnalyzerAll::
visitQuiescence(const HashKey& key, int id,
		const SimpleHashRecord *from, const Move move)
{
  const SimpleHashRecord *record = table.find(key);
  if (! record) 
    return;
  if (dejavu.find(record) != dejavu.end())
    return;
  dejavu.insert(record);

  if (from) 
  {
    assert(move.isNormal());
    writer->showArc(from, record, MoveLogProb(move,id), false);
  }
  writer->showNodeQuiescence(key.turn(), record, 0, LogWriter::NORMAL);

  const QuiescenceRecord& qrecord = record->qrecord;
  MoveVector moves;
  qrecord.loadMoves(moves);
  for (size_t i=0; i<moves.size(); ++i)
  {
    const Move move = moves[i];
    assert(key.isPlayerOfTurn(BLACK)
	   == (move.player() == BLACK));
    const HashKey newKey = key.newHashWithMove(move);
    visitQuiescence(newKey, i, record, move);
  }
}

void osl::search::analyzer::DotAnalyzerAll::
analyzeQuiescence(const HashKey& root)
{
  visitQuiescence(root, 0, 0, Move::INVALID());
}

void osl::search::analyzer::DotAnalyzerAll::
analyze(const HashKey& key, int limit)
{
  MoveLogProb dummy;
  visit(key, limit, 0, dummy);
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
