/* activityCount.cc
 */
#include "osl/stat/activityCount.h"
#include <algorithm>
#include <cstdio>

osl::stat::
ActivityCount::ActivityCount(unsigned int l)
  : counts(new int[l]), length(l)
{
  std::fill(&counts[0], &counts[l], 0);
}

osl::stat::
ActivityCount::~ActivityCount()
{
}

void osl::stat::
ActivityCount::clear()
{
  std::fill(&counts[0], &counts[length], 0);
}

void osl::stat::
ActivityCount::setBinary(int threshold)
{
  unsigned int actives = 0;
  for (unsigned int i=0; i<length; ++i)
  {
    if (counts[i] >= threshold)
    {
      ++actives;
      counts[i] = 1;
    }
    else
    {
      counts[i] = 0;
    }
  }
  fprintf(stderr, "active count %d\n", actives);
}

void osl::stat::
ActivityCount::loadBinary(const char *filename, int threshold)
{
  clear();
  FILE *fp = fopen(filename, "r");
  assert(fp);
  for (unsigned int i=0; i<length; ++i)
  {
    int occur;
    int ret
#ifdef __GNUC__
      __attribute__ ((unused))
#endif
      = fscanf(fp, "%d", &occur);
    assert(ret == 1);
    counts[i] = (occur >= threshold) ? 1 : 0;
  }
  fclose(fp);
}

void osl::stat::
ActivityCount::show(const char *filename) const
{
  FILE *fp = fopen(filename, "w");
  assert(fp);
  for (unsigned int i=0; i<length; ++i)
  {
    fprintf(fp, "%d\n", counts[i]);
  }
  fclose(fp);
}

/* ------------------------------------------------------------------------- */
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
