#include "osl/stat/instanceMultiplier.h"
#include "osl/stat/activityCount.h"
#include <iostream>
#include <fstream>
osl::stat::
InstanceMultiplier::
InstanceMultiplier(unsigned int dim, unsigned int r,
		   const SparseInstanceVector& i, const ActivityCount& a,
		   valarray_t& w, int t, const char *f) 
    : SparseRegressionMultiplier(dim), 
      instances(i), activity(a), cur(0), 
      threshold(t), weights(w), best_correlation(0.0),
      cur_iteration(0), best_iteration(0), reserve_for_validation(r),
      filename(f)
{
  assert(w.size() == this->dim());
}
osl::stat::
InstanceMultiplier::~InstanceMultiplier()
{
}
void osl::stat::
InstanceMultiplier::printCurrentDate() const
{
  struct tm *cur;
  time_t ct;
  time(&ct);
  cur=localtime(&ct);
  std::cerr << cur->tm_hour << ":" << cur->tm_min << ":" << cur->tm_sec
	    << "\n";
}
void osl::stat::
InstanceMultiplier::newIteration() const
{
  ++cur_iteration;
  printCurrentDate();
  const double correlation = validate();
  if (best_correlation < correlation)
  {
    best_correlation = correlation;
    best_iteration = cur_iteration;
    std::ofstream os(filename);
    for (unsigned int i=0; i<dim(); ++i)
    {
      os << weights[i] << "\n";
    }
  }
  std::cerr << "best " << best_iteration << " " << best_correlation << "\n";
}
bool osl::stat::
InstanceMultiplier::getVectorX(unsigned int& num_elements,
			       unsigned int *non_zero_indices, 
			       double *non_zero_values) const
{
  num_elements = 0;
  const SparseInstance& a = instances[cur];
  const unsigned int dim = this->dim();
  for (size_t i=0; i<a.size(); ++i)
  {
    if (activity.isActive(a[i].index))
    {
      assert(a[i].index < dim-1);
      non_zero_indices[num_elements] = a[i].index;
      non_zero_values[num_elements] = a[i].value;
      ++num_elements;
    }
  }
  non_zero_indices[num_elements] = dim-1;
  non_zero_values[num_elements] = 1;
  ++num_elements;

  ++cur;
  if (cur >= instances.size())
  {
    cur = reserve_for_validation;
    return false;
  }
  return true;
}

// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
