#include "osl/effect_util/sendOffPosition.h"
#include "osl/record/csaString.h"

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>
#include <iostream>
using namespace osl;
using namespace osl::effect_util;

class SendOffPositionTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(SendOffPositionTest);
  CPPUNIT_TEST(testSimple);
  CPPUNIT_TEST(testDefense);
  CPPUNIT_TEST(testEdge);
  CPPUNIT_TEST_SUITE_END();
public:
  void testSimple();
  void testDefense();
  void testEdge();
};

CPPUNIT_TEST_SUITE_REGISTRATION(SendOffPositionTest);

void SendOffPositionTest::testSimple()
{
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  * +HI *  * \n"
			   "P2 *  * -OU-KI *  *  *  *  * \n"
			   "P3 *  *  *  *  *  *  *  *  * \n"
			   "P4 *  *  *  *  *  *  *  *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9+OU *  *  *  *  *  *  *  * \n"
			   "P+00AL\n"
			   "+\n").getInitialState());
    Position8 sendoffs;
    Position king(7,2);
    SendOffPosition::find<BLACK>(state, king, sendoffs);
    CPPUNIT_ASSERT(sendoffs.empty());
  }
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  *  *  *  * \n"
			   "P2 *  * -OU-KI *  * +HI *  * \n"
			   "P3 *  *  *  *  *  *  *  *  * \n"
			   "P4 *  *  *  *  *  *  *  *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9+OU *  *  *  *  *  *  *  * \n"
			   "P+00AL\n"
			   "+\n").getInitialState());
    Position8 sendoffs;
    Position king(7,2);
    SendOffPosition::find<BLACK>(state, king, sendoffs);
    CPPUNIT_ASSERT_EQUAL((size_t)3, sendoffs.size());
  }
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  *  *  *  * \n"
			   "P2 *  *  * -KI *  * +HI *  * \n"
			   "P3 *  * -OU *  *  *  *  *  * \n"
			   "P4 *  *  *  *  *  *  *  *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9+OU *  *  *  *  *  *  *  * \n"
			   "P+00AL\n"
			   "+\n").getInitialState());
    Position8 sendoffs;
    Position king(7,3);
    SendOffPosition::find<BLACK>(state, king, sendoffs);
    CPPUNIT_ASSERT_EQUAL((size_t)5, sendoffs.size());
  }
}
void SendOffPositionTest::testDefense()
{
  {
    NumEffectState state(CsaString(
			   "P1 *  *  *  *  *  *  *  *  * \n"
			   "P2 *  *  * -KI *  * +HI *  * \n"
			   "P3 *  * -OU *  *  *  *  *  * \n"
			   "P4 *  * -KI *  *  *  *  *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9+OU *  *  *  *  *  *  *  * \n"
			   "P+00AL\n"
			   "+\n").getInitialState());
    Position8 sendoffs;
    Position king(7,3);
    SendOffPosition::find<BLACK>(state, king, sendoffs);
#if 0
    for (Position8::const_iterator p=sendoffs.begin(); p!=sendoffs.end(); ++p)
    {
      std::cerr << *p << "\n";
    }
#endif
    CPPUNIT_ASSERT_EQUAL((size_t)3, sendoffs.size()); // 82, 83, 72
  }
}

void SendOffPositionTest::testEdge()
{
  {
    NumEffectState state(CsaString(
			   "P1 * -KI * +HI *  *  *  *  * \n"
			   "P2-OU *  *  *  *  *  *  *  * \n"
			   "P3 *  *  *  *  *  *  *  *  * \n"
			   "P4 *  *  *  *  *  *  *  *  * \n"
			   "P5 *  *  *  *  *  *  *  *  * \n"
			   "P6 *  *  *  *  *  *  *  *  * \n"
			   "P7 *  *  *  *  *  *  *  *  * \n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9+OU *  *  *  *  *  *  *  * \n"
			   "P+00AL\n"
			   "+\n").getInitialState());
    Position8 sendoffs;
    Position king(9,2);
    SendOffPosition::find<BLACK>(state, king, sendoffs);
    CPPUNIT_ASSERT_EQUAL((size_t)2, sendoffs.size()); // 93, 83
  }
}

// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
