#include "../recursiveDoUndoMove.h"
#include "../doUndoMoveLockTest.h"
#include "osl/apply_move/applyMove.h"
#include "osl/record/csaRecord.h"
#include "osl/record/csaString.h"
#include "osl/state/numEffectState.h"
#include "osl/effect/numSimpleEffect.tcc"

#include "osl/container/pieceVector.h"
#include "osl/effect_action/storePiece.h"
#include "osl/oslConfig.h"

#include <iomanip>
#include <fstream>
#include <vector>
#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

typedef osl::NumEffectState state_t;

class EffectTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE( EffectTest );
  CPPUNIT_TEST(testMove);
  CPPUNIT_TEST(testPieceHasEffectTo);
  /*
   * foreachEffect 
   */
  CPPUNIT_TEST(testForeachEffect);
  CPPUNIT_TEST(testForeachEffectPtype);
  CPPUNIT_TEST(testForeachEffectNotBy);
  CPPUNIT_TEST(testForeachEffectOfPiece);
  CPPUNIT_TEST(testForSortedEffect);
  CPPUNIT_TEST( testForEachEffectOfPiece );
  CPPUNIT_TEST(testHasEffectFromTo);
  /*
   * hasEffect
   */
  CPPUNIT_TEST(testHasEffectBy);
  CPPUNIT_TEST(testHasEffectByWithRemove);
  CPPUNIT_TEST(testHasEffectNotBy);
  CPPUNIT_TEST(testHasMultipleEffectBy);
  CPPUNIT_TEST(testHasEffectPiece);
  CPPUNIT_TEST(testHasEffectDir);
  CPPUNIT_TEST(testHasEffectByPtype);

  CPPUNIT_TEST(testCountEffect2);
  CPPUNIT_TEST(testCountEffect);
  CPPUNIT_TEST(testCountEffectPin);
  CPPUNIT_TEST(testCountKingEffect);
  CPPUNIT_TEST(testKingCanBeCaptured);

  CPPUNIT_TEST(testCsaFiles);

  CPPUNIT_TEST(testEffectPtype);

  CPPUNIT_TEST(testChangedEffect);
  CPPUNIT_TEST(testSelectLong);
  CPPUNIT_TEST(testEffected);

  CPPUNIT_TEST(testKingMobility);
  CPPUNIT_TEST(testLongEffectOfDirection);

  CPPUNIT_TEST_SUITE_END();
private:
  osl::NumEffectState state;
  state_t eState;
public:
  EffectTest();
  void testMove();
  void testPieceHasEffectTo();

  void testForeachEffect();
  void testForeachEffectPtype();
  void testForeachEffectNotBy();
  void testForeachEffectOfPiece();
  void testForSortedEffect();
  void testForEachEffectOfPiece();
  void testHasEffectFromTo();

  void testHasEffectBy();
  void testHasEffectByWithRemove();
  void testHasEffectNotBy();
  void testHasMultipleEffectBy();
  void testHasEffectPiece();
  void testHasEffectDir();
  void testHasEffectByPtype();

  void testCountEffect2();
  void testCountEffect();
  void testCountEffectPin();
  void testCountKingEffect();
  void testKingCanBeCaptured();
  void testCsaFiles();

  void testEffectPtype();
  void testChangedEffect();
  void testSelectLong();
  void testEffected();
  void testKingMobility();
  void testLongEffectOfDirection();
};


CPPUNIT_TEST_SUITE_REGISTRATION(EffectTest);

using namespace osl;
using namespace osl::effect_action;

EffectTest::EffectTest() :
  state(CsaString(
	  "P1-KY-KE *  *  *  *  *  *  * \n"
	  "P2 *  *  *  *  *  * -KI-OU * \n"
	  "P3 *  * -FU *  * -KI *  *  * \n"
	  "P4-FU-HI *  *  *  *  * +GI-KY\n"
	  "P5 * -FU+FU * -FU-FU-FU-FU+FU\n"
	  "P6+FU *  *  *  *  * +KE *  * \n"
	  "P7 * +FU * +FU * +FU+FU+FU-KY\n"
	  "P8+KY *  *  *  * +KI+KI *  * \n"
	  "P9 * +KE-UM * +KA *  * +KE+OU\n"
	  "P-00FU\n"
	  "P+00FU\n"
	  "P-00FU\n"
	  "P-00GI\n"
	  "P-00GI\n"
	  "P-00GI\n"
	  "P-00HI\n"
	  "+\n"
	  ).getInitialState()),eState(state)
{
#if 0
  state=CsaString(
    "P1-KY-KE *  *  *  *  *  *  * \n"
    "P2 *  *  *  *  *  * -KI-OU * \n"
    "P3 *  * -FU *  * -KI *  *  * \n"
    "P4-FU-HI *  *  *  *  * +GI-KY\n"
    "P5 * -FU+FU * -FU-FU-FU-FU+FU\n"
    "P6+FU *  *  *  *  * +KE *  * \n"
    "P7 * +FU * +FU * +FU+FU+FU-KY\n"
    "P8+KY *  *  *  * +KI+KI *  * \n"
    "P9 * +KE-UM * +KA *  * +KE+OU\n"
    "P-00FU\n"
    "P+00FU\n"
    "P-00FU\n"
    "P-00GI\n"
    "P-00GI\n"
    "P-00GI\n"
    "P-00HI\n"
    "+\n"
    ).getInitialState();
  eState=state_t(state);
#endif
}



void EffectTest::testMove()
{
}

struct StorePositionVector{
  vector<Position> store;
  template<Player P,Ptype Type>
  void doActionPtype(Piece p,Position pos){
    store.push_back(pos);
  }
  template<Player P>
  void doAction(Piece /*p*/,Position pos){
    store.push_back(pos);
  }
  bool isMember(Position pos) const{
    return std::find(store.begin(),store.end(),pos)!=store.end();
  }
};

void EffectTest::testHasEffectFromTo()
{
  {
    NumEffectState state(CsaString(
			   "P1-KY * +UM *  *  *  * -KE-KY\n"
			   "P2 *  *  *  *  *  *  * -OU * \n"
			   "P3 *  *  *  *  * -HI * -FU-FU\n"
			   "P4-FU * -FU * -FU-KI-FU-GI * \n"
			   "P5 *  *  *  *  *  *  *  * +FU\n"
			   "P6+FU+FU+FU+KI+FU * +FU *  * \n"
			   "P7 * +KI * +FU *  * -UM *  * \n"
			   "P8 * +OU * +GI *  * -NG *  * \n"
			   "P9+KY+KE *  *  *  *  *  * -RY\n"
			   "P+00KI00GI00KY00FU\n"
			   "P-00KE00KE00FU00FU00FU00FU\n"
			   "+\n").getInitialState());
    CPPUNIT_ASSERT(state.hasEffectFromTo(newPtypeO(BLACK,PBISHOP),
					 Position(8,2),
					 Position(3,7)));
  }
  {
    NumEffectState state(CsaString(
			   "P1+NY+TO *  *  *  * -OU-KE-KY\n"
			   "P2 *  *  *  *  * -GI-KI *  *\n"
			   "P3 * +RY *  * +UM * -KI-FU-FU\n"
			   "P4 *  * +FU-FU *  *  *  *  *\n"
			   "P5 *  * -KE * +FU *  * +FU *\n"
			   "P6-KE *  * +FU+GI-FU *  * +FU\n"
			   "P7 *  * -UM *  *  *  *  *  *\n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9 * +OU * -GI *  *  *  * -NG\n"
			   "P+00HI00KI00KE00KY00FU00FU00FU00FU00FU00FU\n"
			   "P-00KI00KY00FU00FU\n"
			   "P-00AL\n"
			   "+\n").getInitialState());
    // (8,3)ε
    CPPUNIT_ASSERT(state.hasEffectFromTo(newPtypeO(BLACK,PROOK),Position(8,3),Position(8,9)));
    CPPUNIT_ASSERT(!state.hasEffectFromTo(newPtypeO(BLACK,PROOK),Position(8,3),Position(3,3)));
    // (4,2)ζ
    CPPUNIT_ASSERT(state.hasEffectFromTo(newPtypeO(WHITE,SILVER),Position(4,2),Position(5,3)));
    CPPUNIT_ASSERT(!state.hasEffectFromTo(newPtypeO(WHITE,SILVER),Position(4,2),Position(3,2)));
    // (8,2)ε, Ǥǽ
    CPPUNIT_ASSERT(state.hasEffectFromTo(newPtypeO(BLACK,PROOK),Position(8,2),Position(7,1)));
  }
}

void EffectTest::testForEachEffectOfPiece()
{
  NumEffectState state(CsaString(
			 "P1+NY+TO *  *  *  * -OU-KE-KY\n"
			 "P2 *  *  *  *  * -GI-KI *  *\n"
			 "P3 * +RY *  * +UM * -KI-FU-FU\n"
			 "P4 *  * +FU-FU *  *  *  *  *\n"
			 "P5 *  * -KE * +FU *  * +FU *\n"
			 "P6-KE *  * +FU+GI-FU *  * +FU\n"
			 "P7 *  * -UM *  *  *  *  *  *\n"
			 "P8 *  *  *  *  *  *  *  *  * \n"
			 "P9 * +OU * -GI *  *  *  * -NG\n"
			 "P+00HI00KI00KE00KY00FU00FU00FU00FU00FU00FU\n"
			 "P-00KI00KY00FU00FU\n"
			 "P-00AL\n"
			 "+\n").getInitialState());
  {
    // 53
    StorePositionVector store;
    state.forEachEffectOfPiece<BLACK,PBISHOP,StorePositionVector>(Position(5,3),store);
    CPPUNIT_ASSERT(store.isMember(Position(4,2)));
    CPPUNIT_ASSERT(store.isMember(Position(2,6)));
    // block
    CPPUNIT_ASSERT(!store.isMember(Position(3,1)));
    CPPUNIT_ASSERT(!store.isMember(Position(5,1)));
  }
  {
    // 53
    StorePositionVector store;
    Piece p=state.getPieceAt(Position(5,3));
    state.forEachEffectOfPiece<StorePositionVector>(p,store);
    CPPUNIT_ASSERT(store.isMember(Position(4,2)));
    CPPUNIT_ASSERT(store.isMember(Position(2,6)));
    // block
    CPPUNIT_ASSERT(!store.isMember(Position(3,1)));
    CPPUNIT_ASSERT(!store.isMember(Position(5,1)));
  }
}

void EffectTest::testPieceHasEffectTo()
{
  {
    // (7,9)
    Piece p=eState.getPieceAt(Position(7,9));
    CPPUNIT_ASSERT(eState.hasEffectTo(p,Position(3,5)));
    CPPUNIT_ASSERT(eState.hasEffectTo(p,Position(8,9)));
    CPPUNIT_ASSERT(eState.hasEffectTo(p,Position(9,7)));
    CPPUNIT_ASSERT(!eState.hasEffectTo(p,Position(7,9)));
    CPPUNIT_ASSERT(!eState.hasEffectTo(p,Position(2,4)));
    CPPUNIT_ASSERT(!eState.hasEffectTo(p,Position(9,9)));
  }
  {
    Piece p=eState.getPieceAt(Position(2,2));
    CPPUNIT_ASSERT(eState.hasEffectTo(p,Position(1,3)));
    CPPUNIT_ASSERT(eState.hasEffectTo(p,Position(1,1)));
    CPPUNIT_ASSERT(!eState.hasEffectTo(p,Position(2,2)));
    CPPUNIT_ASSERT(!eState.hasEffectTo(p,Position(4,4)));
  }

  {
    NumEffectState state(CsaString(
			   "P1+NY+TO *  *  *  * -OU-KE-KY\n"
			   "P2 *  *  *  *  * -GI-KI *  *\n"
			   "P3 * +RY *  * +UM * -KI-FU-FU\n"
			   "P4 *  * +FU-FU *  *  *  *  *\n"
			   "P5 *  * -KE * +FU *  * +FU *\n"
			   "P6-KE *  * +FU+GI-FU *  * +FU\n"
			   "P7 *  * -UM *  *  *  *  *  *\n"
			   "P8 *  *  *  *  *  *  *  *  * \n"
			   "P9 * +OU * -GI *  *  *  * -NG\n"
			   "P+00HI00KI00KE00KY00FU00FU00FU00FU00FU00FU\n"
			   "P-00KI00KY00FU00FU\n"
			   "P-00AL\n"
			   "+\n").getInitialState());
    
    {
      // (7,7)
      Piece p=state.getPieceAt(Position(7,7));
      CPPUNIT_ASSERT(state.hasEffectTo(p,Position(9,5)));
      CPPUNIT_ASSERT(state.hasEffectTo(p,Position(7,6)));
      CPPUNIT_ASSERT(state.hasEffectTo(p,Position(6,6)));
      CPPUNIT_ASSERT(!state.hasEffectTo(p,Position(5,5)));
      CPPUNIT_ASSERT(!state.hasEffectTo(p,Position(7,5)));
      CPPUNIT_ASSERT(!state.hasEffectTo(p,Position(7,7)));
    }
    {
      Piece p=state.getPieceAt(Position(1,1));
      CPPUNIT_ASSERT(state.hasEffectTo(p,Position(1,3)));
      CPPUNIT_ASSERT(!state.hasEffectTo(p,Position(1,1)));
      CPPUNIT_ASSERT(!state.hasEffectTo(p,Position(2,3)));
      CPPUNIT_ASSERT(!state.hasEffectTo(p,Position(1,4)));
    }
  }
}

void EffectTest::testForeachEffect()
{
  PieceVector myPieces;
  StorePiece myPiecesStore(&myPieces);
  {
    Position to=Position(2,8);
    eState.forEachEffect<BLACK,StorePiece>
      (to, myPiecesStore);
    CPPUNIT_ASSERT(myPieces.isMember(eState.getPieceAt(Position(1,9))));
    CPPUNIT_ASSERT(myPieces.isMember(eState.getPieceAt(Position(3,8))));
    CPPUNIT_ASSERT(!myPieces.isMember(eState.getPieceAt(Position(2,8))));
  }
}

void EffectTest::testForeachEffectPtype()
{
  PieceVector myPieces;
  StorePiece myPiecesStore(&myPieces);
  {
    Position to=Position(2,8);
    eState.forEachEffectPtype<BLACK,GOLD,StorePiece>
      (to, myPiecesStore);
    CPPUNIT_ASSERT(!myPieces.isMember(eState.getPieceAt(Position(1,9))));
    CPPUNIT_ASSERT(myPieces.isMember(eState.getPieceAt(Position(3,8))));
    CPPUNIT_ASSERT(!myPieces.isMember(eState.getPieceAt(Position(2,8))));
  }
}

void EffectTest::testForeachEffectNotBy()
{
  PieceVector myPieces;
  StorePiece myPiecesStore(&myPieces);
  {
    Position to=Position(2,8);
    Piece p=eState.getPieceAt(Position(1,9));
    eState.forEachEffectNotBy<BLACK,StorePiece>
      (to, p, myPiecesStore);
    CPPUNIT_ASSERT(!myPieces.isMember(eState.getPieceAt(Position(1,9))));
    CPPUNIT_ASSERT(myPieces.isMember(eState.getPieceAt(Position(3,8))));
    CPPUNIT_ASSERT(!myPieces.isMember(eState.getPieceAt(Position(2,8))));
  }
}

void EffectTest::testForeachEffectOfPiece()
{
  NumEffectState state1(CsaString(
		       "P1+NY * +TO *  *  * -OU-KE-KY\n"
		       "P2 *  *  *  *  * -GI-KI *  * \n"
		       "P3 * +RY *  * +UM * -KI-FU-FU\n"
		       "P4 *  * +FU-FU *  * -FU *  * \n"
		       "P5 *  * -KE * +FU * +KY+FU * \n"
		       "P6-KE *  * +FU+GI-FU *  * +FU\n"
		       "P7 *  * -UM *  *  *  *  *  * \n"
		       "P8 *  *  *  *  *  *  *  *  * \n"
		       "P9 * +OU * -GI *  *  *  * -NG\n"
		       "P+00FU\n"
		       "P+00FU\n"
		       "P+00FU\n"
		       "P+00FU\n"
		       "P+00FU\n"
		       "P+00FU\n"
		       "P+00FU\n"
		       "P+00KE\n"
		       "P+00KI\n"
		       "P-00KI\n"
		       "P-00KY\n"
		       "P+00HI\n"
		       "+\n"
		       ).getInitialState());
  state_t ne_state(state);
  PieceVector myPieces;
  StorePiece myPiecesStore(&myPieces);
  ne_state.forEachEffectOfPiece<StorePiece>(ne_state.getPieceAt(Position(3,5)),myPiecesStore);
  CPPUNIT_ASSERT(!myPieces.isMember(eState.getPieceAt(Position(1,3))));
}

void EffectTest::testForSortedEffect()
{
}

void EffectTest::testHasEffectBy()
{
  // 7-3 ˤϸ롥
  CPPUNIT_ASSERT((eState.hasEffectBy(WHITE,Position(7,3))));
  // 7-1 ˤϸʤ
  CPPUNIT_ASSERT(!(eState.hasEffectBy(WHITE,Position(7,1))));
  // 9-5 ˤ롥
  CPPUNIT_ASSERT((eState.hasEffectBy(BLACK,Position(9,5))));
  // 9-8 ˤϸʤ
  CPPUNIT_ASSERT(!(eState.hasEffectBy(BLACK,Position(9,8))));
}

void EffectTest::testHasEffectByWithRemove()
{
  {
    NumEffectState state1(CsaString(
			 "P1-KY+RY *  *  *  *  *  * -KY\n"
			 "P2 *  *  *  * +UM * +NK *  * \n"
			 "P3-FU-OU-GI-FU-FU-FU *  * -FU\n"
			 "P4 *  * -FU *  *  *  *  *  * \n"
			 "P5 *  *  *  * +KA *  *  *  * \n"
			 "P6 *  *  *  *  *  *  *  *  * \n"
			 "P7+FU * +FU+FU+FU+FU+FU * +FU\n"
			 "P8 *  * -NK * +OU *  *  *  * \n"
			 "P9+KY+KE * -HI * +KI+GI * +KY\n"
			 "P-00FU\n"
			 "P+00FU\n"
			 "P+00FU\n"
			 "P-00FU\n"
			 "P-00FU\n"
			 "P+00KE\n"
			 "P-00GI\n"
			 "P-00GI\n"
			 "P+00KI\n"
			 "P-00KI\n"
			 "P-00KI\n"
			 "-\n"
			 ).getInitialState());
    NumEffectState sState(state1);
    // 8-4 ˤ8-3롥
    CPPUNIT_ASSERT((sState.
		    hasEffectByWithRemove<BLACK>(Position(8,4),Position(8,3))));
    // 8-9 ˤ8-3롥
    CPPUNIT_ASSERT((sState.
		    hasEffectByWithRemove<BLACK>(Position(8,9),Position(8,3))));
  }
  {
    NumEffectState state1(CsaString(
			 "P1-KY-KE+GI+KA *  * +RY * -KY\n"
			 "P2 *  * -OU * -KI * +NK *  * \n"
			 "P3-FU * -GI-FU-FU-FU *  * -FU\n"
			 "P4 *  * -FU *  *  *  *  *  * \n"
			 "P5 *  *  *  * +KA *  *  *  * \n"
			 "P6 *  *  *  *  *  *  *  *  * \n"
			 "P7+FU * +FU+FU+FU+FU+FU * +FU\n"
			 "P8 *  * -NK * +OU *  *  *  * \n"
			 "P9+KY+KE * -HI * +KI+GI * +KY\n"
			 "P+00FU\n"
			 "P+00FU\n"
			 "P+00FU\n"
			 "P-00FU\n"
			 "P-00FU\n"
			 "P-00GI\n"
			 "P-00KI\n"
			 "P-00KI\n"
			 "-\n"
			 ).getInitialState());
    NumEffectState sState(state1);
    // 8-3 ˤ7-2롥
    CPPUNIT_ASSERT((sState.
		    hasEffectByWithRemove<BLACK>(Position(8,3),Position(7,2))));
  }
}

void EffectTest::testHasEffectNotBy()
{
  // 9-3 ϸι(9-1),(8-1)롥
  CPPUNIT_ASSERT((eState.hasEffectNotBy(WHITE,eState.getPieceAt(Position(9,1)),Position(9,3))));
  // 9-2 ϸι(9-1)Τ
  CPPUNIT_ASSERT(!(eState.hasEffectNotBy(WHITE,eState.getPieceAt(Position(9,1)),Position(9,2))));
}

void EffectTest::testHasMultipleEffectBy()
{
  // 9-3 ˤϸʣ롥
  CPPUNIT_ASSERT((eState.hasMultipleEffectBy(WHITE,Position(9,3))));
  // 5-7 ˤϸʣʤ
  CPPUNIT_ASSERT(!(eState.hasMultipleEffectBy(WHITE,Position(5,7))));
  // 2-8 ˤʣ롥
  CPPUNIT_ASSERT((eState.hasMultipleEffectBy(BLACK,Position(2,8))));
  // 2-3 ˤʣʤ
  CPPUNIT_ASSERT(!(eState.hasMultipleEffectBy(BLACK,Position(2,3))));
}

void EffectTest::testHasEffectPiece()
{
  {
    Piece p=eState.hasEffectPiece(WHITE,Position(5,7));
    CPPUNIT_ASSERT(p==eState.getPieceAt(Position(7,9)));
  }
  {
    // 2,3ؤϤɤǤɤ
    Piece p=eState.hasEffectPiece(WHITE,Position(2,3));
    CPPUNIT_ASSERT(p==eState.getPieceAt(Position(2,2)) ||
		   p==eState.getPieceAt(Position(3,2)));
  }
  {
    //ʤϸƤɤ
    Piece p=eState.hasEffectPiece(WHITE,Position(5,9));
    CPPUNIT_ASSERT(p==Piece::EMPTY());
  }
}

void EffectTest::testHasEffectDir()
{
  CPPUNIT_ASSERT((eState.hasEffectDir<LONG_DR,WHITE>(Position(9,7))));
  CPPUNIT_ASSERT((eState.hasEffectDir<LONG_UL,BLACK>(Position(7,7))));
  CPPUNIT_ASSERT((! eState.hasEffectDir<LONG_UL,BLACK>(Position(4,8))));
}

void EffectTest::testHasEffectByPtype()
{
  {
    NumEffectState state(CsaString(
			   "P1-KY-KE * -KI * -OU * -KE-KY\n"
			   "P2 * -HI *  *  *  * -KI *  * \n"
			   "P3-FU *  * -FU-FU-FU * -FU-FU\n"
			   "P4 *  * -FU * -GI * -FU-GI * \n"
			   "P5 * -FU *  *  *  *  *  *  * \n"
			   "P6 *  * +FU+FU *  *  *  *  * \n"
			   "P7+FU+FU+KE *  * +FU+FU * +FU\n"
			   "P8 *  * +KI+GI * +GI * +HI+KA\n"
			   "P9+KY *  * +OU * +KI * +KE+KY\n"
			   "P+00FU\n"
			   "P-00KA00FU\n"
			   "+\n").getInitialState());
    CPPUNIT_ASSERT(state.hasEffectByPtype<BISHOP>(BLACK, Position(5,4)));
    CPPUNIT_ASSERT(! state.hasEffectByPtype<BISHOP>(WHITE, Position(5,4)));
    CPPUNIT_ASSERT(! state.hasEffectByPtype<BISHOP>(BLACK, Position(5,5)));
    CPPUNIT_ASSERT(! state.hasEffectByPtype<BISHOP>(WHITE, Position(5,5)));
  }
  {
    const NumEffectState state(CsaString(
			   "P1-OU *  *  *  *  *  *  *  * \n"
			   "P2 *  *  *  *  *  *  * +GI * \n"
			   "P3 *  *  *  *  * +NK * +NG * \n"
			   "P4 *  *  * +KE *  *  *  *  * \n"
			   "P5 *  *  *  *  * +TO *  *  * \n"
			   "P6 *  *  * +FU+FU *  *  *  * \n"
			   "P7 * +NY *  *  * +UM *  *  * \n"
			   "P8 *  *  *  *  * +KA+RY *  * \n"
			   "P9 *  * +KY *  * +HI *  * +OU\n"
			   "P-00KI00KI00KI00KI00GI00GI00KE00KE00KY00KY00FU00FU00FU00FU00FU00FU00FU00FU00FU00FU00FU00FU00FU00FU00FU\n"
			   "+\n").getInitialState());
    CPPUNIT_ASSERT(state.isConsistent(true));

    CPPUNIT_ASSERT(state.hasEffectByPtypeStrict<PAWN>(BLACK, Position(6,5)));
    CPPUNIT_ASSERT(!state.hasEffectByPtypeStrict<PPAWN>(BLACK, Position(6,5)));
    CPPUNIT_ASSERT(state.hasEffectByPtypeStrict<PAWN>(BLACK, Position(5,5)));
    CPPUNIT_ASSERT(state.hasEffectByPtypeStrict<PPAWN>(BLACK, Position(5,5)));
    CPPUNIT_ASSERT(state.hasEffectByPtypeStrict<PPAWN>(BLACK, Position(4,6)));
    CPPUNIT_ASSERT(!state.hasEffectByPtypeStrict<PAWN>(BLACK, Position(4,6)));

    CPPUNIT_ASSERT(state.hasEffectByPtypeStrict<KNIGHT>(BLACK, Position(5,2)));
    CPPUNIT_ASSERT(state.hasEffectByPtypeStrict<PKNIGHT>(BLACK, Position(5,2)));
    CPPUNIT_ASSERT(state.hasEffectByPtypeStrict<KNIGHT>(BLACK, Position(7,2)));
    CPPUNIT_ASSERT(!state.hasEffectByPtypeStrict<PKNIGHT>(BLACK, Position(7,2)));
    CPPUNIT_ASSERT(!state.hasEffectByPtypeStrict<KNIGHT>(BLACK, Position(4,2)));
    CPPUNIT_ASSERT(state.hasEffectByPtypeStrict<PKNIGHT>(BLACK, Position(4,2)));
  }
}



static void testFile(const std::string& fileName)
{
  Record rec=CsaFile(fileName).getRecord();
  NumEffectState state=rec.getInitialState();
  state_t ne_state(state);
  const vector<osl::Move> moves=rec.getMoves();

  DoUndoMoveLockTest<state_t> lockTest(ne_state, moves);
  lockTest.run();

  size_t depth=0;
  if (moves.begin()->player()==BLACK)
  {
    RecursiveDoUndoMove<BLACK,state_t>
      func(ne_state, &*moves.begin(), &*moves.end(), &depth);
    ApplyMove<BLACK>::doUndoMove(ne_state, *moves.begin(), func);
  }
  else{
    RecursiveDoUndoMove<WHITE,state_t>
      func(ne_state, &*moves.begin(), &*moves.end(), &depth);
    ApplyMove<WHITE>::doUndoMove(ne_state, *moves.begin(), func);
  }
  CPPUNIT_ASSERT_EQUAL(moves.size(), depth);
}

void EffectTest::testCsaFiles()
{
  extern bool isShortTest;
  std::ifstream ifs(OslConfig::testCsaFile("FILES"));
  CPPUNIT_ASSERT(ifs);
  int i=0;
  int count=100;
  if (isShortTest)
    count=10;
  std::string fileName;
  while ((ifs >> fileName) && (++i<count)) {
    if (fileName == "") 
      break;
    if (! isShortTest)
      std::cerr << fileName << " ";
    testFile(OslConfig::testCsaFile(fileName));
  }
}

class MyAction
{
  vector<Piece> pieces;
public:
  void doAction(state_t& /*state*/,Piece p,Position /*pos*/)
  {
    pieces.push_back(p);
  }
  vector<Piece> getPieces() const{
    return pieces;
  }
};
std::ostream& operator<<(std::ostream& os,MyAction const& a)
{
  vector<Piece> pieces=a.getPieces();
  os << "MyAction: " << std::endl;
  for (size_t i=0;i<pieces.size();i++)
    os << i << " : " << pieces[i] << std::endl;
  return os << std::endl;
}
  
template <Player P>
bool kingCanBeCaptured(state_t& s)
{
  const int kingIndex=Ptype_Table.getKingIndex(P);
  const Position kingPosition=s.getPieceOf(kingIndex).position();
  return s.hasEffectBy(PlayerTraits<P>::opponent, kingPosition);
}
bool kingCanBeCaptured(Player P, state_t& s)
{
  if (P==BLACK)
    return kingCanBeCaptured<BLACK>(s);
  else
    return kingCanBeCaptured<WHITE>(s);
}

void EffectTest::testCountEffect2()
{
  NumEffectState state=CsaString(
    "P1-KY-KE * -KI-OU-KI-GI-KE-KY\n"
    "P2 * -HI *  *  *  *  * -KA * \n"
    "P3-FU-FU * -FU-FU-FU-FU-FU-FU\n"
    "P4 *  * -FU-GI *  *  *  *  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  * +FU *  *  *  *  *  * \n"
    "P7+FU+FU * +FU+FU+FU+FU+FU+FU\n"
    "P8 * +KA+KI *  *  *  * +HI * \n"
    "P9+KY+KE+GI * +OU+KI+GI+KE+KY\n"
    "+\n").getInitialState();
  state_t ne_state(state);
  CPPUNIT_ASSERT_EQUAL(2, ne_state.countEffect2(BLACK, Position(7,7)));
  CPPUNIT_ASSERT_EQUAL(2, ne_state.countEffect2(BLACK, Position(6,6)));
  CPPUNIT_ASSERT_EQUAL(1, ne_state.countEffect2(BLACK, Position(5,6)));
  CPPUNIT_ASSERT_EQUAL(0, ne_state.countEffect2(BLACK, Position(6,5)));
  
  CPPUNIT_ASSERT_EQUAL(2, ne_state.countEffect2(WHITE, Position(7,3)));
  CPPUNIT_ASSERT_EQUAL(2, ne_state.countEffect2(WHITE, Position(7,5)));
  CPPUNIT_ASSERT_EQUAL(1, ne_state.countEffect2(WHITE, Position(6,5)));
  CPPUNIT_ASSERT_EQUAL(1, ne_state.countEffect2(WHITE, Position(5,5)));
  CPPUNIT_ASSERT_EQUAL(0, ne_state.countEffect2(WHITE, Position(4,5)));
}

void EffectTest::testCountEffect()
{
  NumEffectState state=CsaString(
    "P1-KY-KE * -KI-OU-KI-GI-KE-KY\n"
    "P2 * -HI *  *  *  *  * -KA * \n"
    "P3-FU-FU * -FU-FU-FU-FU-FU-FU\n"
    "P4 *  * -FU-GI *  *  *  *  * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  * +FU *  *  *  *  *  * \n"
    "P7+FU+FU * +FU+FU+FU+FU+FU+FU\n"
    "P8 * +KA+KI *  *  *  * +HI * \n"
    "P9+KY+KE+GI * +OU+KI+GI+KE+KY\n"
    "+\n").getInitialState();
  state_t ne_state(state);
  CPPUNIT_ASSERT_EQUAL(3, ne_state.countEffect(BLACK, Position(7,7)));
  CPPUNIT_ASSERT_EQUAL(2, ne_state.countEffect(BLACK, Position(6,6)));
  CPPUNIT_ASSERT_EQUAL(1, ne_state.countEffect(BLACK, Position(5,6)));
  CPPUNIT_ASSERT_EQUAL(0, ne_state.countEffect(BLACK, Position(6,5)));
  
  CPPUNIT_ASSERT_EQUAL(2, ne_state.countEffect(WHITE, Position(7,3)));
  CPPUNIT_ASSERT_EQUAL(2, ne_state.countEffect(WHITE, Position(7,5)));
  CPPUNIT_ASSERT_EQUAL(1, ne_state.countEffect(WHITE, Position(6,5)));
  CPPUNIT_ASSERT_EQUAL(1, ne_state.countEffect(WHITE, Position(5,5)));
  CPPUNIT_ASSERT_EQUAL(0, ne_state.countEffect(WHITE, Position(4,5)));
  CPPUNIT_ASSERT_EQUAL(4, ne_state.countEffect(WHITE, Position(5,2)));
}

void EffectTest::testCountEffectPin()
{
  NumEffectState state=CsaString(
    "P1-KY * -KI * -KE *  *  * +RY\n"
    "P2 * -OU * -GI+GI *  *  *  * \n"
    "P3 * -GI * -KI *  *  *  *  * \n"
    "P4-FU+KE-FU-FU-FU * +KA * -FU\n"
    "P5 * -FU *  *  * +FU *  *  * \n"
    "P6+FU * +FU+FU+FU *  *  * +FU\n"
    "P7 * +FU *  *  * +GI *  *  * \n"
    "P8 * +OU+KI-RY *  * +FU *  * \n"
    "P9+KY *  *  *  *  *  *  * +KY\n"
    "P+00KA00KE00KY00FU00FU00FU\n"
    "P-00KI00KE00FU\n"
    "-\n").getInitialState();
  state_t ne_state(state);
  CPPUNIT_ASSERT_EQUAL(2, ne_state.countEffect(BLACK, Position(7,7)));
  PieceMask pin;
  pin.set(ne_state.getPieceOnBoard(Position(7,8)).number());
  CPPUNIT_ASSERT_EQUAL(1, ne_state.countEffect(BLACK, Position(7,7), pin));
}

void EffectTest::testCountKingEffect()
{
  NumEffectState state=CsaString(
    "P1-KY-KE * -KI * -KI-GI-KE-KY\n"
    "P2 * -HI *  *  *  *  * -KA * \n"
    "P3-FU-FU * -FU-FU-FU-FU-FU-FU\n"
    "P4 *  * -FU-GI *  *  * -OU * \n"
    "P5 *  *  *  *  *  *  *  *  * \n"
    "P6 *  * +FU *  *  *  *  *  * \n"
    "P7+FU+FU * +FU+FU+FU+FU+FU+FU\n"
    "P8 * +KA+KI *  *  *  * +HI * \n"
    "P9+KY+KE+GI * +OU+KI+GI+KE+KY\n"
    "+\n").getInitialState();
  state_t ne_state(state);
  CPPUNIT_ASSERT_EQUAL(0, ne_state.countKingEffect(BLACK, Position(6,5)));
  CPPUNIT_ASSERT_EQUAL(1, ne_state.countKingEffect(BLACK, Position(6,9)));
  CPPUNIT_ASSERT_EQUAL(2, ne_state.countKingEffect(BLACK, Position(4,8)));
  CPPUNIT_ASSERT_EQUAL(2, ne_state.countKingEffect(BLACK, Position(3,8)));
  
  CPPUNIT_ASSERT_EQUAL(0, ne_state.countKingEffect(WHITE, Position(2,6)));
  CPPUNIT_ASSERT_EQUAL(1, ne_state.countKingEffect(WHITE, Position(2,5)));
  CPPUNIT_ASSERT_EQUAL(2, ne_state.countKingEffect(WHITE, Position(3,4)));
  CPPUNIT_ASSERT_EQUAL(2, ne_state.countKingEffect(WHITE, Position(4,4)));
}

void EffectTest::testKingCanBeCaptured()
{
  NumEffectState state=CsaString(
    "P1+NY+TO *  *  *  * -OU-KE-KY\n"
    "P2 *  *  *  *  * -GI-KI *  *\n"
    "P3 * +RY *  * +UM * -KI-FU-FU\n"
    "P4 *  * +FU-FU *  *  *  *  *\n"
    "P5 *  * -KE * +FU *  * +FU *\n"
    "P6-KE *  * +FU+GI-FU *  * +FU\n"
    "P7 *  * -UM *  *  *  *  *  *\n"
    "P8 *  *  *  *  *  *  *  *  * \n"
    "P9 * +OU * -GI *  *  *  * -NG\n"
    "P+00HI00KI00KE00KY00FU00FU00FU00FU00FU00FU\n"
    "P-00KI00KY00FU00FU\n"
    "P-00AL\n"
    "+\n").getInitialState();
  state_t ne_state(state);
  CPPUNIT_ASSERT(! kingCanBeCaptured(BLACK,ne_state));
  CPPUNIT_ASSERT(! kingCanBeCaptured(WHITE,ne_state));
  ApplyMoveOfTurn::doMove(ne_state, Move(Position(8,1),Position(7,1),PPAWN,PTYPE_EMPTY,false,BLACK));
  CPPUNIT_ASSERT(! kingCanBeCaptured(BLACK,ne_state));
  CPPUNIT_ASSERT(! kingCanBeCaptured(WHITE,ne_state));
  ApplyMoveOfTurn::doMove(ne_state, Move(Position::STAND(),Position(8,8),PAWN,PTYPE_EMPTY,false,WHITE));
  CPPUNIT_ASSERT(kingCanBeCaptured(BLACK,ne_state));
  CPPUNIT_ASSERT(! kingCanBeCaptured(WHITE,ne_state));
  ApplyMoveOfTurn::doMove(ne_state, Move(Position(7,1),Position(8,1),PPAWN,PTYPE_EMPTY,false,BLACK));
  CPPUNIT_ASSERT(kingCanBeCaptured(BLACK,ne_state));
  CPPUNIT_ASSERT(! kingCanBeCaptured(WHITE,ne_state));
}

void EffectTest::testEffectPtype()
{
  NumEffectState state(CsaString(
			 "P1-KY-KE-GI-KI-OU-KI-GI-KE-KY\n"
			 "P2 * -HI *  *  *  *  * -KA * \n"
			 "P3-FU-FU-FU-FU-FU-FU * -FU-FU\n"
			 "P4 *  *  *  *  *  * -FU *  * \n"
			 "P5 *  *  *  *  *  *  *  *  * \n"
			 "P6 *  * +FU *  *  *  *  *  * \n"
			 "P7+FU+FU * +FU+FU+FU+FU+FU+FU\n"
			 "P8 * +KA *  *  *  *  * +HI * \n"
			 "P9+KY+KE+GI+KI+OU+KI+GI+KE+KY\n"
			 "+\n").getInitialState());
  CPPUNIT_ASSERT_EQUAL(Position(2,2), state.effectPtype<BISHOP>(WHITE, Position(6,6)).position());
}
void EffectTest::testChangedEffect()
{
  extern bool isShortTest;
  std::ifstream ifs(OslConfig::testCsaFile("FILES"));
  CPPUNIT_ASSERT(ifs);
  int i=0;
  int count=100;
  if (isShortTest)
    count=10;
  std::string fileName;
  while ((ifs >> fileName) && (++i<count)) {
    if (fileName == "") 
      break;
    if (! isShortTest)
      std::cerr << fileName << " ";
    Record rec=CsaFile(OslConfig::testCsaFile(fileName)).getRecord();
    NumEffectState state(rec.getInitialState());
    const vector<osl::Move> moves=rec.getMoves();
    for(size_t i=0;i<moves.size();i++){
      PieceMask before[9][9][2];
      {
	const NumEffectState& ne_state=state;
	for(int x=1;x<=9;x++)
	  for(int y=1;y<=9;y++){
	    const Position pos(x,y);
	    for (int z=0; z<2; ++z) {
	      const Player pl = indexToPlayer(z);
	      before[x-1][y-1][z]=
		ne_state.getEffect(pos)&ne_state.getOnBoardMask(pl);
	    }
	  }
      }
      ApplyMoveOfTurn::doMove(state,moves[i]);
      PieceMask after[9][9][2];
      {
	const NumEffectState& ne_state=state;
	for(int x=1;x<=9;x++)
	  for(int y=1;y<=9;y++){
	    const Position pos(x,y);
	    for (int z=0; z<2; ++z) {
	      const Player pl = indexToPlayer(z);
	      after[x-1][y-1][z]=
		ne_state.getEffect(pos)&ne_state.getOnBoardMask(pl);
	    }
	  }
	for(int x=1;x<=9;x++)
	  for(int y=1;y<=9;y++){
	    const Position pos(x,y);
	    for (int z=0; z<2; ++z) {
	      const Player pl = indexToPlayer(z);
	      PieceMask b=before[x-1][y-1][z];
	      PieceMask a=after[x-1][y-1][z];
	      if(a!=b || (pos==moves[i].to() && moves[i].capturePtype()!=PTYPE_EMPTY)){
		for(int num=0;num<48;num++){
		  if(b.test(num)!=a.test(num) || 
		     (pos==moves[i].to() && moves[i].capturePtype()!=PTYPE_EMPTY && (b.test(num) || a.test(num)))){
		    CPPUNIT_ASSERT(ne_state.changedPieces().test(num));
		  }
		}
	      }
	      if(ne_state.changedEffects(pl).test(pos)){
		Piece p=ne_state.getPieceAt(moves[i].to());
		if(b==a){
		  if(!a.test(p.number())){
		    std::cerr << std::endl << state << ",move=" << moves[i] << ",x=" << x << ",y=" << y << std::endl;
		    std::cerr << "changed[" << pl << "]=" << std::endl << ne_state.changedEffects(pl);
		  }
		}
	      }
	      else{
		if(b!=a){
		  std::cerr << state << ",move=" << moves[i] << ",x=" << x << ",y=" << y << std::endl;
		  std::cerr << "before " << b << "\nafter " << a << "\n";
		  std::cerr << "changed[ " << pl << "]=" << std::endl << ne_state.changedEffects(pl);
		}
		CPPUNIT_ASSERT_EQUAL(b, a);
	      }
	    }
	  }
      }
    }
  }
}

void EffectTest::testSelectLong()
{
  NumEffectState state(CsaString(
			 "P1-KY-KE-GI-KI-OU-KI-GI-KE-KY\n"
			 "P2 * -HI *  *  *  *  * -KA * \n"
			 "P3-FU-FU-FU-FU-FU-FU * -FU-FU\n"
			 "P4 *  *  *  *  *  * -FU *  * \n"
			 "P5 *  *  *  *  *  *  *  *  * \n"
			 "P6 *  * +FU *  *  *  *  *  * \n"
			 "P7+FU+FU * +FU+FU+FU+FU+FU+FU\n"
			 "P8 * +KA *  *  *  *  * +HI * \n"
			 "P9+KY+KE+GI+KI+OU+KI+GI+KE+KY\n"
			 "+\n").getInitialState());
  CPPUNIT_ASSERT(state.selectLong<LANCE>(Position(9,2)).any());
  CPPUNIT_ASSERT(state.selectLong<ROOK>(Position(9,2)).any());
  CPPUNIT_ASSERT(state.selectLong(Position(9,2), WHITE).countBit()==2);
}

static void testEffectedState(NumEffectState const& state,Move move)
{
  PieceMask b_mask=state.effectedMask(BLACK);
  PieceMask w_mask=state.effectedMask(WHITE);
  for(int num=0;num<40;num++){
    Piece p=state.getPieceOf(num);
    if(p.isOnBoard()){
      Position pos=p.position();
      if(b_mask.test(num)){
	if(!state.hasEffectBy(BLACK,pos)){
	  std::cerr << std::endl << state << std::endl;
	  std::cerr << "b_mask=" << b_mask << ",num=" << num << ",pos=" << pos << ",move=" << move << std::endl;
	}
	CPPUNIT_ASSERT(state.hasEffectBy(BLACK,pos));
      }
      else{
	if(state.hasEffectBy(BLACK,pos)){
	  std::cerr << std::endl << state << std::endl;
	  std::cerr << "b_mask=" << b_mask << ",num=" << num << ",pos=" << pos << ",move=" << move << std::endl;
	}
	CPPUNIT_ASSERT(!state.hasEffectBy(BLACK,pos));
      }
      if(w_mask.test(num)){
	if(!state.hasEffectBy(WHITE,pos)){
	  std::cerr << std::endl << state << std::endl;
	  std::cerr << "w_mask=" << w_mask << ",num=" << num << ",pos=" << pos << ",move=" << move << std::endl;
	}
	CPPUNIT_ASSERT(state.hasEffectBy(WHITE,pos));
      }
      else{
	if(state.hasEffectBy(WHITE,pos)){
	  std::cerr << std::endl << state << std::endl;
	  std::cerr << "w_mask=" << w_mask << ",num=" << num << ",pos=" << pos << ",move=" << move << std::endl;
	}
	CPPUNIT_ASSERT(!state.hasEffectBy(WHITE,pos));
      }
    }
  }
}

void EffectTest::testEffected()
{
  NumEffectState state=CsaString(
    "P1+NY+TO *  *  *  * -OU-KE-KY\n"
    "P2 *  *  *  *  * -GI-KI *  *\n"
    "P3 * +RY *  * +UM * -KI-FU-FU\n"
    "P4 *  * +FU-FU *  *  *  *  *\n"
    "P5 *  * -KE * +FU *  * +FU *\n"
    "P6-KE *  * +FU+GI-FU *  * +FU\n"
    "P7 *  * -UM *  *  *  *  *  *\n"
    "P8 *  *  *  *  *  *  *  *  * \n"
    "P9 * +OU * -GI *  *  *  * -NG\n"
    "P+00HI00KI00KE00KY00FU00FU00FU00FU00FU00FU\n"
    "P-00KI00KY00FU00FU\n"
    "P-00AL\n"
    "+\n").getInitialState();
  testEffectedState(state,Move::INVALID());
  { // simple move
    NumEffectState state1=state;
    Move move(Position(8,3),Position(8,2),PROOK,PTYPE_EMPTY,false,BLACK);
    ApplyMoveOfTurn::doMove(state1, move);
    testEffectedState(state1,move);
  }
  { // drop move
    NumEffectState state1=state;
    Move move(Position(8,2),ROOK,BLACK);
    ApplyMoveOfTurn::doMove(state1,move);
    testEffectedState(state1,move);
  }
  { // capture move
    NumEffectState state1=state;
    Move move(Position(5,3),Position(4,2),PBISHOP,SILVER,false,BLACK);
    ApplyMoveOfTurn::doMove(state1, move);
    testEffectedState(state1,move);
  }
  extern bool isShortTest;
  std::ifstream ifs(OslConfig::testCsaFile("FILES"));
  CPPUNIT_ASSERT(ifs);
  int i=0;
  int count=100;
  if (isShortTest)
    count=10;
  std::string fileName;
  while ((ifs >> fileName) && (++i<count)) {
    if (fileName == "") 
      break;
    if (! isShortTest)
      std::cerr << fileName << " ";
    Record rec=CsaFile(OslConfig::testCsaFile(fileName)).getRecord();
    NumEffectState state(rec.getInitialState());
    const vector<osl::Move> moves=rec.getMoves();
    for(size_t i=0;i<moves.size();i++){
      ApplyMoveOfTurn::doMove(state,moves[i]);
      testEffectedState(state,moves[i]);
    }
  }
}

void EffectTest::testKingMobility()
{
  NumEffectState state=CsaString(
    "P1+NY+TO *  *  *  * -OU-KE-KY\n"
    "P2 *  *  *  *  * -GI-KI *  *\n"
    "P3 * +RY *  * +UM * -KI-FU-FU\n"
    "P4 *  * +FU-FU *  *  *  *  *\n"
    "P5 *  * -KE * +FU *  * +FU *\n"
    "P6-KE *  * +FU+GI-FU *  * +FU\n"
    "P7 *  * -UM *  *  *  *  *  *\n"
    "P8 *  *  *  *  *  *  *  *  * \n"
    "P9 * +OU * -GI *  *  *  * -NG\n"
    "P+00HI00KI00KE00KY00FU00FU00FU00FU00FU00FU\n"
    "P-00KI00KY00FU00FU\n"
    "P-00AL\n"
    "+\n").getInitialState();
  CPPUNIT_ASSERT_EQUAL(Position(8, 10), state.kingMobilityAbs(BLACK, U));
  CPPUNIT_ASSERT_EQUAL(Position(8, 3), state.kingMobilityAbs(BLACK, D));
  CPPUNIT_ASSERT_EQUAL(Position(6, 9), state.kingMobilityAbs(BLACK, L));
  CPPUNIT_ASSERT_EQUAL(Position(10, 9), state.kingMobilityAbs(BLACK, R));
  CPPUNIT_ASSERT_EQUAL(Position(5, 6), state.kingMobilityAbs(BLACK, DL));
  CPPUNIT_ASSERT_EQUAL(Position(7, 10), state.kingMobilityAbs(BLACK, UL));
  CPPUNIT_ASSERT_EQUAL(Position(10, 7), state.kingMobilityAbs(BLACK, DR));
  CPPUNIT_ASSERT_EQUAL(Position(9, 10), state.kingMobilityAbs(BLACK, UR));

  CPPUNIT_ASSERT_EQUAL(Position(3, 2), state.kingMobilityAbs(WHITE, U));
  CPPUNIT_ASSERT_EQUAL(Position(3, 0), state.kingMobilityAbs(WHITE, D));
  CPPUNIT_ASSERT_EQUAL(Position(2, 1), state.kingMobilityAbs(WHITE, L));
  CPPUNIT_ASSERT_EQUAL(Position(8, 1), state.kingMobilityAbs(WHITE, R));
  CPPUNIT_ASSERT_EQUAL(Position(4, 2), state.kingMobilityAbs(WHITE, UR));
  CPPUNIT_ASSERT_EQUAL(Position(4, 0), state.kingMobilityAbs(WHITE, DR));
  CPPUNIT_ASSERT_EQUAL(Position(1, 3), state.kingMobilityAbs(WHITE, UL));
  CPPUNIT_ASSERT_EQUAL(Position(2, 0), state.kingMobilityAbs(WHITE, DL));

  CPPUNIT_ASSERT_EQUAL(Position(8, 3), state.kingMobilityOfPlyaer(BLACK, U));
  CPPUNIT_ASSERT_EQUAL(Position(8, 10), state.kingMobilityOfPlyaer(BLACK, D));
  CPPUNIT_ASSERT_EQUAL(Position(10, 9), state.kingMobilityOfPlyaer(BLACK, L));
  CPPUNIT_ASSERT_EQUAL(Position(6, 9), state.kingMobilityOfPlyaer(BLACK, R));
  CPPUNIT_ASSERT_EQUAL(Position(5, 6), state.kingMobilityOfPlyaer(BLACK, UR));
  CPPUNIT_ASSERT_EQUAL(Position(9, 10), state.kingMobilityOfPlyaer(BLACK, DL));
  CPPUNIT_ASSERT_EQUAL(Position(10, 7), state.kingMobilityOfPlyaer(BLACK, UL));
  CPPUNIT_ASSERT_EQUAL(Position(7, 10), state.kingMobilityOfPlyaer(BLACK, DR));

  CPPUNIT_ASSERT_EQUAL(Position(3, 2), state.kingMobilityOfPlyaer(WHITE, U));
  CPPUNIT_ASSERT_EQUAL(Position(3, 0), state.kingMobilityOfPlyaer(WHITE, D));
  CPPUNIT_ASSERT_EQUAL(Position(2, 1), state.kingMobilityOfPlyaer(WHITE, L));
  CPPUNIT_ASSERT_EQUAL(Position(8, 1), state.kingMobilityOfPlyaer(WHITE, R));
  CPPUNIT_ASSERT_EQUAL(Position(4, 2), state.kingMobilityOfPlyaer(WHITE, UR));
  CPPUNIT_ASSERT_EQUAL(Position(4, 0), state.kingMobilityOfPlyaer(WHITE, DR));
  CPPUNIT_ASSERT_EQUAL(Position(1, 3), state.kingMobilityOfPlyaer(WHITE, UL));
  CPPUNIT_ASSERT_EQUAL(Position(2, 0), state.kingMobilityOfPlyaer(WHITE, DL));
}

void EffectTest::testLongEffectOfDirection()
{
  {
    NumEffectState state=CsaString(
      "P1+NY+TO *  *  *  * -OU-KE-KY\n"
      "P2 *  *  *  *  * -GI-KI *  *\n"
      "P3 * +RY *  * +UM * -KI-FU-FU\n"
      "P4 *  * +FU-FU *  *  *  *  *\n"
      "P5 *  * -KE * +FU *  * +FU *\n"
      "P6-KE *  * +FU+GI-FU *  * +FU\n"
      "P7 *  * -UM *  *  *  *  *  *\n"
      "P8 *  *  *  *  *  *  *  *  * \n"
      "P9 * +OU * -GI *  *  *  * -NG\n"
      "P+00HI00KI00KE00KY00FU00FU00FU00FU00FU00FU\n"
      "P-00KI00KY00FU00FU\n"
      "P-00AL\n"
      "+\n").getInitialState();
    CPPUNIT_ASSERT_EQUAL(Piece::EMPTY(), state.longEffectOfDirection(Position(5,3), UL));
    CPPUNIT_ASSERT_EQUAL(Piece::EMPTY(), state.longEffectOfDirection(Position(5,3), U));
    CPPUNIT_ASSERT_EQUAL(Piece::EMPTY(), state.longEffectOfDirection(Position(5,3), UR));
    CPPUNIT_ASSERT_EQUAL(state.getPieceAt(Position(8,3)), state.longEffectOfDirection(Position(5,3), L));
    CPPUNIT_ASSERT_EQUAL(Piece::EMPTY(), state.longEffectOfDirection(Position(5,3), R));
    CPPUNIT_ASSERT_EQUAL(Piece::EMPTY(), state.longEffectOfDirection(Position(5,3), DL));
    CPPUNIT_ASSERT_EQUAL(Piece::EMPTY(), state.longEffectOfDirection(Position(5,3), D));
    CPPUNIT_ASSERT_EQUAL(Piece::EMPTY(), state.longEffectOfDirection(Position(5,3), R));

    CPPUNIT_ASSERT_EQUAL(Piece::EMPTY(), state.longEffectOfDirection(Position(4,2), UL));
    CPPUNIT_ASSERT_EQUAL(Piece::EMPTY(), state.longEffectOfDirection(Position(4,2), U));
    CPPUNIT_ASSERT_EQUAL(state.getPieceAt(Position(5,3)), state.longEffectOfDirection(Position(4,2), UR));
    CPPUNIT_ASSERT_EQUAL(Piece::EMPTY(), state.longEffectOfDirection(Position(4,2), L));
    CPPUNIT_ASSERT_EQUAL(Piece::EMPTY(), state.longEffectOfDirection(Position(4,2), R));
    CPPUNIT_ASSERT_EQUAL(Piece::EMPTY(), state.longEffectOfDirection(Position(4,2), DL));
    CPPUNIT_ASSERT_EQUAL(Piece::EMPTY(), state.longEffectOfDirection(Position(4,2), D));
    CPPUNIT_ASSERT_EQUAL(Piece::EMPTY(), state.longEffectOfDirection(Position(4,2), R));

    // king?
    CPPUNIT_ASSERT_EQUAL(Piece::EMPTY(), state.longEffectOfDirection(Position(8,3), D));
  }
  {
    NumEffectState state=CsaString(
      "P1-KY-KE+GI+KA *  * +RY * -KY\n"
      "P2 *  * -OU * -KI * +NK *  * \n"
      "P3-FU * -GI-FU-FU-FU *  * -FU\n"
      "P4 *  * -FU *  *  *  *  *  * \n"
      "P5 *  *  *  * +KA *  *  *  * \n"
      "P6 *  *  *  *  *  *  *  *  * \n"
      "P7+FU * +FU+FU+FU+FU+FU * +FU\n"
      "P8 *  * -NK * +OU *  *  *  * \n"
      "P9+KY+KE * -HI * +KI+GI * +KY\n"
      "P+00FU00FU00FU\n"
      "P-00KI00KI00GI00FU00FU\n"
      "-\n").getInitialState();
    CPPUNIT_ASSERT(state.isConsistent());
    CPPUNIT_ASSERT_EQUAL(Piece::EMPTY(), state.longEffectOfDirection(Position(7,2), UR));
    CPPUNIT_ASSERT_EQUAL(state.getPieceAt(Position(6,1)), state.longEffectOfDirection(Position(7,2), DL));
    CPPUNIT_ASSERT_EQUAL(Piece::EMPTY(), state.longEffectOfDirection(Position(3,2), D));
    CPPUNIT_ASSERT_EQUAL(state.getPieceAt(Position(3,1)), state.longEffectOfDirection(Position(3,2), U));
  }
}
// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
