#include "../recursiveDoUndoMove.h"
#include "../doUndoMoveLockTest.h"
#include "osl/record/record.h"
#include "osl/record/csaRecord.h"
#include "osl/record/csaString.h"

#include "osl/state/hashEffectState.h"
#include "osl/oslConfig.h"
#include "osl/stl/hash_map.h"

#include <boost/scoped_array.hpp>
#include <fstream>
#include <iostream>
#include <map>

#include <cppunit/TestCase.h>
#include <cppunit/extensions/HelperMacros.h>

class SimpleHashStateTest : public CppUnit::TestFixture 
{
  CPPUNIT_TEST_SUITE(SimpleHashStateTest);
  CPPUNIT_TEST(testIsPlayerOfTurn);
  CPPUNIT_TEST(testNotEqual);
  CPPUNIT_TEST(testDiff);
  CPPUNIT_TEST(testHashUniqueness);
  CPPUNIT_TEST(testDoUndoMove);
  CPPUNIT_TEST(testMovePass);
  CPPUNIT_TEST_SUITE_END();
 public:
  void testIsPlayerOfTurn();
  void testNotEqual();
  void testDiff();
  void testHashUniqueness();
  void testDoUndoMove();
  void testMovePass();
};

CPPUNIT_TEST_SUITE_REGISTRATION(SimpleHashStateTest);
extern bool isShortTest;
using namespace osl;
using namespace osl::hash;

void SimpleHashStateTest::testIsPlayerOfTurn()
{
  HashEffectState state((SimpleState(HIRATE)));
  CPPUNIT_ASSERT_EQUAL(true, state.getHash().isPlayerOfTurn(BLACK));
  Move m76fu = Move(Position(7,7),Position(7,6),PAWN,
		       PTYPE_EMPTY,false,BLACK);
  ApplyMoveOfTurn::doMove(state, m76fu);
  CPPUNIT_ASSERT_EQUAL(false, state.getHash().isPlayerOfTurn(BLACK));
}

void SimpleHashStateTest::testNotEqual()
{
  HashEffectState hState(CsaString(
"P1-KY *  * -KI-HI *  * -KE-KY\n"
"P2 * -OU-GI *  *  * -KI *  * \n"
"P3-FU-FU *  *  *  *  * -FU-FU\n"
"P4 *  * -FU * +FU * -FU *  * \n"
"P5 *  *  * -FU+HI+FU * +FU * \n"
"P6 *  * +FU-GI *  *  *  *  * \n"
"P7+FU+FU+KI *  *  * +FU * +FU\n"
"P8 * +OU+KI *  *  *  *  *  * \n"
"P9+KY+KE *  *  *  *  * +KE+KY\n"
"P+00FU\n"
"P+00FU\n"
"P+00FU\n"
"P+00KE\n"
"P+00GI\n"
"P+00GI\n"
"P-00KA\n"
"P+00KA\n"
"+\n").getInitialState());
  // Ѥλˤ
  HashEffectState hState2(CsaString(
"P1-KY *  * -KI-HI *  * -KE-KY\n"
"P2 * -OU-GI *  *  * -KI *  * \n"
"P3-FU-FU *  *  *  *  * -FU-FU\n"
"P4 *  * -FU * +FU * -FU *  * \n"
"P5 *  *  * -FU+HI+FU * +FU * \n"
"P6 *  * +FU-GI *  *  *  *  * \n"
"P7+FU+FU+KI *  *  * +FU * +FU\n"
"P8 * +OU+KI *  *  *  *  *  * \n"
"P9+KY+KE *  *  *  *  * +KE+KY\n"
"P+00FU\n"
"P+00FU\n"
"P+00FU\n"
"P+00KE\n"
"P+00GI\n"
"P+00GI\n"
"P-00KA\n"
"P-00KA\n"
"+\n").getInitialState());
  CPPUNIT_ASSERT(hState.getHash() != hState2.getHash());
}


static void testDiffFile(const std::string& fileName){
  Record rec=CsaFile(fileName).getRecord();
  HashEffectState hState(rec.getInitialState());
  vector<osl::Move> moves=rec.getMoves();

  DoUndoMoveLockTest<HashEffectState> lockTest(hState, moves);
  lockTest.run();

  for(unsigned int i=0;i<moves.size();i++)
  {
    const Move move=moves[i];

    // isConsistent ǽʬʵ뤬 do/undo Ƥhash ѤʤȤ򶯤ǧƤߤ
    const HashKey before = hState.getHash();
    CPPUNIT_ASSERT_EQUAL(before, hState.getHash());
    
    ApplyMoveOfTurn::doMove(hState, move);
    CPPUNIT_ASSERT(hState.isConsistent());
    CPPUNIT_ASSERT_EQUAL(hState.getHash(), before.newHashWithMove(move));
  }
}


void SimpleHashStateTest::testDiff(){
  std::ifstream ifs(OslConfig::testCsaFile("FILES"));
  CPPUNIT_ASSERT(ifs);
  int i=0;
  std::string fileName;
  while((ifs >> fileName) && ++i<100){
    if(fileName == "") break;
    testDiffFile(OslConfig::testCsaFile(fileName));
  }
}

/** ʬζ̤򤤤Ƥhash ͤʤΤƥ */
static void testHashUniqueness(const std::string& fileName){
  typedef std::map<HashKey, SimpleState*> map_t;
  map_t stateMap;
  
  if (!isShortTest)
    std::cerr << fileName << "\n";
  
  Record rec=CsaFile(fileName).getRecord();
  HashEffectState hState(rec.getInitialState());
  vector<osl::Move> moves=rec.getMoves();
  for(unsigned int i=0;i<moves.size();i++){
    Move move=moves[i];
    ApplyMoveOfTurn::doMove(hState, move);
    CPPUNIT_ASSERT(hState.isConsistent());

    const HashKey& key = hState.getHash();
    map_t::const_iterator pos = stateMap.find(key);
    if ((pos != stateMap.end()) && (! (*pos->second == hState)))
    {
      std::cerr << "collsion detected " << key << "\n"
		<< hState
		<< pos->second;
    }
    else
    {
	// memory leak ɡ100MBԤʤȤꤢ
	stateMap[key] = new SimpleState(hState);
    }
  }
}

void SimpleHashStateTest::testHashUniqueness(){
  std::ifstream ifs(OslConfig::testCsaFile("FILES"));
  CPPUNIT_ASSERT(ifs);
  int i=0;
  std::string fileName;
  while((ifs >> fileName) && ++i<100){
    if(fileName == "") 
      break;
    ::testHashUniqueness(OslConfig::testCsaFile(fileName));
  }
}

void SimpleHashStateTest::testDoUndoMove(){
  extern bool isShortTest;
  std::ifstream ifs(OslConfig::testCsaFile("FILES"));
  CPPUNIT_ASSERT(ifs);
  int i=0;
  int count=100;
  if (isShortTest)
    count=10;
  std::string fileName;
  while((ifs >> fileName) && (++i<count)) {
    if(fileName == "") 
      break;
    if (! isShortTest)
      std::cerr << fileName << " ";

    CsaFile file(OslConfig::testCsaFile(fileName));
    Record rec=file.getRecord();
    NumEffectState state(rec.getInitialState());
    HashEffectState hState(state);

    vector<osl::Move> moves=rec.getMoves();
  
    size_t depth=0;
    typedef HashEffectState state_t;
    if (moves.begin()->player()==BLACK){
      RecursiveDoUndoMove<BLACK,state_t>
	func(hState, &*moves.begin(), &*moves.end(), &depth);
      ApplyMove<BLACK>::doUndoMove(hState, *moves.begin(), func);
    }
    else{
      RecursiveDoUndoMove<WHITE,state_t>
	func(hState, &*moves.begin(), &*moves.end(), &depth);
      ApplyMove<WHITE>::doUndoMove(hState, *moves.begin(), func);
    }
    CPPUNIT_ASSERT_EQUAL(moves.size(), depth);
  }
}

void SimpleHashStateTest::testMovePass(){
  HashEffectState state((SimpleState(HIRATE)));

  const Player turn = state.getTurn();
  const HashKey after_pass = state.getHash().newHashWithMove(Move::PASS(turn));
  state.changeTurn();
  const HashKey change_turn = state.getHash();
  CPPUNIT_ASSERT_EQUAL(after_pass, change_turn);
}

// ;;; Local Variables:
// ;;; mode:c++
// ;;; c-basic-offset:2
// ;;; End:
